// Process-Price-Promotion-Data/Scripts/cpp-Source/test-v2.cpp
// Links the CPP algorithm to R using the new RCpp interface (i.e. using Rcpp::Export)

// Author: Günter J. Hitsch, 2014
// Last Edited: Jihong Song, Sept. 2016

// This is the cleaned up version for test.cpp
// It links the Base Price Detection Algorithm in Product.cpp to the R Script for Processtion
// This script removes all functionalities for imputation.
// All imputations will be done in R using rolling join of data.table

// CAUTION:
// As imputations will be done in R, this script is only responsible for detecting base prices
// Since the UPC's in the new build data are stored as bit64 integers, importing it into C++ will cause problems
// Therefore, this script only returns a 1-column data.frame containing all detected base prices
// This 1-column data.frame will be added to the raw movement data using "cbind" in R (see the relevant R code)


#define ARMA_64BIT_WORD
#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]


#include <iostream>
#include <fstream>
#include <vector>
#include "Product.h"
//#include "Config.h"
#include <time.h>
#include <cmath>
#include <numeric>

using namespace std;
using namespace Rcpp;
using namespace arma;



// [[Rcpp::export]]
DataFrame BasePriceDetectionHybridAlgorithm(DataFrame move, DataFrame move_lookup_table){
   // cout<<"Initiated"<<endl;
   clock_t init, load, base, final;
   init=clock();

   // Load columns of "move" as vectors
   vector<double> dprice = as< vector<double> > (move["price"]);
   vector<double> dindex_first      = as< vector<double> > (move_lookup_table["index_begin"]);
   vector<double> dindex_last       = as< vector<double> > (move_lookup_table["index_end"]);

   // cout<<"\nLinked to C++ binary...\n" << endl;
   // cout<<"Processing UPC indices: "<<dindex_first.front()<<"-"<<dindex_last.back()<<endl;

   // Create Empty Vectors to stack things together
      vector<double> pbpsH   ;


   base = clock();
   const int N = dindex_first.size();
   for (int k = 0; k < N; k++){
      long ind_fst = dindex_first[k];
      long ind_lst = dindex_last[k];

      vector<double> prices(dprice.begin()+ind_fst-1, dprice.begin()+ind_lst);
      
      // ----------------------------------------------------------------
      // Processing Core Unit
      // ----------------------------------------------------------------
      Product p(prices);
      p.BasePriceDetectionHybridAlgorithm();
      // Important
      // Assign base prices as not-promoted prices if base prices were defined
      for (int i=0; i<p.bps_Snapshot.size(); i++){
         if(p.bps_Snapshot[i]>0) {
            p.bps[i] = p.bps_Snapshot[i];
         }
      }
      // Core Unit Processing Finished
      // ---------------------------------------------------------------

      // Insert thing to output vectors
      pbpsH.insert(pbpsH.end(), p.bps.begin(), p.bps.end());
   }

   // Output: turn 1 output vector into "vec" format
   vec out_pbpsH(pbpsH.data(), pbpsH.size(), false);

   //final=clock()-init;

   // Put everything in a data.frame
   DataFrame move_output = DataFrame::create(
      _["base_price"] = out_pbpsH);

   return(move_output);
}


	











