# ------------------------------------------------------------------------------
# Advertising-Generalizability/1-Build-Ad-Intel/Local-Channel-Information.R
#
# ------------------------------------------------------------------------------
# This script extracts information about local TV channels ("Distributors")
# For each Ad-Intel month, it builds a table with the following fields:
# 		DistributorID, DistributorCode, MarketCode, Affiliation
#		Note that "DistributorID" is needed for matching with the impressions
# ------------------------------------------------------------------------------

rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)

source_dir <- "/kilts/adintel"
output_dir <- "/project/ghitsch/data/Ad-Intel-2020/Dynamic-References/Local-Distributors/"

Find.Local.Distributors <- function(year, month){
  cat(year, month, "\n")

  # Find the information for DistributorID and DistributorCode
  dir <- paste0(source_dir,  "/", year, "/Occurrences")

  ym_start <- as.Date(paste0(year,"-",month, "-01"))
  
  if(month != 12){
    ym_end <- as.Date(paste0(year,"-",month+1, "-01")) - 1
  } else{
    ym_end <- as.Date(paste0(year,"-12-31"))
  }
  
  ym_start <- as.Date(cut(ym_start, "week"))
  
  if(wday(ym_end) != 1){
    ym_end <- as.Date(cut(ym_end, "week")) - 1
  } 
  
  occ <- fread(paste0(dir, '/SpotTV.tsv'), showProgress=F)
  
  occ[,`:=`(AdDate=as.Date(AdDate),
            DistributorCode = as.integer(DistributorCode))]
  
  occ <- occ[AdDate >= ym_start & AdDate <= ym_end]
  
  occ <- unique(occ[, .(DistributorID, DistributorCode)])
  
  # Load Official Distributor Data
  dir <- paste0(source_dir,  "/", year, "/References")
  dis <- fread(paste0(dir, "/Distributor.tsv"), showProgress=F)
  dis <- unique(dis[MediaTypeId == 5, .(MarketCode, DistributorCode = as.integer(DistributorCode), Affiliation)])
  dis <- merge(occ, dis, by="DistributorCode")
  
  # Write
  outfile <- paste0(output_dir, year, "-", month, ".csv")
  fwrite(dis, file=outfile)
  
  rm(dis)
  return(0)
}

ys <- rep(2010:2018, each=12)
ms <- rep(1:12, 9)

cl <- makeCluster(2, type="FORK", outfile="")
registerDoParallel(cl)

proc.list <- foreach(i=1:108) %dopar% {
  x <- Find.Local.Distributors(ys[i], ms[i])
  x
}

stopCluster(cl)

print("Done!")