rm(list=ls())
library(data.table)
library(bit64)
library(lubridate)
library(foreach)
library(doParallel)

station_info_dir <- "/project/ghitsch/data/Ad-Intel-2020/Dynamic-References/Local-Distributors/"
clearance_dir <- "/project/ghitsch/data/Ad-Intel-2020/Occurrence-Level/Local-Clearance/"

load.station <- function(year, month){
  cat(year, month, "\n")
  
  info <- fread(paste0(station_info_dir, year, "-", month, ".csv"))
  info[, `:=`(year=year, month=month)]
  return(info)
}
  
# Load Local data and find days
load.dates <- function(year, month){
  cat(year, month, "\n")
  clear <- readRDS(paste0(clearance_dir, year, "-", month, ".rds"))
  dates <- unique(clear$AdDate)
  output <- range(dates)
  output <- data.table(year=year, month=month, start=output[1], end=output[2])
  return(output)
}


years <- rep(2010:2018, each=12)
months <- rep(1:12, 9)

cl <- makeCluster(2, type="FORK", outfile="")

stations <- rbindlist(parLapply(cl, 1:108, function(x) load.station(years[x], months[x])))
dates <- rbindlist(parLapply(cl, 1:108, function(x) load.dates(years[x], months[x])))

stopCluster(cl)

output <- merge(stations, dates, by=c("year", "month"))
setnames(output, c("start", "end"), c("month_start", "month_end"))
fwrite(output, file="/project/ghitsch/data/Ad-Intel-2020/Dynamic-References/Local-Distributors-Stacked.csv")
