library(data.table)
library(bit64)
library(lubridate)
library(foreach)
library(doParallel)

clearance_dir <- "/project/ghitsch/data/Ad-Intel-2020/Occurrence-Level/Local-Clearance/"
national_dir <- "/project/ghitsch/data/Ad-Intel-2020/Occurrence-Level/National-TV/"
spot_dir <- "/project/ghitsch/data/Ad-Intel-2020/Occurrence-Level/Spot-TV/"

Extract <- function(year, month){
  cat(year, month, "\n")
  nat <- readRDS(paste0(national_dir, year, "-", month, ".rds"))
  nat <- nat[MediaTypeID %in% c(1,4), .(AdDate, AdTime, DistributorCode, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  
  clear <- readRDS(paste0(clearance_dir, year, "-", month, ".rds"))
  synd <- clear[MediaTypeID==14, .(AdDate, AdTime, MarketCode, DistributorCode, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  clear <- clear[MediaTypeID==13, .(AdDate, AdTime, MarketCode, DistributorCode, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  
  spot <- readRDS(paste0(spot_dir, year, "-", month, ".rds"))
  spot <- spot[, .(AdDate, AdTime, MarketCode, DistributorCode, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  
  out <- list(nat=nat, clear=clear, spot=spot, synd=synd)
  return(out)
}

years <- rep(2010:2018, each=12)
months <- rep(1:12, 9)	

cl <- makeCluster(2, type="FORK", outfile="")
proc.list <- parLapply(cl, 1:108, function(x) Extract(years[x], months[x]))
stopCluster(cl)

nat <- rbindlist(lapply(proc.list, function(x) x$nat))
clear <- rbindlist(lapply(proc.list, function(x) x$clear))
spot <- rbindlist(lapply(proc.list, function(x) x$spot)) 
synd <- rbindlist(lapply(proc.list, function(x) x$synd)) 

# Write Files
output_dir <- "/project/ghitsch/data/Ad-Intel-2020/Data-By-Station/"

dis_nat <- sort(unique(nat$DistributorCode))
for(dis in dis_nat){
  cat("Saving", dis, "\n")
  out <- nat[DistributorCode==dis]
  saveRDS(out, file=paste0(output_dir, "National/", dis, ".rds"))
}

dis_clear <- sort(unique(clear$DistributorCode))
for(dis in dis_clear){
  cat("Saving", dis, "\n")
  out <- clear[DistributorCode==dis]
  saveRDS(out, file=paste0(output_dir, "Clearance/", dis, ".rds"))
}

dis_spot <- sort(unique(spot$DistributorCode))
for(dis in dis_spot){
  cat("Saving", dis, "\n")
  out <- spot[DistributorCode==dis]
  saveRDS(out, file=paste0(output_dir, "Spot/", dis, ".rds"))
}

dis_synd <- sort(unique(synd$DistributorCode))
for(dis in dis_synd){
  cat("Saving", dis, "\n")
  out <- synd[DistributorCode==dis]
  saveRDS(out, file=paste0(output_dir, "Syndicated/", dis, ".rds"))
}