rm(list=ls())
library(data.table)
library(bit64)
library(lubridate)
library(foreach)
library(doParallel)

source_dir <- "/kilts/adintel"

stations <- fread("/project/ghitsch/data/Ad-Intel-2020/Dynamic-References/Local-Distributors-Stacked.csv")
markets <- fread("../../Auxiliary-Data/DMA_and_recording_method.csv")

has_clearance <- list.files("/project/ghitsch/data/Ad-Intel-2020/Data-By-Station/Clearance/")
has_clearance <- as.integer(gsub(".rds", "", has_clearance, fixed=T))

# Remove Alaska and Hawaii and non-clearance stations
# We don't fill stuff if a station never shows up in the clearance data
markets_to_remove <- markets[!(time_zone %in% c("ETZ", "CTZ", "PTZ", "MTZ")), MarketCode]
stations <- stations[!(MarketCode %in% markets_to_remove)]
stations <- stations[DistributorCode %in% has_clearance]

stations[, N_Affiliation := uniqueN(Affiliation), DistributorCode]

# Good stations (not changing affiliation)
good_stations <- stations[N_Affiliation==1, .(start_date=min(month_start), end_date=max(month_end)), .(DistributorCode, DistributorID, MarketCode, Affiliation)]
good_stations <- merge(good_stations, markets[, .(MarketCode, time_zone)], by="MarketCode")
good_stations[time_zone=="ETZ", time_zone := "US/Eastern"]
good_stations[time_zone=="CTZ", time_zone := "US/Central"]
good_stations[time_zone=="MTZ", time_zone := "US/Mountain"]
good_stations[time_zone=="PTZ", time_zone := "US/Pacific"]

# Bad stations (changing affiliation)
# We need to break this up by spell
bad_stations <- stations[N_Affiliation>1]
setorder(bad_stations, DistributorCode, year, month)
bad_stations[, group := .GRP, .(DistributorCode, Affiliation)]
bad_stations <- bad_stations[, .(start_date=min(month_start), end_date=max(month_end)), .(DistributorCode, MarketCode, DistributorID, Affiliation, group)]
bad_stations <- merge(bad_stations, markets[, .(MarketCode, time_zone)], by="MarketCode")
bad_stations[time_zone=="ETZ", time_zone := "US/Eastern"]
bad_stations[time_zone=="CTZ", time_zone := "US/Central"]
bad_stations[time_zone=="MTZ", time_zone := "US/Mountain"]
bad_stations[time_zone=="PTZ", time_zone := "US/Pacific"]

# Just Concat
good_stations[, group := nrow(bad_stations) + (1:.N)]
stations <- rbind(good_stations, bad_stations, fill=T)

# Fetch network abbreviations
Read.Distributor.Info <- function(year){
  dir <- paste0(source_dir, "/",year, "/References")
  
  dis <- fread(paste0(dir, "/Distributor.tsv"), showProgress=F)
  dis <- dis[MediaTypeId %in% c(1,4), 
             .(DistributorCode, Abbreviation)]
  return(dis)
}

networks <- unique(rbindlist(mapply(Read.Distributor.Info, 2010:2018, SIMPLIFY=F), fill=T))
networks[Abbreviation=="COZI", Abbreviation := "COZ"]
setnames(networks, c("DistributorCode", "Abbreviation"), c("Network", "Affiliation"))

# There are minor glitches:
## Network "COZI" is (presumably) coded as "COZ" in local station info files
## Network "MT3" has records from 2016.7 to 2016.12; it doesn't correspond to any local stations
## Network "ESC" has records from 2010.1 to 2010.7; it doesn't correspond to any local stations
## Local station 5934 (ID 4019, Market 131) are affiliated to network "MNT" which doesn't exist in the data
## Local stations with affiliation "IND" are independent stations without network affiliation

stations <- merge(stations, networks, by="Affiliation") 
fwrite(stations, file="/project/ghitsch/data/Ad-Intel-2020/Auxiliary-Data/Stations-For-Missing-Network-Imputation.csv")







