rm(list=ls())
library(data.table)
library(bit64)
library(lubridate)
library(foreach)
library(doParallel)
library(chron)

# Transfer from monthly level data to station level data
station_level_dir <- "/project/ghitsch/data/Ad-Intel-2020/Data-By-Station/Missing-Network-And-Ghost-Clearance/"
output_dir <- "/project/ghitsch/data/Ad-Intel-2020/Occurrence-Level/Missing-Network-Occurrences/"

# Months
months <- fread('/project/ghitsch/data/Ad-Intel-2020/Dynamic-References/Local-Distributors-Stacked.csv')
months <- unique(months[, .(year, month, month_start, month_end)])

# Stack All Data
files <- list.files(station_level_dir)

# Load Time Intervals
aux_data_dir <- "/project/ghitsch/data/Ad-Intel-2020/Auxiliary-Data/"
time_intervals <- fread(paste0(aux_data_dir, 'Time_Interval_spot.csv'))
time_intervals <- time_intervals[, .(time_interval_number, start_time=chron::times(start_time),
                                     end_time=chron::times(end_time))]
time_intervals[, start_min := hours(start_time)*60+minutes(start_time)]
time_intervals[, end_min := hours(end_time)*60+minutes(end_time)]

process_1 <- function(file){
  print(file)
  load(paste0(station_level_dir, file))
  time_zone <- missing_network$time_zone[1]
  missing_network <- missing_network[, .(DistributorID, time, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  ghost_clearance <- ghost_clearance[, .(DistributorID, time, PrimBrandCode, ScndBrandCode, TerBrandCode, Duration)]
  if(time_zone == "US/Pacific"){
    missing_network_1 <- copy(missing_network)
    missing_network_1[, delay_weight := 1/2]
    missing_network_1[, time := time + 10800]
    missing_network[, delay_weight := 1/2]
    missing_network <- rbind(missing_network, missing_network_1, fill=TRUE)
  } else if(time_zone == "US/Mountain"){
    missing_network_1 <- copy(missing_network)
    missing_network_1[, delay_weight := 1/2]
    missing_network_1[, time := time + 3600]
    missing_network[, delay_weight := 1/2]
    missing_network <- rbind(missing_network, missing_network_1, fill=TRUE)
  } else{
    missing_network[, delay_weight := 1]
  }
  ghost_clearance[, delay_weight := 1]
  missing_network[, type := "Missing Network"]
  ghost_clearance[, type := "Ghost Clearance"]
  stacked <- rbind(missing_network, ghost_clearance, fill=TRUE)
  
  # Get Date and Time
  stacked[, AdDate := date(time)]
  stacked[, Hour := hour(time)]
  stacked[, Minute := minute(time)]
  stacked[, Second := second(time)]
  stacked[, AdTime := paste(Hour, Minute, Second, sep=":")]
  stacked[, start_min := Hour*60+Minute]
  stacked[, AdTime := chron::times(AdTime)]
  stacked[, c("time", "Hour", "Minute", "Second") := NULL]
  
  # Get Time Interval
  use_intervals <- time_intervals[, .(start_min, TimeIntervalNumber=time_interval_number)]
  setkey(stacked, start_min)
  setkey(use_intervals, start_min)
  output <- use_intervals[stacked, roll=TRUE, rollends=c(TRUE, TRUE)]
  output[, start_min := NULL]
  output[, DayOfWeek := wday(AdDate, week_start=7)]
  return(output)
}

process <- function(file){
  x <- NULL
  try(x <- process_1(file))
  return(x)
}

cl <- makeCluster(2, type="FORK", outfile="")
proc <- rbindlist(parLapply(cl, files, process), fill=TRUE)
stopCluster(cl)

setcolorder(proc, c("AdDate", "AdTime", "type", "delay_weight",
                    "PrimBrandCode", "ScndBrandCode", "TerBrandCode",
                    "Duration", "DistributorID", "DayOfWeek", "TimeIntervalNumber"))

# Save by nielsen month
months[, month_start := date(month_start)]
months[, month_end := date(month_end)]

for(i in 1:nrow(months)){
  s <- months[i, month_start]
  e <- months[i, month_end]
  y <- months[i, year]
  m <- months[i, month]
  cat("Saving", i, y, m, "\n")
  to_save <- proc[AdDate %between% c(s,e)]
  outfile <- paste0(output_dir, y, "-", m, ".rds")
  saveRDS(to_save, file=outfile)
}

# Save Files