rm(list=ls())
library(data.table)
library(bit64)

load('../Meta-Data/Products-Corrected.RData')
load('../Meta-Data/Meta-Data-Corrected.RData')

# Process Prod_Meta
products.use <- products[(department_code %in% 0:9) & 
                           (product_module_code > 1000) & 
                           (product_module_code < 9999), 
                         .(upc, upc_ver_uc_corrected, product_module_code, product_module_descr,
                           brand_code_uc=brand_code_uc_corrected, 
                           brand_descr=brand_descr_corrected,
                           multi, size1_amount, size1_units, upc_descr)]
meta_data.use <- meta_data[year >= 2010, .(revenue_RMS=sum(revenue_RMS)),
                           .(product_module_code, upc, upc_ver_uc_corrected)]
prod_meta.use <- merge(products.use, meta_data.use, by=c("upc", "upc_ver_uc_corrected", "product_module_code"), all.x=T)
prod_meta.use[is.na(revenue_RMS), revenue_RMS := 0]
prod_meta.use[size1_units == "LI", `:=`(size1_units="ML", size1_amount=size1_amount*1000)]
prod_meta.use[size1_units == "PO", `:=`(size1_units="OZ", size1_amount=size1_amount*16)]

# UPC Dict
upc_dict <- prod_meta.use[, .(product_module_code, brand_code_uc, upc, upc_ver_uc_corrected, 
                              multi, size1_amount, size1_units, revenue_RMS, upc_descr)]
upc_dict <- unique(upc_dict)
# saveRDS(upc_dict, file='../Generated-Files/UPC-Dictionary-180508.rds')

# Brand Dict
brand_dict_orig <- prod_meta.use[, .(revenue_RMS=sum(revenue_RMS)), .(product_module_code, brand_code_uc, product_module_descr, brand_descr)]
brand_dict_orig <- unique(brand_dict_orig)
# saveRDS(brand_dict_orig, file="../Generated-Files/Original-Brand-Dictionary-180508.rds")

# Load list of brands
# brand_dict_orig <- readRDS('../Generated-Files/Original-Brand-Dictionary-180508.rds')

# Load module aggregation lists (manually created)
module_agg <- fread('../Information-Files/RMS-Module-Aggregation.csv')

# Load All Flavor Aggregations
flavor_agg <- rbindlist(lapply(list.files('../Flavor-Aggregation-Files/', full.names=T), fread), fill=T)
flavor_agg <- flavor_agg[, c(4,2,11,9), with=F]
names(flavor_agg) <- c("tmp_module_code", "tmp_brand_code_uc", "product_module_code", "brand_code_uc")

# Load designated module codes and brand names for the flavor aggregated brands
flavor_info <- fread('../Information-Files/RMS-Flavor-Aggregation.csv')
flavor_info <- flavor_info[, .(tmp_module_code=product_module_code,
                           tmp_brand_code_uc=brand_code_uc,
                           new_module_code=new_product_module_code, 
                           new_brand_code_uc, new_brand_descr)]

# Merge
flavor_agg <- merge(flavor_agg, flavor_info, by=c("tmp_module_code", "tmp_brand_code_uc"))
flavor_agg[, c('tmp_module_code', 'tmp_brand_code_uc') := NULL]
flavor_agg[new_module_code==30001, product_module_code := 30001]
names(flavor_info) <- names(flavor_agg)
flavor_agg <- rbind(flavor_info, flavor_agg, fill=T)
flavor_agg <- unique(flavor_agg)

# 1. Implement Module Aggregations
working <- copy(brand_dict_orig)
working <- merge(working, module_agg[,-2,with=F], by=c('product_module_code'), all.x=T)
working[is.na(new_module_code), `:=`(new_module_code=product_module_code,
                                     new_module_descr=product_module_descr)]

# 2: Flavor Aggregation and Adjustments
names(flavor_agg)[1] <- c('new_module_code')
names(flavor_agg)[3] <- c('renew_module_code')
working <- merge(working, flavor_agg, by=c("new_module_code", "brand_code_uc"), all.x=T)
working[!is.na(renew_module_code), new_module_code := renew_module_code]
working[is.na(new_brand_descr), `:=`(new_brand_code_uc=brand_code_uc, new_brand_descr=brand_descr)]
working[, renew_module_code := NULL]

# Write new brand dictionary
brand_dict_new <- working[, .(new_module_code, new_brand_code_uc, new_module_descr, new_brand_descr,
                              product_module_code, brand_code_uc, product_module_descr, brand_descr, revenue_RMS)]
brand_dict_new <- unique(brand_dict_new)
setkey(brand_dict_new, new_module_code, new_brand_code_uc)
brand_dict_new[new_module_code==30002, new_module_code := product_module_code]

# 3: Find Potentially Workable Cases
normal_cases <- fread('../Information-Files/Normally-Matched-Brands.csv')
normal_cases <- unique(normal_cases[, .(new_module_code=product_module_code, 
                                        new_brand_code_uc=brand_code_uc,
                                        new_brand_descr=brand_descr)])
agg_cases <- unique(flavor_agg[, .(new_module_code=renew_module_code, 
                                   new_brand_code_uc, new_brand_descr)])
usable_cases <- rbind(normal_cases, agg_cases, fill=T)

# 4. Find Competitors
competitors <- brand_dict_new[new_module_code %in% usable_cases$new_module_code, 
                              .(revenue_RMS=sum(revenue_RMS)), 
                              .(new_module_code, new_brand_code_uc, new_brand_descr)]
competitors[, in_module_rank := rank(-revenue_RMS, ties.method="min"), new_module_code]
competitors <- competitors[in_module_rank <= 8]

## Module Revenues
module_revenues <- brand_dict_new[, .(revenue_total=sum(revenue_RMS)), new_module_code]

# 5. Create UPC Files

## All cases = 358 matched cases + their potential competitors
all_brands <- rbind(usable_cases, competitors[, -c("revenue_RMS", "in_module_rank")], fill=T)
all_brands <- unique(all_brands)

## Load UPC Files
# upc_dict <- readRDS('../Generated-Files/UPC-Dictionary-180508.rds')
upc_master <- vector("list", nrow(all_brands))
for(i in 1:nrow(all_brands)){
  print(i)
  entry <- all_brands[i]
  brands <- brand_dict_new[new_module_code==entry$new_module_code & new_brand_code_uc==entry$new_brand_code_uc]
  brands <- unique(brands[, .(product_module_code, brand_code_uc, brand_descr)])
  upcs <- merge(upc_dict[product_module_code %in% (brands$product_module_code)], brands, by=c("product_module_code", "brand_code_uc"))
  upcs <- upcs[, .(product_module_code, upc, upc_ver_uc_corrected, multi, size1_amount, size1_units, revenue_RMS, brand_code_uc, brand_descr, upc_descr)]
  upcs <- unique(upcs)
  units <- upcs[, .(rev=sum(revenue_RMS)), size1_units]
  units[, majority_units_revshare := rev/sum(rev)]
  units[, majority_units := size1_units[which.max(majority_units_revshare)]]
  units[, majority_units_revshare := majority_units_revshare[which.max(majority_units_revshare)]]
  units <- unique(units[, .(majority_units_revshare, majority_units)])
  upcs <- cbind(upcs, units)
  upcs[, `:=`(new_module_code=entry$new_module_code,
              new_brand_code_uc=entry$new_brand_code_uc,
              new_brand_descr=entry$new_brand_descr)]
  upc_master[[i]] <- upcs
}
upc_master <- rbindlist(upc_master, fill=TRUE)

# Unit Failure
units <- unique(upc_master[, .(new_module_code, new_brand_code_uc, new_brand_descr, majority_units_revshare)])
unit_failures <- units[majority_units_revshare < 0.95]
fwrite(unit_failures, file="../Information-Files/Unit-Failures.csv")

# Create Consolidated UPC Files
upc_master <- upc_master[majority_units_revshare >= 0.95 & size1_units==majority_units, 
                   .(new_module_code, new_brand_code_uc, new_brand_descr, 
                     product_module_code, upc, upc_ver_uc_corrected,
                     multi, size1_amount, size1_units, revenue_RMS)]

# 6. Find final competitors and save
brands_master <- upc_master[, .(revenue_RMS=sum(revenue_RMS)), .(new_module_code, new_brand_code_uc, new_brand_descr)]
brands_master <- merge(brands_master,
                       usable_cases[, .(new_module_code, new_brand_code_uc, main=1)],
                       by=c("new_module_code", "new_brand_code_uc"), all.x=T)
brands_master[is.na(main), main := 0]

## Find top 4 brands of each module
use_modules <- brands_master[main==1, unique(new_module_code)]
brands_master <- brands_master[new_module_code %in% use_modules]
brands_master[, in_module_rank := rank(-revenue_RMS, ties.method="min"), new_module_code]
top4 <- brands_master[in_module_rank <= 4, -c("main", "in_module_rank"), with=F]
core <- brands_master[main==1, -c("main", "in_module_rank"), with=F]
core <- merge(core, top4, by="new_module_code", allow.cartesian=T)
core <- core[new_brand_code_uc.x != new_brand_code_uc.y]
core[, rank := rank(-revenue_RMS.y, ties.method="min"), .(new_module_code, new_brand_code_uc.x)]
core <- dcast(core[rank<=3], new_module_code+new_brand_code_uc.x+new_brand_descr.x+revenue_RMS.x~rank,
              value.var=c("new_brand_code_uc.y", "new_brand_descr.y", "revenue_RMS.y"))

# Designate top ad competitor
for_ad <- core[, .(new_module_code, new_brand_code_uc.x, new_brand_descr.x,
                   adcomp_descr=new_brand_descr.y_1,
                   new_brand_descr.y_2, new_brand_descr.y_3,
                   adcomp_code=new_brand_code_uc.y_1, 
                   new_brand_code_uc.y_2, new_brand_code_uc.y_3)]
setkey(for_ad, new_module_code, new_brand_code_uc.x)

change_to_2_modules <- c(10007, 1173, 1323, 1466, 1532, 
                         1553, 7012, 7012, 7260, 7734, 8398, 8404, 8619, 3555)
change_to_2_brands <- c(638002, 582939, 586556, 643938, 667466, 
                        531433, 653799, 653805, 526997, 518919, 548662, 535984, 535985, 660639)
change_to_3_modules <- c(1055, 7825, 8404)
change_to_3_brands <- c(659806, 648171, 536024)

core[, `:=`(adcomp_code=new_brand_code_uc.y_1, adcomp_descr=new_brand_descr.y_1)]

for(i in 1:length(change_to_2_modules)){
  m <- as.character(change_to_2_modules[i])
  b <- as.character(change_to_2_brands[i])
  core[new_module_code==m & new_brand_code_uc.x==b, 
       `:=`(adcomp_code=new_brand_code_uc.y_2, adcomp_descr=new_brand_descr.y_2)]
}

for(i in 1:length(change_to_3_modules)){
  m <- as.character(change_to_3_modules[i])
  b <- as.character(change_to_3_brands[i])
  core[new_module_code==m & new_brand_code_uc.x==b, 
       `:=`(adcomp_code=new_brand_code_uc.y_3, adcomp_descr=new_brand_descr.y_3)]
}

## Merge in module revenue
module_revenues[, new_module_code := as.integer(new_module_code)]
core[, new_module_code := as.integer(new_module_code)]
core <- merge(core, module_revenues, by="new_module_code")
core[, `:=`(share0 = revenue_RMS.x/revenue_total,
            share1 = (revenue_RMS.x+revenue_RMS.y_1)/revenue_total,
            share2 = (revenue_RMS.x+revenue_RMS.y_1+revenue_RMS.y_2)/revenue_total,
            share3 = (revenue_RMS.x+revenue_RMS.y_1+revenue_RMS.y_2+revenue_RMS.y_3)/revenue_total)]
core[, c("revenue_total", "revenue_RMS.y_1", "revenue_RMS.y_2", "revenue_RMS.y_3") := NULL]
core[share0 > 0.8, `:=`(new_brand_code_uc.y_1=NA, new_brand_code_uc.y_2=NA, new_brand_code_uc.y_3=NA,
                        new_brand_descr.y_1=NA, new_brand_descr.y_2=NA, new_brand_descr.y_3=NA)]
core[share0 <= 0.8 & share1 > 0.8, `:=`(new_brand_code_uc.y_2=NA, new_brand_code_uc.y_3=NA,
                                        new_brand_descr.y_2=NA, new_brand_descr.y_3=NA)]
core[share1 <= 0.8 & share2 > 0.8, `:=`(new_brand_code_uc.y_3=NA, new_brand_descr.y_3=NA)]

# Get Cleaned Core
universe <- core[, .(new_module_code, new_brand_code_uc=new_brand_code_uc.x,
                     new_brand_descr=new_brand_descr.x,
                     adcomp_code, adcomp_descr,
                     comp1_code=new_brand_code_uc.y_1, comp1_descr=new_brand_descr.y_1,
                     comp2_code=new_brand_code_uc.y_2, comp2_descr=new_brand_descr.y_2,
                     comp3_code=new_brand_code_uc.y_3, comp3_descr=new_brand_descr.y_3,
                     revenue_RMS=revenue_RMS.x, share0=round(share0,3), 
                     share1=round(share1,3), share2=round(share2,3), share3=round(share3,3))]

# Limit UPC Master to universe modules
upc_master <- upc_master[new_module_code %in% universe$new_module_code]

# Brands to match
main_match <- universe[, .(new_module_code, new_brand_code_uc, new_brand_descr)]
adcomp_match <- universe[, .(new_module_code, adcomp_code, adcomp_descr)]
adcomp_match <- merge(adcomp_match, main_match, by.x=c("new_module_code", "adcomp_code"),
                      by.y=c("new_module_code", "new_brand_code_uc"), all.x=T)
adcomp_match <- adcomp_match[is.na(new_brand_descr) & !grepl("CTL BR", adcomp_descr), .(new_module_code, adcomp_code, adcomp_descr)]
adcomp_match <- unique(adcomp_match)

# Save Files
saveRDS(upc_master, file="../Exported-Files/Consolidated-UPC-Dictionary-For-Aggregation-180508.rds")
fwrite(universe, file="../Exported-Files/Universe-For-Estimation-180508.csv")
fwrite(main_match, file="../Information-Files/Brands-Needing-Main-Matching.csv")
fwrite(adcomp_match, file="../Information-Files/Brands-Needing-Comp-Matching.csv")
saveRDS(brand_dict_new, file="../Information-Files/Brand-Dictionary-After-Flavor-Aggregations.rds")

