rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)
library(chron)

national_dir <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Occurrence-Level/National-TV/"
clearance_dir <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Occurrence-Level/Local-Clearance/"
spot_dir <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Occurrence-Level/Spot-TV/"
missing_network_dir <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Occurrence-Level/Missing-Network-Full/"

aux_data_dir <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Auxillary-Data/"
markets <- fread(paste0(aux_data_dir, "DMA_and_recording_method.csv"))
markets <- markets[Market=="FDM" & time_zone %in% c("ETZ", "CTZ", "MTZ", "PTZ")]

station_info_path <- "/grpshares/hitsch_shapiro_ads/data/Ad-Intel-Build-2018/Dynamic-References/Local-Distributors-Stacked.csv"
stations <- fread(station_info_path)
setnames(stations, c("year", "month"), c("Year", "Month"))

home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"
main <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/All-Main-Matches-180508.csv"))
comp <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/All-Comp-Matches-180508.csv"))
matches <- unique(rbind(main, comp, fill=TRUE))
brands <- sort(unique(matches$BrandCode))

week_range <- date(c("2010-01-09", "2016-12-17"))

Extract.Weekly.Ads <- function(year, month){
	cat(year, month, "\n")

	# Station info
	station_info <- stations[Year==year & Month==month, .(DistributorCode, Affiliation)]

	# Spot
	spot <- readRDS(paste0(spot_dir, year, "-", month, ".rds"))
	spot <- spot[MarketCode %in% markets$MarketCode]
	spot <- spot[(PrimBrandCode %in% brands) | (ScndBrandCode %in% brands) | (TerBrandCode %in% brands)]
	spot[, `:=`(GRP=100*Impression/Universe, Impression=NULL, Universe=NULL)]
	spot[, `:=`(Duration=as.numeric(Duration), Spend=as.numeric(Spend))]
	spot[!is.na(TerBrandCode), `:=`(Duration=Duration/3, Spend=Spend/3, GRP=GRP/3)]
	spot[is.na(TerBrandCode) & !is.na(ScndBrandCode), `:=`(Duration=Duration/2, Spend=Spend/2, GRP=GRP/2)]
	spot_melt <- melt(spot, id.vars=c("AdDate", "AdTime", "TVDaypartCode", "MarketCode", "DistributorCode",
								      "Duration", "Spend", "GRP"), 
							measure.vars=c("PrimBrandCode", "ScndBrandCode", "TerBrandCode"),
							value.name="BrandCode")
	spot_melt <- spot_melt[BrandCode %in% brands]
	spot_melt[, dma_code := MarketCode + 400]
	spot_melt[, week_end := ceiling_date(AdDate, "week", week_start=7)-1]
	spot_melt <- merge(spot_melt, station_info, by="DistributorCode")
	spot_melt[, major_network := Affiliation %in% c("ABC", "NBC", "CBS", "FOX")]
	spot_agg <- spot_melt[, .(Duration=sum(Duration, na.rm=T), GRP=sum(GRP, na.rm=T), Class="Spot"),
						  .(BrandCode, week_end, dma_code, major_network)]

	# Clearance
	clear <- readRDS(paste0(clearance_dir, year, "-", month, ".rds"))
	clear <- clear[MarketCode %in% markets$MarketCode]
	clear <- clear[(PrimBrandCode %in% brands) | (ScndBrandCode %in% brands) | (TerBrandCode %in% brands)]
	clear[, `:=`(GRP=100*Impression/Universe, Impression=NULL, Universe=NULL)]
	clear[, `:=`(Duration=as.numeric(Duration), Spend=as.numeric(Spend))]
	clear[!is.na(TerBrandCode), `:=`(Duration=Duration/3, Spend=Spend/3, GRP=GRP/3)]
	clear[is.na(TerBrandCode) & !is.na(ScndBrandCode), `:=`(Duration=Duration/2, Spend=Spend/2, GRP=GRP/2)]
	clear_melt <- melt(clear, id.vars=c("AdDate", "AdTime", "MediaTypeID", "TVDaypartCode", "MarketCode", "DistributorCode",
								      "Duration", "Spend", "GRP"), 
							measure.vars=c("PrimBrandCode", "ScndBrandCode", "TerBrandCode"),
							value.name="BrandCode")
	clear_melt <- clear_melt[BrandCode %in% brands]
	clear_melt[, dma_code := MarketCode + 400]
	clear_melt[, week_end := ceiling_date(AdDate, "week", week_start=7)-1]
	clear_melt <- merge(clear_melt, station_info, by="DistributorCode")
	clear_melt[, major_network := Affiliation %in% c("ABC", "NBC", "CBS", "FOX")]
	clear_melt[MediaTypeID==13, Class := "Network"]
	clear_melt[MediaTypeID==14, Class := "Syndicated"]
	clear_agg <- clear_melt[, .(Duration=sum(Duration, na.rm=T), GRP=sum(GRP, na.rm=T)),
						  .(BrandCode, week_end, dma_code, Class, major_network)]

	# Missing Network / Ghost Clearance
	missing <- readRDS(paste0(missing_network_dir, year, "-", month, ".rds"))					  
	missing <- missing[MarketCode %in% markets$MarketCode]
	missing <- missing[(PrimBrandCode %in% brands) | (ScndBrandCode %in% brands) | (TerBrandCode %in% brands)]
	missing[, `:=`(GRP=100*Impression/Universe, Impression=NULL, Universe=NULL)]
	missing[, `:=`(Duration=as.numeric(Duration))]
	missing[!is.na(TerBrandCode), `:=`(Duration=Duration/3, GRP=GRP/3)]
	missing[is.na(TerBrandCode) & !is.na(ScndBrandCode), `:=`(Duration=Duration/2, GRP=GRP/2)]
	missing_melt <- melt(missing, id.vars=c("AdDate", "AdTime", "MarketCode", "DistributorCode",
									  "Affiliation", "type", "delay_weight",
								      "Duration", "GRP"), 
							measure.vars=c("PrimBrandCode", "ScndBrandCode", "TerBrandCode"),
							value.name="BrandCode")
	missing_melt <- missing_melt[BrandCode %in% brands]
	missing_melt[, dma_code := MarketCode + 400]
	missing_melt[, week_end := ceiling_date(AdDate, "week", week_start=7)-1]
	missing_melt[, major_network := Affiliation %in% c("ABC", "NBC", "CBS", "FOX")]
	missing_agg <- missing_melt[, .(Duration=sum(Duration*delay_weight, na.rm=T), GRP=sum(GRP*delay_weight, na.rm=T)),
						  .(BrandCode, week_end, dma_code, Class=type, major_network)]

	# Cable
	nat <- readRDS(paste0(national_dir, year, "-", month, ".rds"))
	nat <- nat[MediaTypeID %in% c(2,22)] # Cable
	nat <- nat[(PrimBrandCode %in% brands) | (ScndBrandCode %in% brands) | (TerBrandCode %in% brands)]
	nat[, `:=`(GRP=100*Impression/Universe, Impression=NULL, Universe=NULL)]
	nat[, `:=`(Duration=as.numeric(Duration), Spend=as.numeric(Spend))]
	nat[!is.na(TerBrandCode), `:=`(Duration=Duration/3, Spend=Spend/3, GRP=GRP/3)]
	nat[is.na(TerBrandCode) & !is.na(ScndBrandCode), `:=`(Duration=Duration/2, Spend=Spend/2, GRP=GRP/2)]
	nat_melt <- melt(nat, id.vars=c("AdDate", "AdTime", "MediaTypeID", "TVDaypartCode", "DistributorCode",
									"Duration", "Spend", "GRP"),
					 measure.vars=c("PrimBrandCode", "ScndBrandCode", "TerBrandCode"),
					 value.name="BrandCode")
	nat_melt <- nat_melt[BrandCode %in% brands]
	nat_melt[, week_end := ceiling_date(AdDate, "week", week_start=7)-1]
	nat_agg <- nat_melt[, .(Duration=sum(Duration, na.rm=T), GRP=sum(GRP, na.rm=T), Class="Cable"),
						  .(BrandCode, week_end)]
	nat_agg <- nat_agg[rep(1:.N, each=nrow(markets))]
	nat_agg[, dma_code := markets$MarketCode+400]
	
	# Combine
	out <- rbind(nat_agg, clear_agg, spot_agg, missing_agg, fill=TRUE)
	return(out)
}

years <- rep(2010:2016, each=12)
months <- rep(1:12, 7)	

cl <- makeCluster(42, type="FORK", outfile="/dev/stdout")
proc <- rbindlist(parLapply(cl, 1:84, function(x) Extract.Weekly.Ads(years[x], months[x])), fill=TRUE)
stopCluster(cl)

# Remove Bad Weeks
use <- proc[week_end %between% week_range]
use <- use[, .(Duration=sum(Duration), GRP=sum(GRP)), .(dma_code, week_end, BrandCode, Class, major_network)]

# Merge in matches
matches <- matches[, .(new_module_code, new_brand_code_uc, BrandCode, match_tier)]
merged <- merge(use, matches, by="BrandCode", allow.cartesian=TRUE)

# ZEROS ARE NOT FILLED
rms_brands <- unique(matches[, .(new_module_code, new_brand_code_uc)])
setkey(rms_brands, new_module_code, new_brand_code_uc)

output_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
for(i in 1:nrow(rms_brands)){
	m <- rms_brands[i, new_module_code]
	b <- rms_brands[i, new_brand_code_uc]
	cat("Saving", i, m, b, "\n")
	out <- merged[new_module_code==m & new_brand_code_uc==b]
	outfile <- paste0(output_dir, m, "-", b, ".rds")
	saveRDS(out, file=outfile)
}


