rm(list=ls())
library(data.table)
library(bit64)
library(parallel)
library(foreach)
library(doParallel)
setNumericRounding(0)

source_dir <- '/grpshares/ghitsch/data/RMS-Build-2016/RMS-Processed/Modules/' # Results from 0-Build-Nielsen-Data
output_dir <- '/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/Extract-From-All-Other/'
dir.create(output_dir, showWarnings=FALSE)

home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"
universe <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
upc_table <- readRDS(paste0(home_dir, "2-String-Matching/Exported-Files/Consolidated-UPC-Dictionary-For-Aggregation-180508.rds"))

upc_table <- upc_table[revenue_RMS > 0]

Process.Module <- function(module){
	info <- upc_table[product_module_code==module, .(upc, upc_ver_uc_corrected)]
	other_path <- paste0(source_dir, module, "/All-Other.RData")
	load(other_path)
	move <- merge(move, info, by=c("upc", "upc_ver_uc_corrected"))
	move <- move[processed==T & year(week_end) >= 2010]
	move <- move[, .(upc, upc_ver_uc_corrected, store_code_uc, week_end,
					 base_price, imputed_price, units, feature, display)]
	return(move)
}

module_list <- sort(unique(upc_table$product_module_code))
n <- length(module_list)

cl <- makeCluster(2, type="FORK", outfile="/dev/stdout")
registerDoParallel(cl)

status <- foreach(i=1:n) %dopar% {
	module <- module_list[i]
	outfile <- paste0(output_dir, module, ".rds")
	if(!file.exists(outfile)){
		cat(i, module, "\n")
		proc <- NULL
		try(proc <- Process.Module(module))
		if(!is.null(proc)){
			saveRDS(proc, outfile)
			ret <- 0
		} else{
			ret <- -1
		}
	} else{
		ret <- 1
	}
	ret
}

stopCluster(cl)








