# We will refrain from using zip codes, because borders are DEFINED using FIPS county codes
# Limited to FDM only

rm(list=ls())
library(data.table)
library(bit64)
library(lubridate)
library(stringr)
        
# ------------------------------ #
# 1. Process Borders
# ------------------------------ #
markets <- fread('../Auxiliary-Data/DMA_and_recording_method.csv')
borders <- fread('../Auxiliary-Data/borders.csv')
borders <- borders[, .(dma_descr1=market_name1, dma_descr2=gsub(market_name1, "", bordername, fixed=T)), .(fips, bordernum)]
borders[, dma_descr2 := str_trim(dma_descr2)]
marketnames <- markets[, .(dma_code, dma_descr=MarketName)]
borders[dma_descr1=="SANTABARBARA-SANMAR-SANLUOB", dma_descr1 := "SANTABARBRA-SANMAR-SANLUOB"]
borders[dma_descr2=="SANTABARBARA-SANMAR-SANLUOB", dma_descr2 := "SANTABARBRA-SANMAR-SANLUOB"]
borders[dma_descr1=="TAMPA-ST PETE (SARASOTA)", dma_descr1 := "TAMPA-ST. PETE (SARASOTA)"]
borders[dma_descr2=="TAMPA-ST PETE (SARASOTA)", dma_descr2 := "TAMPA-ST. PETE (SARASOTA)"]
borders[dma_descr1=="GREENVILLE-SPART-ASHEVLL-AND", dma_descr1 := "GREENVLL-SPART-ASHEVLL-AND"]
borders[dma_descr2=="GREENVILLE-SPART-ASHEVLL-AND", dma_descr2 := "GREENVLL-SPART-ASHEVLL-AND"]
borders <- merge(borders, marketnames, by.x="dma_descr1", by.y="dma_descr", all.x=T)
setnames(borders, "dma_code", "dma_code1")
borders <- merge(borders, marketnames, by.x="dma_descr2", by.y="dma_descr", all.x=T)
setnames(borders, "dma_code", "dma_code2")
borders <- borders[, .(fips, bordernum, dma_code1, dma_descr1, dma_code2, dma_descr2)]
setkey(borders, fips, bordernum)
fdm <- markets[Market=="FDM", dma_code]
borders <- borders[(dma_code1 %in% fdm) & (dma_code2 %in% fdm)]
borders[, border_id := as.integer(paste0(pmin(dma_code1, dma_code2), pmax(dma_code1, dma_code2)))]

# ------------------------------ #
# 2. Find FIPS-DMA Correspondences
#    Remove bad FIPS
# ------------------------------ #

load('./Data/Stores.RData') # From RMS data build
panelists <- readRDS("./Data/Panelists.rds") # From HMS data build + ./Grid-Extractt-Panelist.R

fips_dma.rms <- unique(stores[year %in% 2010:2014, .(fips=fips_state_code*1000+fips_county_code, dma_code.rms=dma_code)])
fips_dma.hms <- unique(panelists[panel_year %in% 2010:2014, .(fips=fips_state_code*1000+fips_county_code, dma_code.hms=dma_code)])
fips_dma <- merge(fips_dma.rms, fips_dma.hms, by="fips", all=T)
fips_dma <- fips_dma[!is.na(fips)]

# Inconsistent RMS/HMS:
inconsistent <- fips_dma[dma_code.rms != dma_code.hms]
consistent <- fips_dma[!(fips %in% inconsistent$fips)]

# Duplicates
consistent[, occ := .N, fips]
once <- consistent[occ==1]
duplicates <- consistent[occ>1]

# Clear-up
fips_dma <- once[, .(dma_code=max(dma_code.rms, dma_code.hms, na.rm=T)), fips]
bad_fips <- unique(c(inconsistent$fips, duplicates$fips))

# Remove contaminated borders
borders[, bad := fips %in% bad_fips]
borders[, bad := any(bad), border_id]
borders <- borders[bad==F]
borders[, bad := NULL]

# ------------------------------ #
# 3. Process (All) Stores
# ------------------------------ #
load("./Data/store_meta.RData") # From RMS data build

# (1) Stores that stays in a same fips
stores1 <- unique(stores[!is.na(dma_code) & (year %in% 2010:2014), 
                 .(store_code_uc, parent_code, fips=fips_state_code*1000+fips_county_code, dma_code)])
stores1[, occ := .N, store_code_uc]
stores1 <- stores1[occ==1, -c("occ")]
stores1 <- stores1[!(fips %in% bad_fips)]

# (2) Stores covered for 261 weeks in sample
stores2 <- RMS_store[!is.na(store_code_uc) & year >= 2010]
stores2 <- stores2[, .(revenue=sum(revenue), N_weeks=sum(N_weeks)), store_code_uc]
stores2 <- stores2[N_weeks==261, .(store_code_uc, total_revenue=revenue)]

# (3) Merge and cumshare
use.stores <- merge(stores1, stores2, by="store_code_uc")
use.stores <- use.stores[dma_code %in% fdm]
setorder(use.stores, -total_revenue)
use.stores[, cumshare := cumsum(total_revenue/sum(total_revenue))]
use.stores[, revenue_rank := 1:.N]

# ------------------------------ #
# 4. Process (Border) Stores
# ------------------------------ #
border.stores <- merge(use.stores, borders, by="fips", allow.cartesian=TRUE)
# Remove border if only covered one side
border.stores[, nside := uniqueN(dma_code), border_id]
border.stores <- border.stores[nside==2, -c("nside")]

# ------------------------------ #
# 5. Process (All) Households
# ------------------------------ #
hh <- panelists[, .(household_code, panel_year, projection_factor,
                    fips=1000*fips_state_code+fips_county_code, dma_code)]
hh <- hh[!(fips %in% bad_fips)]
hh <- hh[dma_code %in% fdm]
# Remove if one household code shifts fips or dma
test <- unique(hh[, .(household_code, fips, dma_code)])
test[, occ := .N, household_code]
nonunique.hh <- test[occ>1]
use.hh <- hh[household_code %in% test[occ==1, household_code]]

use.hh.projection <- use.hh[, .(household_code, panel_year, projection_factor)]
use.hh.location <- unique(use.hh[, .(household_code, fips, dma_code)])

# ------------------------------ #
# 6. Process (Border) Households
# ------------------------------ #
border.hh <- merge(use.hh.location, borders, by="fips", allow.cartesian=TRUE)
# Remove border if only covered one side
border.hh[, nside := uniqueN(dma_code), border_id]
border.hh <- border.hh[nside==2, -c("nside")]

# ------------------------------ #
# 7. CLEAN UP
# ------------------------------ #
fips_dma <- fips_dma[dma_code %in% fdm]

fips_borders <- borders[, .(fips, bordernum, border_id, dma_code1, dma_code2, dma_descr1, dma_descr2)]
fips_borders[, `:=`(hms_coverage=F, rms_coverage=F)]
fips_borders[border_id %in% border.stores$border_id, rms_coverage := T]
fips_borders[border_id %in% border.hh$border_id, hms_coverage := T]

use.stores[, fips := NULL]
border.stores <- border.stores[, .(store_code_uc, parent_code, dma_code, cumshare, total_revenue, revenue_rank, bordernum, border_id)]

use.hh <- use.hh[, .(household_code, panel_year, dma_code, projection_factor)]
border.hh <- border.hh[, .(household_code, dma_code, bordernum, border_id)]
border.hh <- merge(border.hh, use.hh[, -c("dma_code"), with=F], by="household_code", allow.cartesian=T)
border.hh <- border.hh[, .(household_code, dma_code, bordernum, border_id, panel_year, projection_factor)]

# Save: fips_dma, fips_borders, stores, hh
fwrite(fips_dma, file="./Processed-Data/FIPS-DMA-Table.csv")
fwrite(fips_borders, file="./Processed-Data/FIPS-Border-Table.csv")
save(use.hh, border.hh, file="./Processed-Data/Selected-Panelists.RData")
save(use.stores, border.stores, file="./Processed-Data/Selected-Stores.RData")

