rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)
library(lfe)

ad_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"
load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))

all_weeks <- seq(date("2010-01-09"), date("2014-12-27"), 7)
all_dmas <- unique(use.stores$dma_code)

Process.R2 <- function(m, b){
	cat(m, b, "\n")
	ad <- readRDS(paste0(ad_dir, m, "-", b, ".rds"))

	ad <- ad[Class != "Ghost Clearance"]
	ad <- ad[week_end %in% all_weeks, .(GRP=sum(GRP), Duration=sum(Duration)), .(week_end, dma_code)]

	str <- data.table(expand.grid(week_end=all_weeks, dma_code=all_dmas))
	ad <- merge(str, ad, by=names(str), all.x=T)
	ad[is.na(ad)] <- 0

	ad[, month := floor_date(week_end, "month")]
	ad[, woy := pmin(week(week_end), 52)]

	market_regular <- copy(ad)
	store_regular <- merge(use.stores[cumshare<0.9, .(store_code_uc, dma_code)], ad, by="dma_code", allow.cartesian=TRUE)

	store_border <- merge(border.stores[cumshare<0.9, .(store_code_uc, dma_code, border_id)], ad, by="dma_code", allow.cartesian=TRUE)
	bad_borders <- store_border[, uniqueN(dma_code), border_id][V1<2, border_id]
	store_border <- store_border[!(border_id %in% bad_borders)]
	market_border <- merge(unique(store_border[, .(dma_code, border_id)]), ad, by="dma_code", allow.cartesian=TRUE)

	# Regular, Market
	g0 <- market_regular[, var(log(1+GRP))]
	d0 <- market_regular[, var(log(1+Duration))]
	gfit <- felm(log(1+GRP)~1|dma_code+month+woy, market_regular)
	dfit <- felm(log(1+Duration)~1|dma_code+month+woy, market_regular)
	g1 <- as.numeric(var(resid(gfit)))
	d1 <- as.numeric(var(resid(dfit)))
	gfit <- felm(log(1+GRP)~1|dma_code+week_end, market_regular)
	dfit <- felm(log(1+Duration)~1|dma_code+week_end, market_regular)
	g2 <- as.numeric(var(resid(gfit)))
	d2 <- as.numeric(var(resid(dfit)))
	out_1 <- data.table(Region="Market", Strategy="Regular",
						G0=g0, G1=g1, G2=g2, D0=d0, D1=d1, D2=d2)

	# Regular, Store
	g0 <- store_regular[, var(log(1+GRP))]
	d0 <- store_regular[, var(log(1+Duration))]
	gfit <- felm(log(1+GRP)~1|store_code_uc+month+woy, store_regular)
	dfit <- felm(log(1+Duration)~1|store_code_uc+month+woy, store_regular)
	g1 <- as.numeric(var(resid(gfit)))
	d1 <- as.numeric(var(resid(dfit)))
	gfit <- felm(log(1+GRP)~1|store_code_uc+week_end, store_regular)
	dfit <- felm(log(1+Duration)~1|store_code_uc+week_end, store_regular)
	g2 <- as.numeric(var(resid(gfit)))
	d2 <- as.numeric(var(resid(dfit)))
	out_2 <- data.table(Region="Store", Strategy="Regular",
						G0=g0, G1=g1, G2=g2, D0=d0, D1=d1, D2=d2)

	# Border, Market
	g0 <- market_border[, var(log(1+GRP))]
	d0 <- market_border[, var(log(1+Duration))]
	market_border[, border_dma := .GRP, .(border_id, dma_code)]
	market_border[, border_month := .GRP, .(border_id, month)]
	market_border[, border_week := .GRP, .(border_id, week_end)]
	gfit <- felm(log(1+GRP)~1|border_dma+border_month+woy, market_border)
	dfit <- felm(log(1+Duration)~1|border_dma+border_month+woy, market_border)
	g1 <- as.numeric(var(resid(gfit)))
	d1 <- as.numeric(var(resid(dfit)))
	gfit <- felm(log(1+GRP)~1|border_dma+border_week, market_border)
	dfit <- felm(log(1+Duration)~1|border_dma+border_week, market_border)
	g2 <- as.numeric(var(resid(gfit)))
	d2 <- as.numeric(var(resid(dfit)))
	out_3 <- data.table(Region="Market", Strategy="Border",
						G0=g0, G1=g1, G2=g2, D0=d0, D1=d1, D2=d2)

	# Border, Store
	g0 <- store_border[, var(log(1+GRP))]
	d0 <- store_border[, var(log(1+Duration))]
	store_border[, border_month := .GRP, .(border_id, month)]
	store_border[, border_week := .GRP, .(border_id, week_end)]
	gfit <- felm(log(1+GRP)~1|store_code_uc+border_month+woy, store_border)
	dfit <- felm(log(1+Duration)~1|store_code_uc+border_month+woy, store_border)
	g1 <- as.numeric(var(resid(gfit)))
	d1 <- as.numeric(var(resid(dfit)))
	gfit <- felm(log(1+GRP)~1|store_code_uc+border_week, store_border)
	dfit <- felm(log(1+Duration)~1|store_code_uc+border_week, store_border)
	g2 <- as.numeric(var(resid(gfit)))
	d2 <- as.numeric(var(resid(dfit)))
	out_4 <- data.table(Region="Store", Strategy="Border",
						G0=g0, G1=g1, G2=g2, D0=d0, D1=d1, D2=d2)

	# Combine
	output <- rbind(out_1, out_2, out_3, out_4, fill=TRUE)
	output[, `:=`(new_module_code=m, new_brand_code_uc=b)]
	return(output)
}

screening <- readRDS("/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]

setkey(screening, new_module_code, new_brand_code_uc)
modules <- screening$new_module_code
brands <- screening$new_brand_code_uc

cl <- makeCluster(24, type="FORK", outfile="/dev/stdout")
proc.list <- parLapply(cl, 1:nrow(screening), function(x) Process.R2(modules[x], brands[x]))
stopCluster(cl)

proc <- rbindlist(proc.list, fill=TRUE)
saveRDS(proc, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Advertising-R2.rds")







