rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)
library(lfe)

ad_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"
load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))

all_weeks <- seq(date("2011-01-08"), date("2014-12-27"), 7)
all_dmas <- unique(use.stores$dma_code)

options(lfe.eps=1e-4)

Process.CV <- function(m, b){
	cat(m, b, "\n")
	ad <- readRDS(paste0(ad_dir, m, "-", b, ".rds"))

	ad <- ad[Class != "Ghost Clearance"]
	ad <- ad[week_end %in% all_weeks, .(GRP=sum(GRP), Duration=sum(Duration)), .(week_end, dma_code)]
	str <- data.table(expand.grid(week_end=all_weeks, dma_code=all_dmas))
	ad <- merge(str, ad, by=names(str), all.x=T)
	ad[is.na(ad)] <- 0
	ad[, month := floor_date(week_end, "month")]
	ad[, woy := pmin(week(week_end), 52)]

	store_regular <- merge(use.stores[cumshare<0.9, .(store_code_uc, dma_code)], ad, by="dma_code", allow.cartesian=TRUE)
	store_border <- merge(border.stores[cumshare<0.9, .(store_code_uc, dma_code, border_id)], ad, by="dma_code", allow.cartesian=TRUE)
	bad_borders <- store_border[, uniqueN(dma_code), border_id][V1<2, border_id]
	store_border <- store_border[!(border_id %in% bad_borders)]

	# Regular, Store, Log
	mean <- store_regular[, mean(log(1+GRP))]
	g0 <- store_regular[, var(log(1+GRP))]
	gfit <- felm(log(1+GRP)~1|store_code_uc+month+woy, store_regular)
	g1 <- as.numeric(var(resid(gfit)))
	out1 <- data.table(Region="Store", Strategy="Regular", Log=TRUE,
						G0=g0, G1=g1, M=mean)

	# Regular, Store, No Log
	mean <- store_regular[, mean(GRP)]
	g0 <- store_regular[, var(GRP)]
	gfit <- felm(GRP~1|store_code_uc+month+woy, store_regular)
	g1 <- as.numeric(var(resid(gfit)))
	out2 <- data.table(Region="Store", Strategy="Regular", Log=FALSE,
						G0=g0, G1=g1, M=mean)

	# Border, Store, Log
	mean <- store_border[, mean(log(1+GRP))]
	g0 <- store_border[, var(log(1+GRP))]
	store_border[, border_month := .GRP, .(border_id, month)]
	gfit <- felm(log(1+GRP)~1|store_code_uc+border_month+woy, store_border)
	g1 <- as.numeric(var(resid(gfit)))
	out3 <- data.table(Region="Store", Strategy="Border", Log=TRUE,
						G0=g0, G1=g1, M=mean)

	# Border, Store, No Log
	mean <- store_border[, mean(GRP)]
	g0 <- store_border[, var(GRP)]
	store_border[, border_month := .GRP, .(border_id, month)]
	gfit <- felm(GRP~1|store_code_uc+border_month+woy, store_border)
	g1 <- as.numeric(var(resid(gfit)))
	out4 <- data.table(Region="Store", Strategy="Border", Log=FALSE,
						G0=g0, G1=g1, M=mean)

	# Combine
	output <- rbind(out1, out2, out3, out4, fill=TRUE)
	output[, `:=`(new_module_code=m, new_brand_code_uc=b)]
	return(output)
}

screening <- readRDS("/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]

setkey(screening, new_module_code, new_brand_code_uc)
modules <- screening$new_module_code
brands <- screening$new_brand_code_uc

cl <- makeCluster(24, type="FORK", outfile="/dev/stdout")
proc.list <- parLapply(cl, 1:nrow(screening), function(x) Process.CV(modules[x], brands[x]))
stopCluster(cl)

proc <- rbindlist(proc.list, fill=TRUE)
saveRDS(proc, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Advertising-Residual-CV.rds")







