rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)

ad_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"
universe <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))

all_weeks <- seq(date("2010-01-09"), date("2014-12-27"), 7)
all_dmas <- unique(use.stores$dma_code)

m <- 1484
b <- 531429

Process <- function(m, b){
	cat(m, b, "\n")
	ad <- readRDS(paste0(ad_dir, m, "-", b, ".rds"))

	ad <- ad[Class != "Ghost Clearance"]
	ad[Class=="Missing Network", Class := "Network"]
	ad[Class=="Spot" & major_network==TRUE, Class := "Spot Major"]
	ad[Class=="Spot" & major_network==FALSE, Class := "Spot Minor"]
	ad[Class=="Network" & major_network==TRUE, Class := "Network Major"]
	ad[Class=="Network" & major_network==FALSE, Class := "Network Minor"]
	ad[match_tier %in% c(1,3), match_tier := 13]
	ad[match_tier %in% c(2,4), match_tier := 24]

	ad <- ad[week_end %in% all_weeks, .(GRP=sum(GRP), Duration=sum(Duration)), .(week_end, dma_code, match_tier, Class)]
	ad <- dcast(ad, week_end+dma_code+Class~match_tier, value.var=c("GRP", "Duration"))
	if(!("GRP_13" %in% names(ad))){
		ad[, `:=`(GRP_13=0, Duration_13=0)]
	}
	if(!("GRP_24" %in% names(ad))){
		ad[, `:=`(GRP_24=0, Duration_24=0)]
	}

	str <- data.table(expand.grid(week_end=all_weeks, dma_code=all_dmas,
								  Class=c("Cable", "Network Major", "Network Minor", "Spot Major", "Spot Minor", "Syndicated")))
	ad <- merge(str, ad, by=names(str), all.x=T)
	ad[is.na(ad)] <- 0

	# Screening: For 2011-2014, calculate tier 1+3 fracpos and cmean
	screening <- ad[week_end > date("2011-01-01"), .(GRP_13=sum(GRP_13)), .(dma_code, week_end)]
	screening <- screening[, .(new_module_code=m, new_brand_code_uc=b, fracpos=mean(GRP_13>0), cmean=mean(GRP_13[GRP_13>0]))]

	# Share by type:
	shares <- ad[, lapply(.SD, sum), .(Class), .SDcols=c("GRP_13", "GRP_24", "Duration_13", "Duration_24")]
	shares[, `:=`(new_module_code=m, new_brand_code_uc=b)]

	return(list(screening=screening, shares=shares))
}

setkey(universe, new_module_code, new_brand_code_uc)
modules <- universe$new_module_code
brands <- universe$new_brand_code_uc

cl <- makeCluster(42, type="FORK", outfile="/dev/stdout")
proc.list <- parLapply(cl, 1:nrow(universe), function(x) Process(modules[x], brands[x]))
stopCluster(cl)

screening <- rbindlist(lapply(proc.list, function(x) x$screening))
shares <- rbindlist(lapply(proc.list, function(x) x$shares))

# Merge in brand names
info <- universe[, .(new_module_code, new_brand_code_uc, new_brand_descr)]
screening <- merge(screening, info, by=c("new_module_code", "new_brand_code_uc"))
shares <- merge(shares, info, by=c("new_module_code", "new_brand_code_uc"))

saveRDS(screening, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
saveRDS(shares, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Shares-By-Media-Type.rds")








