# Ad Variation Analysis
# Target: Histogram of Weekly Advertising
#	Observation Unit: Brand-Market-Week

rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)
library(lfe)

rms_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"
ad_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"

load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))

all_weeks <- seq(date("2010-01-09"), date("2014-12-27"), 7)[-(1:52)]

universe <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, adcomp_code, comp1_code, comp2_code, comp3_code)]

screening <- readRDS("/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))

Process <- function(module, brand){
	ad <- readRDS(paste0(ad_dir, module, "-", brand, ".rds"))
	ad <- ad[dma_code %in% use.stores[cumshare<0.9, dma_code]]
	ad <- ad[week_end %in% all_weeks]
	ad <- ad[Class != "Ghost Clearance"]
	ad <- ad[match_tier %in% c(1,3)]
	ad <- ad[, .(GRP=sum(GRP, na.rm=T), Duration=sum(Duration, na.rm=T)/3600), .(new_module_code, new_brand_code_uc, dma_code, week_end)]
	return(ad)
}

master <- vector("list", nrow(universe))
for(i in 1:nrow(universe)){
	print(i)
	proc <- Process(universe[i, new_module_code], universe[i, new_brand_code_uc])
	master[[i]] <- proc
}
master <- rbindlist(master, fill=TRUE)

saveRDS(master, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Ad-Intel-Variation.rds")













