# Magnitude Analysis
# Histogram: Revenue
#	(1) Observation: Brand. Sum across weeks
#	(2) Observation: Brand-Week
# Histogram: GRP
#	(1) Observation: Brand. Sum across weeks
#	(2) Observation: Brand-Week
# Scatterplot: log(1+GRP) ~ log(1+Revenue)
#	(1) Observation: Brand. Sum across weeks
#	(2) Observation: Brand-Week

rm(list=ls())
library(data.table)
library(bit64)
library(ggplot2)
library(lubridate)
library(foreach)
library(doParallel)
library(parallel)
library(lfe)

rms_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"
ad_dir <- "/grpshares/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel/"
home_dir <- "/grpshares/hitsch_shapiro_ads/Advertising-Generalizability/"

load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))

all_weeks <- seq(date("2010-01-09"), date("2014-12-27"), 7)[-(1:52)]

universe <- fread(paste0(home_dir, "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, adcomp_code, comp1_code, comp2_code, comp3_code)]

screening <- readRDS("/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))

Process <- function(module, brand){
	move <- readRDS(paste0(rms_dir, module, "/", brand, ".rds"))
	ad <- readRDS(paste0(ad_dir, module, "-", brand, ".rds"))

	move <- move[store_code_uc %in% use.stores[cumshare<0.9, store_code_uc]]
	move <- move[week_end %in% all_weeks]
	move_brand_week <- move[, .(new_module_code=module, new_brand_code_uc=brand, revenue=sum(units*price)), .(week_end)]
	move_brand <- move[, .(new_module_code=module, new_brand_code_uc=brand, revenue=sum(units*price))]

	ad <- ad[dma_code %in% use.stores[cumshare<0.9, dma_code]]
	ad <- ad[week_end %in% all_weeks]
	ad <- ad[Class != "Ghost Clearance"]
	ad <- ad[match_tier %in% c(1,3)]
	ad_brand_week <- ad[, .(GRP=sum(GRP, na.rm=T), Duration=sum(Duration, na.rm=T)/3600), .(new_module_code, new_brand_code_uc, week_end)]
	ad_brand <- ad[, .(GRP=sum(GRP, na.rm=T), Duration=sum(Duration, na.rm=T)/3600), .(new_module_code, new_brand_code_uc)]

	brand_week <- merge(move_brand_week, ad_brand_week, by=c("new_module_code", "new_brand_code_uc", "week_end"), all=T)
	brand_level <- merge(move_brand, ad_brand, by=c("new_module_code", "new_brand_code_uc"), all=T)
	brand_week[is.na(brand_week)] <- 0
	brand_level[is.na(brand_level)] <- 0
	return(list(week=brand_week, brand=brand_level))
}

master_week <- vector("list", nrow(universe))
master_brand <- vector("list", nrow(universe))
for(i in 1:nrow(universe)){
	print(i)
	proc <- Process(universe[i, new_module_code], universe[i, new_brand_code_uc])
	master_week[[i]] <- proc$week
	master_brand[[i]] <- proc$brand
}
master_week <- rbindlist(master_week, fill=TRUE)
master_brand <- rbindlist(master_brand, fill=TRUE)

save(master_week, master_brand, file="/grpshares/hitsch_shapiro_ads/data/Descriptive-Statistics/Magnitude-Analysis.RData")













