rm(list=ls())
library(data.table)
library(ggplot2)
source('~/Dropbox/Generic-Plot-Settings-v1.R')

graph_par$x_title_size <- 7.5

use <- readRDS("../Processed-Data/Advertising-R2.rds")

use1 <- use[, .(new_module_code, new_brand_code_uc, Region, Strategy,
                variable="GRP", Time="Month", value=1-G1/G0)]
use2 <- use[, .(new_module_code, new_brand_code_uc, Region, Strategy,
                variable="GRP", Time="Week", value=1-G2/G0)]
use3 <- use[, .(new_module_code, new_brand_code_uc, Region, Strategy,
                variable="Duration", Time="Month", value=1-D1/D0)]
use4 <- use[, .(new_module_code, new_brand_code_uc, Region, Strategy,
                variable="Duration", Time="Week", value=1-D2/D0)]
use <- rbind(use1, use2, use3, use4, fill=TRUE)

# Month
x1 <- use[Region=="Market" & Strategy=="Regular" & Time=="Month" & variable=="GRP", value]
x2 <- use[Region=="Market" & Strategy=="Regular" & Time=="Month" & variable=="Duration", value]
x3 <- use[Region=="Market" & Strategy=="Border" & Time=="Month" & variable=="GRP", value]
x4 <- use[Region=="Market" & Strategy=="Border" & Time=="Month" & variable=="Duration", value]
x5 <- use[Region=="Store" & Strategy=="Regular" & Time=="Month" & variable=="GRP", value]
x6 <- use[Region=="Store" & Strategy=="Regular" & Time=="Month" & variable=="Duration", value]
x7 <- use[Region=="Store" & Strategy=="Border" & Time=="Month" & variable=="GRP", value]
x8 <- use[Region=="Store" & Strategy=="Border" & Time=="Month" & variable=="Duration", value]

plt1 <- Custom.Hist(x1, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nMarket + Month + Week-of-year")
plt2 <- Custom.Hist(x2, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nMarket + Month + Week-of-year")
plt3 <- Custom.Hist(x3, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nBorder-Market + Border-Month + Week-of-year")
plt4 <- Custom.Hist(x4, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nBorder-Market + Border-Month + Week-of-year")
plt5 <- Custom.Hist(x5, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nStore + Month + Week-of-year")
plt6 <- Custom.Hist(x6, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nStore + Month + Week-of-year")
plt7 <- Custom.Hist(x7, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nStore + Border-Month + Week-of-year")
plt8 <- Custom.Hist(x8, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nStore + Border-Month + Week-of-year")

pdf("../Figures/Month-R2.pdf", width=10, height=13)
grid.arrange(plt1, plt2, plt3, plt4,
             plt5, plt6, plt7, plt8, nrow=4)
dev.off()

# Week
x1 <- use[Region=="Market" & Strategy=="Regular" & Time=="Week" & variable=="GRP", value]
x2 <- use[Region=="Market" & Strategy=="Regular" & Time=="Week" & variable=="Duration", value]
x3 <- use[Region=="Market" & Strategy=="Border" & Time=="Week" & variable=="GRP", value]
x4 <- use[Region=="Market" & Strategy=="Border" & Time=="Week" & variable=="Duration", value]
x5 <- use[Region=="Store" & Strategy=="Regular" & Time=="Week" & variable=="GRP", value]
x6 <- use[Region=="Store" & Strategy=="Regular" & Time=="Week" & variable=="Duration", value]
x7 <- use[Region=="Store" & Strategy=="Border" & Time=="Week" & variable=="GRP", value]
x8 <- use[Region=="Store" & Strategy=="Border" & Time=="Week" & variable=="Duration", value]

plt1 <- Custom.Hist(x1, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nMarket + Week")
plt2 <- Custom.Hist(x2, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nMarket +Week")
plt3 <- Custom.Hist(x3, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nBorder-Market + Border-Week")
plt4 <- Custom.Hist(x4, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nBorder-Market + Border-Week")
plt5 <- Custom.Hist(x5, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nStore + Week")
plt6 <- Custom.Hist(x6, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nStore + Week")
plt7 <- Custom.Hist(x7, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="GRP\nStore + Border-Week")
plt8 <- Custom.Hist(x8, xbreaks=seq(0,1,0.1), binwidth=0.05, xtitle="R Squared",
                    main_title="Duration\nStore + Border-Week")

pdf("../Figures/Week-R2.pdf", width=10, height=13)
grid.arrange(plt1, plt2, plt3, plt4,
             plt5, plt6, plt7, plt8, nrow=4)
dev.off()




