rm(list=ls())
library(data.table)

screening <- readRDS("../Processed-Data/Screening.rds")
shares <- readRDS("../Processed-Data/Shares-By-Media-Type.rds")

screening <- screening[fracpos > 0.05 & cmean > 10, .(new_module_code, new_brand_code_uc)]
shares <- merge(shares, screening, by=names(screening))

shares[grepl("Network", Class), Class := "Network"]
shares[grepl("Spot", Class), Class := "Spot"]

shares <- shares[, .(GRP=sum(GRP_13+GRP_24), Duration=sum((Duration_13+Duration_24)/3600)), 
                 .(new_module_code, new_brand_code_uc, new_brand_descr, Class)]

shares[, `:=`(Total_GRP=sum(GRP), Total_Duration=sum(Duration)), .(new_module_code, new_brand_code_uc)]
shares[, `:=`(GRP=GRP/Total_GRP, Duration=Duration/Total_Duration)]

g <- dcast(shares, new_module_code+new_brand_code_uc+new_brand_descr+Total_GRP~Class, value.var="GRP")
d <- dcast(shares, new_module_code+new_brand_code_uc+new_brand_descr+Total_Duration~Class, value.var="Duration")

g[, tmp := Spot/(Spot+Network)]
d[, tmp := Spot/(Spot+Network)]
setnames(g, "tmp", "Spot/(Spot+Network)")
setnames(d, "tmp", "Spot/(Spot+Network)")

g <- g[, lapply(.SD, function(x) round(x, 3)), .(new_module_code, new_brand_code_uc, new_brand_descr, Total_GRP)]
d <- d[, lapply(.SD, function(x) round(x, 3)), .(new_module_code, new_brand_code_uc, new_brand_descr, Total_Duration)]

setnames(g, c("Cable", "Network", "Spot", "Syndicated"), paste0(c("Cable", "Network", "Spot", "Syndicated"), "/Total"))
setnames(d, c("Cable", "Network", "Spot", "Syndicated"), paste0(c("Cable", "Network", "Spot", "Syndicated"), "/Total"))

setorder(g, -Total_GRP)
setorder(d, -Total_Duration)

fwrite(g, file="../Tables/GRP-Shares-By-Media-Type.csv")
fwrite(d, file="../Tables/Duration-Shares-By-Media-Type.csv")










