rm(list=ls())
library(data.table)

result_dir <- "/grpshares/hitsch_shapiro_ads/data/Regression-Results/Market-Regular-With-Feature-Display/"
outfile <- "/grpshares/hitsch_shapiro_ads/data/Regression-Results/Collected/C7.RData"

Process.File <- function(file){
	print(file)
	use <- readRDS(paste0(result_dir, file))
	if(is.null(use)){
		return(data.table())
	} else{
		use <- use[grepl("(sl|ls)", variable)]
		use[, variable := gsub("(.*)_(.*)_(.*)", "\\3", variable)]
		use[variable=="Comp", variable := "5"]
		use[, spec := NULL]
		use[, level := "Market"]
		use[, strategy := "Regular"]
		use[, feature := 1]
		setnames(use, c("coef", "se", "pval"), c("c", "s", "p"))
	}
	return(use)
}
		
files <- list.files(result_dir)
proc <- rbindlist(lapply(files, Process.File), fill=TRUE)

# Let's Isolate Tables
id_cols <- c("new_module_code", "new_brand_code_uc", "goodwill", "measure", "time", "level", "strategy",
			 "feature", "ncomp", "decay")

obs <- proc[ncomp>=0, c("new_module_code", "new_brand_code_uc", "ncomp", "nobs"), with=FALSE]
obs <- unique(obs)
obs <- dcast(obs, new_module_code+new_brand_code_uc~ncomp, value.var="nobs")
setnames(obs, as.character(0:3), paste0("n", 0:3))
obs[, r1 := n1 / n0]
obs[, r2 := n2 / n0]
obs[, r3 := n3 / n0]
obs <- obs[, .(new_module_code, new_brand_code_uc, r1, r2, r3)]

gfit <- proc[, c(id_cols, "r2", "r2p", "mse"), with=FALSE]
gfit <- unique(gfit)

best <- gfit[, .(decay=decay[which.min(mse)]), by=eval(id_cols[-10])]

coef <- proc[, c(id_cols, "nobs", "variable", "c", "s", "p"), with=FALSE]

coef[, decay := round(decay*100)]
best[, decay := round(decay*100)]

coef_best <- merge(coef, best, by=c(id_cols))
coef_select <- coef[decay %in% c(0,25,50,75,90,95,100)]

coef_best[, delta_status := "Best"]
coef_select[, delta_status := "0.00"]
coef_select[decay==25, delta_status := "0.25"]
coef_select[decay==50, delta_status := "0.50"]
coef_select[decay==75, delta_status := "0.75"]
coef_select[decay==90, delta_status := "0.90"]
coef_select[decay==95, delta_status := "0.95"]
coef_select[decay==100, delta_status := "1.00"]

coef_output <- rbind(coef_best, coef_select, fill=TRUE)
coef_output[, decay := round(decay / 100, 2)]

coef_output[, c := round(c, 4)]
coef_output[, s := round(s, 4)]
coef_output[, p := round(p, 4)]

obs[, level := "Market"]
obs[, strategy := "Regular"]
obs[, feature := 1]

save(coef_output, obs, file=outfile, compress=TRUE)

