rm(list=ls())
library(data.table)
library(ggplot2)
source("~/Dropbox/Generic-Plot-Settings-v1.R")

graph_par$round <- 4
graph_par$stat_text_xalign <- 0.8
graph_par$stat_text_height <- 0.97
graph_par$stat_move <- 1.05
graph_par$x_title_size <- 9
graph_par$axis_size <- 9
graph_par$label_size <- 9
graph_par$main_title_size <- 9

use <- readRDS("../Data/Collected-Coefficients.rds")
use[, id := .GRP, .(new_module_code, new_brand_code_uc)]
use <- use[strategy %in% c("Border", "Regular") & variable==13, 
           .(id, decay, feature, strategy, se)]
use <- dcast(use, id+decay+feature~strategy, value.var="se")
use <- use[, .(id, decay, feature, value=Border/Regular)]

v1 <- use[feature=="No" & round(decay*100)==90, value]
v2 <- use[feature=="Yes" & round(decay*100)==90, value]
v3 <- use[feature=="No" & round(decay*100)==0, value]
v4 <- use[feature=="Yes" & round(decay*100)==0, value]

p1 <- Custom.Hist(v1, binwidth=0.05, color_val=v1<1,
                  fill_col="grey92", part_col="lightblue2",
                  xbreaks=seq(0,4,0.5), 
                  xtitle="Border SE / Regular SE",
                  main_title="Delta = 0.90, No Feature")

p2 <- Custom.Hist(v2, binwidth=0.05, color_val=v2<1,
                  fill_col="grey92", part_col="lightblue2",
                  xbreaks=seq(0,4,0.5), 
                  xtitle="Border SE / Regular SE",
                  main_title="Delta = 0.90, With Feature")

p3 <- Custom.Hist(v3, binwidth=0.05, color_val=v3<1,
                  fill_col="grey92", part_col="lightblue2",
                  xbreaks=seq(0,4,0.5), 
                  xtitle="Border SE / Regular SE",
                  main_title="Delta = 0, No Feature")

p4 <- Custom.Hist(v4, binwidth=0.05, color_val=v4<1,
                  fill_col="grey92", part_col="lightblue2",
                  xbreaks=seq(0,4,0.5), 
                  xtitle="Border SE / Regular SE",
                  main_title="Delta = 0, With Feature")

pdf("./SE-Quotient.pdf", 6, 4.75)
print(p1)
print(p2)
print(p3)
print(p4)
dev.off()
