# ------------------------------------ #
# Run Residual Variation Estimations for Border Strategy
# 
#
# Estimation Parameters: 
# Strategy: Border
# Delta: seq(0, 0.9, 0.1)
# Custering: dma_code + week_end
# level: Store, Market
# Time: Month
# Measure: GRP
# Feature: No, Yes
# Goodwill: logsum
# Maxcomp:  3
# Maxloss: 0.2
# ------------------------------------ #

# Call regression functions located in the same directory
script.dir <- dirname(sys.frame(1)$ofile)
source(paste0(script.dir, "/Regression-Functions.R"))

# Output directory, change accordingly
output.dir <- "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Full-Regression-Results/Residual-Var-wo-anomaly/"

strategy      = "Border"
delta_ls      = seq(0, 0.9, 0.1)
cluster       = "Border_DMA+week_end"
level_sets    = c("Store", "Market")
time_sets     = c("Month")
measure       = "GRP"
feature_sets  = c("No", "Yes")
goodwill_sets = c("logsum")
maxcomp       = "3"
maxloss       = "0.20"

for (feature in feature_sets) {
	for (level in level_sets) {
		for (goodwill in goodwill_sets) {
			for (time in time_sets) {
			  
			  if (level == "Store") {
			    fe = paste0("|store_code_uc+Border_",
			                time,
			                "+WOY")
			  } else {
			    fe = paste0("|Border_DMA+Border_",
			                time,
			                "+WOY")
			  }
			  
			  
			  output_location = paste0(output.dir,
			                           strategy,
			                           "-",
			                           level,
			                           "-",
			                           feature,
			                           "-",
			                           goodwill,
			                           "-",
			                           cluster,
			                           "-",
			                           time,
			                           "/")
				
				if (!dir.exists(file.path(output_location))) {
					dir.create(file.path(output_location),
					           recursive = TRUE,
							   showWarnings = FALSE)
				} 

				Ad.Generalizability.Regression(output_dir = output_location,
							   				   strategy   = strategy,
							   				   delta_list = delta_ls,
							   				   fixed_effect = fe,
							   				   clustering = cluster,
							   				   level = level,
							   				   time  = time,
							   				   measure = measure,
							   				   feature = feature,
							   				   goodwill = goodwill,
							   				   maxcomp = maxcomp,
							   				   maxloss = maxloss)

			}
		}
	}
}


cat("Done.\n")



