# ---------------------------------------- #
# Advertising Generalizability Estimations
# 
# 
# ---------------------------------------- #

library(data.table)
library(bit64)
library(lfe)
library(RcppRoll)
library(lubridate)
library(Hmisc)
library(doParallel)

# Nielsen Processed Data Directory (change accordingly)
home_dir <- "/project/hitsch_shapiro_ads/Advertising-Generalizability/"
ad_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel-wo-Anomaly/"
rms_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"

N_LAGS <- 52

# lfe option, could speed up the algorithm
# (not used in our current results)
# options(lfe.eps=1e-4)

Formula.DT <- data.table(Maxcomp  = rep(c("Barebone",
                                          "0",
                                          "1",
                                          "2",
                                          "3"), each = 8),
                         Goodwill = rep(rep(c("logsum", 
                                              "sumlog"), each = 2), 10),
                         Measure  = rep(c("GRP", "Duration"), 20),
                         Feature  = rep(rep(c("No", "Yes"), each = 4), 5),
                         Formula  = c(
                            "ls_GRP_13~ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~feature_0+display_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~feature_0+display_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~feature_0+display_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~feature_0+display_0+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+feature_0+display_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+feature_0+display_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+feature_0+display_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+feature_0+display_0+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+feature_0+display_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+feature_0+display_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+feature_0+display_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+feature_0+display_0+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+feature_0+display_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+feature_0+display_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+feature_0+display_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+feature_0+display_0+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+sl_Duration_24+sl_Duration_Comp",
                            
                            "ls_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+feature_0+display_0+ls_GRP_24+ls_GRP_Comp", 
                            "ls_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+feature_0+display_0+ls_Duration_24+ls_Duration_Comp",
                            "sl_GRP_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+feature_0+display_0+sl_GRP_24+sl_GRP_Comp", 
                            "sl_Duration_13~price_0+promo_0+price_1+promo_1+price_2+promo_2+price_3+promo_3+feature_0+display_0+sl_Duration_24+sl_Duration_Comp"))


Load.RMS.Data <- function(module, 
                          brand,
                          store,
                          universe,
                          strategy,
                          feature,
                          level) {
   cat("Loading RMS Data...")
   
   # 1.1: Own Brand
   infile <- paste0(rms_dir, module, "/", brand, ".rds")
   move <- readRDS(infile)
   move[, base_price := NULL]
   
   if (level == "Market") {
      # cat("Level is market\n")
      move <- merge(move, store, by="store_code_uc", allow.cartesian=TRUE)
      if (feature == "Yes") {
         if (strategy == "Border") {
            # cat("feature is Yes and strategy is Border\n")
            move <- move[, .(units=sum(units, na.rm=T), 
                             price=wtd.mean(price, total_revenue, 
                                            na.rm=T),
                             promo=wtd.mean(promo, total_revenue, 
                                            na.rm=T),
                             feature=wtd.mean(feature, total_revenue, 
                                              na.rm=T),
                             display=wtd.mean(display, total_revenue, 
                                              na.rm=T)), 
                         .(dma_code, border_id, week_end)]
            move <- move[!is.na(feature) & !is.na(display)]
            ## Remove one-sided borders
            bad_borders <- move[, uniqueN(dma_code), border_id][V1<2, border_id]
            move <- move[!(border_id %in% bad_borders)]
            
         } else {
            move <- move[, .(units=sum(units, na.rm=T), 
                             price=wtd.mean(price, total_revenue, na.rm=T),
                             promo=wtd.mean(promo, total_revenue, na.rm=T),
                             feature=wtd.mean(feature, total_revenue, na.rm=T),
                             display=wtd.mean(display, total_revenue, na.rm=T)), .(dma_code, week_end)]
            move <- move[!is.na(feature) & !is.na(display)]
         }
      } else {
         if (strategy == "Border") {
            move <- move[, .(units=sum(units, na.rm=T), 
                             price=wtd.mean(price, total_revenue, 
                                            na.rm=T),
                             promo=wtd.mean(promo, total_revenue, 
                                            na.rm=T)), 
                         .(dma_code, border_id, week_end)]
            ## Remove one-sided borders
            bad_borders <- move[, uniqueN(dma_code), border_id][V1<2, border_id]
            move <- move[!(border_id %in% bad_borders)]
            
         } else {
            move <- move[, .(units=sum(units, na.rm=T), 
                             price=wtd.mean(price, total_revenue, 
                                            na.rm=T),
                             promo=wtd.mean(promo, total_revenue, 
                                            na.rm=T)), 
                         .(dma_code, week_end)]
         }
      }
   } else {
      # cat("Level is store\n")
      if (feature == "Yes") {
         # cat("Feature is Yes\n")
         move <- move[!is.na(feature) & !is.na(display)]
      } else {
         move[, feature := NULL]
         move[, display := NULL]
      }
      
      if (strategy == "Border") {
         # cat("strategy is Border\n")
         move <- merge(move, store, by="store_code_uc", allow.cartesian=TRUE)
         ## Remove one-sided borders
         bad_borders <- move[, uniqueN(dma_code), border_id][V1<2, border_id]
         move <- move[!(border_id %in% bad_borders)]
      } else {
         move <- merge(move, store, by="store_code_uc")
      }
   }
   
   move[, brand_code := 0]
   
   # cat("First done\n")
   
   # 1.2: Top Competitor
   comp1 <- universe[new_module_code==module & new_brand_code_uc==brand, comp1_code]
   if(is.na(comp1)){
      move1 <- data.table()
   } else{
      infile <- paste0(rms_dir, module, "/", comp1, ".rds")
      move1 <- readRDS(infile)
      move1[, base_price := NULL]
      
      if (level == "Market") {
         
         move1 <- merge(move1, store, 
                        by="store_code_uc", allow.cartesian=TRUE)
         if (feature == "Yes") {
            if (strategy == "Border") {
               move1 <- move1[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T),
                                  feature=wtd.mean(feature, 
                                                   total_revenue, 
                                                   na.rm=T),
                                  display=wtd.mean(display, 
                                                   total_revenue, 
                                                   na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               move1 <- move1[!is.na(feature) & !is.na(display)]
               ## Remove one-sided borders
               bad_borders <- move1[, uniqueN(dma_code), border_id][V1<2, border_id]
               move1 <- move1[!(border_id %in% bad_borders)]
               
            } else {
               move1 <- move1[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, na.rm=T),
                                  feature=wtd.mean(feature, total_revenue, na.rm=T),
                                  display=wtd.mean(display, total_revenue, na.rm=T)), .(dma_code, week_end)]
               move1 <- move1[!is.na(feature) & !is.na(display)]
            }
         } else {
            if (strategy == "Border") {
               move1 <- move1[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               ## Remove one-sided borders
               bad_borders <- move1[, uniqueN(dma_code), border_id][V1<2, border_id]
               move1 <- move1[!(border_id %in% bad_borders)]
               
            } else {
               move1 <- move1[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, week_end)]
            }
         }
         
      } else {
         if (feature == "Yes") {
            move1 <- move1[!is.na(feature) & !is.na(display)]
         } else {
            move1[, feature := NULL]
            move1[, display := NULL]
         }
         
         if (strategy == "Border") {
            # cat("strategy is Border\n")
            move1 <- merge(move1, store, by="store_code_uc", allow.cartesian=TRUE)
            move1 <- move1[!(border_id %in% bad_borders)]
         } else {
            move1 <- merge(move1, store, by="store_code_uc")
         }
      }
      
      move1[, brand_code := 1]
   }
   
   # cat("Second Done \n")
   
   # 1.3: Second Competitor
   comp2 <- universe[new_module_code==module & new_brand_code_uc==brand, comp2_code]
   if(is.na(comp2)){
      move2 <- data.table()
   } else{
      infile <- paste0(rms_dir, module, "/", comp2, ".rds")
      move2 <- readRDS(infile)
      move2[, base_price := NULL]
      
      if (level == "Market") {
         
         move2 <- merge(move2, store, 
                        by="store_code_uc", allow.cartesian=TRUE)
         if (feature == "Yes") {
            if (strategy == "Border") {
               move2 <- move2[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T),
                                  feature=wtd.mean(feature, 
                                                   total_revenue, 
                                                   na.rm=T),
                                  display=wtd.mean(display, 
                                                   total_revenue, 
                                                   na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               move2 <- move2[!is.na(feature) & !is.na(display)]
               ## Remove one-sided borders
               bad_borders <- move2[, uniqueN(dma_code), border_id][V1<2, border_id]
               move2 <- move2[!(border_id %in% bad_borders)]
               
            } else {
               move2 <- move2[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, na.rm=T),
                                  feature=wtd.mean(feature, total_revenue, na.rm=T),
                                  display=wtd.mean(display, total_revenue, na.rm=T)), .(dma_code, week_end)]
               move2 <- move2[!is.na(feature) & !is.na(display)]
            }
         } else {
            if (strategy == "Border") {
               move2 <- move2[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               ## Remove one-sided borders
               bad_borders <- move2[, uniqueN(dma_code), border_id][V1<2, border_id]
               move2 <- move2[!(border_id %in% bad_borders)]
               
            } else {
               
               move2 <- move2[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, week_end)]
            }
         }
         
      } else {
         if (feature == "Yes") {
            move2 <- move2[!is.na(feature) & !is.na(display)]
         } else {
            move2[, feature := NULL]
            move2[, display := NULL]
         }
         
         if (strategy == "Border") {
            # cat("strategy is Border\n")
            move2 <- merge(move2, store, by="store_code_uc", allow.cartesian=TRUE)
            move2 <- move2[!(border_id %in% bad_borders)]
         } else {
            move2 <- merge(move2, store, by="store_code_uc")
         }
      }
      
      move2[, brand_code := 2]
   }
   
   # cat("Third done \n")
   
   # 1.4: Third Competitor
   comp3 <- universe[new_module_code==module & new_brand_code_uc==brand, comp3_code]
   if(is.na(comp3)){
      move3 <- data.table()
   } else{
      infile <- paste0(rms_dir, module, "/", comp3, ".rds")
      move3 <- readRDS(infile)
      move3[, base_price := NULL]
      
      if (level == "Market") {
         
         move3 <- merge(move3, store, 
                        by="store_code_uc", allow.cartesian=TRUE)
         if (feature == "Yes") {
            if (strategy == "Border") {
               move3 <- move3[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T),
                                  feature=wtd.mean(feature, 
                                                   total_revenue, 
                                                   na.rm=T),
                                  display=wtd.mean(display, 
                                                   total_revenue, 
                                                   na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               move3 <- move3[!is.na(feature) & !is.na(display)]
               ## Remove one-sided borders
               bad_borders <- move3[, uniqueN(dma_code), border_id][V1<2, border_id]
               move3 <- move3[!(border_id %in% bad_borders)]
               
            } else {
               move3 <- move3[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, na.rm=T),
                                  feature=wtd.mean(feature, total_revenue, na.rm=T),
                                  display=wtd.mean(display, total_revenue, na.rm=T)), .(dma_code, week_end)]
               move3 <- move3[!is.na(feature) & !is.na(display)]
            }
         } else {
            if (strategy == "Border") {
               move3 <- move3[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, border_id, week_end)]
               ## Remove one-sided borders
               bad_borders <- move3[, uniqueN(dma_code), border_id][V1<2, border_id]
               move3 <- move3[!(border_id %in% bad_borders)]
               
            } else {
               move3 <- move3[, .(units=sum(units, na.rm=T), 
                                  price=wtd.mean(price, total_revenue, 
                                                 na.rm=T),
                                  promo=wtd.mean(promo, total_revenue, 
                                                 na.rm=T)), 
                              .(dma_code, week_end)]
            }
         }
         
      } else {
         if (feature == "Yes") {
            move3 <- move3[!is.na(feature) & !is.na(display)]
         } else {
            move3[, feature := NULL]
            move3[, display := NULL]
         }
         
         if (strategy == "Border") {
            # cat("strategy is Border\n")
            move3 <- merge(move3, store, by="store_code_uc", allow.cartesian=TRUE)
            move3 <- move3[!(border_id %in% bad_borders)]
         } else {
            move3 <- merge(move3, store, by="store_code_uc")
         }
      }
      
      move3[, brand_code := 3]
   }
   
   # cat("Fourth done \n")
   
   # 1.5: Combine and dcast
   move_combined <- rbindlist(list(move, move1, move2, move3))
   
   if (feature == "Yes") {
      if (strategy == "Border") {
         if (level == "Market") {
            move_combined <- dcast(move_combined, 
                                   dma_code+border_id+week_end~brand_code,
                                   value.var=c("units", "price", "promo", 
                                               "feature", "display"))
         } else {
            move_combined <- dcast(move_combined, 
                                   store_code_uc+dma_code+border_id+week_end~brand_code,
                                   value.var=c("units", "price", "promo", 
                                               "feature", "display"))
         }
         
      } else {
         if (level == "Market") {
            move_combined <- dcast(move_combined, 
                                   dma_code+week_end~brand_code,
                                   value.var=c("units", "price", "promo", "feature", "display"))
         } else {
            move_combined <- dcast(move_combined, 
                                   store_code_uc+dma_code+week_end~brand_code,
                                   value.var=c("units", "price", "promo", "feature", "display"))
         }
         
      }
      
      cols_to_remove <- c(paste0("units_", 1:3), paste0("feature_", 1:3), paste0("display_", 1:3))
      
   } else {
      
      if (strategy == "Border") {
         if (level == "Market") {
            move_combined <- dcast(move_combined, 
                                   dma_code+border_id+week_end~brand_code,
                                   value.var=c("units", "price", "promo"))
         } else {
            move_combined <- dcast(move_combined, 
                                   store_code_uc+dma_code+border_id+week_end~brand_code,
                                   value.var=c("units", "price", "promo"))
         }
         
      } else {
         
         if (level == "Market") {
            move_combined <- dcast(move_combined, 
                                   dma_code+week_end~brand_code,
                                   value.var=c("units", "price", "promo"))
         } else {
            move_combined <- dcast(move_combined, 
                                   store_code_uc+dma_code+week_end~brand_code,
                                   value.var=c("units", "price", "promo"))
         }
         
      }
      
      cols_to_remove <- c(paste0("units_", 1:3))
   }
   
   suppressWarnings(move_combined[, (cols_to_remove) := NULL])
   move_combined <- move_combined[!is.na(units_0)]
   
   cat("Done\n")
   return(move_combined)
}


Load.AdIntel.Data <- function(module, 
                              brand,
                              delta,
                              store,
                              universe,
                              strategy) {
   
   cat("Loading Ad-Intel Data...")
   
   # 2.1 Own Brand
   infile <- paste0(ad_dir, module, "-", brand, ".rds")
   ad <- readRDS(infile)
   
   ad <- ad[!(Class %in% c("Ghost Clearance", 
                           "National Network", 
                           "National Syndicated"))]
   
   ad[match_tier %in% c(1,3), match_tier := 13]
   ad[match_tier %in% c(2,4), match_tier := 24]
   ad[, Duration := Duration / 60]
   ad <- ad[, .(Duration=sum(Duration, na.rm=T), GRP=sum(GRP, na.rm=T)), .(dma_code, week_end, match_tier)]
   ad <- ad[year(week_end) <= 2014 & dma_code %in% unique(store$dma_code)]
   ad <- dcast(ad, dma_code+week_end~match_tier, value.var=c("GRP", "Duration"))
   str <- data.table(expand.grid(dma_code = setdiff(unique(store$dma_code), c(740, 810, 630)),
                                 week_end = seq(date("2010-01-09"), date("2014-12-27"), 7)))
   ad <- merge(str, ad, by=names(str), all.x=T)
   ad[is.na(ad)] <- 0
   if(!("GRP_13" %in% names(ad))){
      ad[, `:=`(GRP_13=0.0, Duration_13=0.0)]
   }
   if(!("GRP_24" %in% names(ad))){
      ad[, `:=`(GRP_24=0.0, Duration_24=0.0)]
   }
   
   # 2.2 Advertising Competitor
   adcomp <- universe[new_module_code==module & new_brand_code_uc==brand, adcomp_code]
   infile <- paste0(ad_dir, module, "-", adcomp, ".rds")
   if(!file.exists(infile)){
      ad1 <- NULL
   } else{
      ad1 <- readRDS(infile)
      ad1 <- ad1[!(Class %in% c("Ghost Clearance",  							
                                "National Network",  							
                                "National Syndicated"))]
      ad1[, Duration := Duration / 60]
      ad1 <- ad1[, .(Duration_Comp=sum(Duration, na.rm=T), GRP_Comp=sum(GRP, na.rm=T)), .(dma_code, week_end)]
      ad1 <- ad1[year(week_end) <= 2014 & dma_code %in% unique(store$dma_code)]
      str <- data.table(expand.grid(dma_code = setdiff(unique(store$dma_code), c(740, 810, 630)),
                                    week_end = seq(date("2010-01-09"), date("2014-12-27"), 7)))
      ad1 <- merge(str, ad1, by=names(str), all.x=T)
      ad1[is.na(ad1)] <- 0
   }
   
   # 2.3: Combine
   if(is.null(ad1)){
      ad[, `:=`(Duration_Comp=0.0, GRP_Comp=0.0)]
   } else{
      ad <- merge(ad, ad1, by=c("dma_code", "week_end"))
   }
   
   cat("Done\n")
   
   cat("Calculating Goodwill...")
   
   setkey(ad, dma_code, week_end)
   geom_weights <- sort(cumprod(c(1.0, rep(delta, times = N_LAGS))))
   
   ad[, ls_GRP_13  := log(roll_sum(GRP_13, n = N_LAGS+1, weights = geom_weights,
                                   normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   ad[, ls_GRP_24  := log(roll_sum(GRP_24, n = N_LAGS+1, weights = geom_weights,
                                   normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   ad[, ls_GRP_Comp  := log(roll_sum(GRP_Comp, n = N_LAGS+1, weights = geom_weights,
                                     normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   ad[, ls_Duration_13  := log(roll_sum(Duration_13, n = N_LAGS+1, weights = geom_weights,
                                        normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   ad[, ls_Duration_24  := log(roll_sum(Duration_24, n = N_LAGS+1, weights = geom_weights,
                                        normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   ad[, ls_Duration_Comp  := log(roll_sum(Duration_Comp, n = N_LAGS+1, weights = geom_weights,
                                          normalize = FALSE, align = "right",  fill = NA)+1), dma_code]
   
   ad[, sl_GRP_13  := roll_sum(log(1+GRP_13), n = N_LAGS+1, weights = geom_weights,
                               normalize = FALSE, align = "right",  fill = NA), dma_code]
   ad[, sl_GRP_24  := roll_sum(log(1+GRP_24), n = N_LAGS+1, weights = geom_weights,
                               normalize = FALSE, align = "right",  fill = NA), dma_code]
   ad[, sl_GRP_Comp  := roll_sum(log(1+GRP_Comp), n = N_LAGS+1, weights = geom_weights,
                                 normalize = FALSE, align = "right",  fill = NA), dma_code]
   ad[, sl_Duration_13  := roll_sum(log(1+Duration_13), n = N_LAGS+1, weights = geom_weights,
                                    normalize = FALSE, align = "right",  fill = NA), dma_code]
   ad[, sl_Duration_24  := roll_sum(log(1+Duration_24), n = N_LAGS+1, weights = geom_weights,
                                    normalize = FALSE, align = "right",  fill = NA), dma_code]
   ad[, sl_Duration_Comp  := roll_sum(log(1+Duration_Comp), n = N_LAGS+1, weights = geom_weights,
                                      normalize = FALSE, align = "right",  fill = NA), dma_code]
   
   ad <- ad[complete.cases(ad)]
   ad[, c("GRP_13", "GRP_24", "GRP_Comp", "Duration_13", "Duration_24", "Duration_Comp") := NULL]
   
   cat("Done\n")
   return(ad)
}

Run.Single.Regression <- function(module,
                                  brand,
                                  store,
                                  universe,
                                  strategy = c("Border",
                                               "Regular",
                                               "Naive",
                                               "MarketFE-only",
                                               "MarketFE-WOY",
                                               "MarketFE+WOY+National^2",
                                               "MarketFE+WOY+National^3",
                                               "MarketFE+WOY+National^4"),
                                  delta_list,
                                  fixed_effect,
                                  clustering,
                                  level = c("Store", "Market"),
                                  time  = c("Month", "Quarter", 
                                            "None", "Week"),
                                  measure = c("GRP",
                                              "Duration"),
                                  feature = c("Yes",
                                              "No"),
                                  goodwill = c("logsum",
                                               "sumlog"),
                                  maxcomp  = c("3",
                                               "2",
                                               "1",
                                               "0",
                                               "Barebone"),
                                  x = 0.1) {
   
   
   move <- Load.RMS.Data(module, 
                         brand,
                         store,
                         universe, 
                         strategy,
                         feature,
                         level)
   
   master <- vector("list", length(delta_list))
   for(i in 1:length(delta_list)){
      delta <- delta_list[i]
      
      ad <- Load.AdIntel.Data(module, 
                              brand,
                              delta,
                              store,
                              universe,
                              strategy)
      cat("Running Regressions for", delta, "\n")
      
      Run.Barebone.Regressions <- function(strategy, 
                                           feature) {
         # print(strategy)
         # print(feature)
         if (strategy == "Border") {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0))]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0))]
               }
               
            }
            regdata <- regdata[complete.cases(regdata)]
            bad_borders <- regdata[, uniqueN(dma_code), 
                                   border_id][V1<2, border_id]
            regdata <- regdata[!(border_id %in% bad_borders)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, Border_Quarter := .GRP, .(border_id, Quarter)]
            regdata[, Border_Month := .GRP, .(border_id, Month)]
            regdata[, Border_DMA := .GRP, .(border_id, dma_code)]
            regdata[, Border_Week := .GRP, .(border_id, week_end)]
            regdata[, WOY := pmin(week(week_end), 52)]
            
         } else {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0))]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0))]
               }
               
               
            }
            regdata <- regdata[complete.cases(regdata)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, WOY := pmin(week(week_end), 52)]
         }
         
         # print(names(regdata))
         
         formula <- Formula.DT[Measure == measure & 
                                  Goodwill == goodwill & 
                                  Feature == feature &
                                  Maxcomp == "Barebone",
                               Formula]
         
         formula <- paste0(formula,
                           fixed_effect, 
                           "|0|",
                           clustering)
         
         # cat(formula, "\n")
         
         fit <- NULL
         try(fit <- felm(as.formula(formula), regdata, exactDOF=TRUE))
         if (is.null(fit)) {
            fit <- felm(as.formula(formula), regdata)
         }
         
         resid_ls <- as.data.table(resid(fit))
         resid_ls[, resid := ls_GRP_13]
         resid_ls[, ls_GRP_13 := NULL]
         resid_ls[, abs_resid := abs(resid)]
         mean_resid_ls = mean(resid_ls[, resid])
         mean_abs_resid_ls = mean(resid_ls[, abs_resid])
         
         mean_ad = mean(regdata[, ls_GRP_13])
         
         fraction = resid_ls[abs_resid >= x * mean_ad, .N] / resid_ls[,.N]
         
         resid_variation = sd(resid_ls[, resid]) / mean_resid_ls
         
         return(data.table(delta = delta,
                           coef_var = sd(resid_ls[, resid]) / mean_ad,
                           resid_var = resid_variation,
                           mean_resid = mean_resid_ls,
                           mean_abs_resid = mean_abs_resid_ls,
                           mean_ad = mean_ad,
                           result  = mean_abs_resid_ls/mean_ad,
                           result1 = fraction))
         
      } 
      
      Run.C0.Regressions <- function(strategy, 
                                     feature) {
         if (strategy == "Border") {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0)]
               }
               
            }
            regdata <- regdata[complete.cases(regdata)]
            bad_borders <- regdata[, uniqueN(dma_code), 
                                   border_id][V1<2, border_id]
            regdata <- regdata[!(border_id %in% bad_borders)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, Border_Quarter := .GRP, .(border_id, Quarter)]
            regdata[, Border_Month := .GRP, .(border_id, Month)]
            regdata[, Border_DMA := .GRP, .(border_id, dma_code)]
            regdata[, Border_Week := .GRP, .(border_id, week_end)]
            regdata[, WOY := pmin(week(week_end), 52)]
            
         } else {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, feature_0, display_0)]	
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0)]
               }
               
               
            }
            regdata <- regdata[complete.cases(regdata)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, WOY := pmin(week(week_end), 52)]
         }
         
         formula <- Formula.DT[Measure == measure & 
                                  Goodwill == goodwill & 
                                  Feature == feature &
                                  Maxcomp == "0",
                               Formula]
         
         formula <- paste0(formula,
                           fixed_effect, 
                           "|0|",
                           clustering)
         
         # cat(formula, "\n")
         
         fit <- NULL
         try(fit <- felm(as.formula(formula), regdata, exactDOF=TRUE))
         if (is.null(fit)) {
            fit <- felm(as.formula(formula), regdata)
         }
         
         resid_ls <- as.data.table(resid(fit))
         resid_ls[, resid := ls_GRP_13]
         resid_ls[, ls_GRP_13 := NULL]
         resid_ls[, abs_resid := abs(resid)]
         mean_resid_ls = mean(resid_ls[, resid])
         mean_abs_resid_ls = mean(resid_ls[, abs_resid])
         
         mean_ad = mean(regdata[, ls_GRP_13])
         
         fraction = resid_ls[abs_resid >= x * mean_ad, .N] / resid_ls[,.N]
         
         resid_variation = sd(resid_ls[, resid]) / mean_resid_ls
         
         return(data.table(delta = delta,
                           coef_var = sd(resid_ls[, resid]) / mean_ad,
                           resid_var = resid_variation,
                           mean_resid = mean_resid_ls,
                           mean_abs_resid = mean_abs_resid_ls,
                           mean_ad = mean_ad,
                           result  = mean_abs_resid_ls/mean_ad,
                           result1 = fraction))
      } 
      
      Run.C1.Regressions <- function(strategy, 
                                     feature) {
         if (strategy == "Border") {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1, 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1, 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1)]
               }
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C1 Regressions for", delta, "\n")
               return(data.table())
            }
            bad_borders <- regdata[, uniqueN(dma_code), 
                                   border_id][V1<2, border_id]
            regdata <- regdata[!(border_id %in% bad_borders)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, Border_Quarter := .GRP, .(border_id, Quarter)]
            regdata[, Border_Month := .GRP, .(border_id, Month)]
            regdata[, Border_DMA := .GRP, .(border_id, dma_code)]
            regdata[, Border_Week := .GRP, .(border_id, week_end)]
            regdata[, WOY := pmin(week(week_end), 52)]
            
         } else {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1)]
               }
               
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C1 Regressions for", delta, "\n")
               return(data.table())
            }
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, WOY := pmin(week(week_end), 52)]
         }
         
         formula <- Formula.DT[Measure == measure & 
                                  Goodwill == goodwill & 
                                  Feature == feature &
                                  Maxcomp == "1",
                               Formula]
         
         formula <- paste0(formula,
                           fixed_effect, 
                           "|0|",
                           clustering)
         
         # cat(formula, "\n")
         
         fit <- NULL
         try(fit <- felm(as.formula(formula), regdata, exactDOF=TRUE))
         if (is.null(fit)) {
            fit <- felm(as.formula(formula), regdata)
         }
         
         resid_ls <- as.data.table(resid(fit))
         resid_ls[, resid := ls_GRP_13]
         resid_ls[, ls_GRP_13 := NULL]
         resid_ls[, abs_resid := abs(resid)]
         mean_resid_ls = mean(resid_ls[, resid])
         mean_abs_resid_ls = mean(resid_ls[, abs_resid])
         
         mean_ad = mean(regdata[, ls_GRP_13])
         
         fraction = resid_ls[abs_resid >= x * mean_ad, .N] / resid_ls[,.N]
         
         resid_variation = sd(resid_ls[, resid]) / mean_resid_ls
         
         return(data.table(delta = delta,
                           coef_var = sd(resid_ls[, resid]) / mean_ad,
                           resid_var = resid_variation,
                           mean_resid = mean_resid_ls,
                           mean_abs_resid = mean_abs_resid_ls,
                           mean_ad = mean_ad,
                           result  = mean_abs_resid_ls/mean_ad,
                           result1 = fraction))
      } 
      
      Run.C2.Regressions <- function(strategy, 
                                     feature) {
         if (strategy == "Border") {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,  
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,  
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2)]
               }
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C2 Regressions for", delta, "\n")
               return(data.table())
            }
            bad_borders <- regdata[, uniqueN(dma_code), 
                                   border_id][V1<2, border_id]
            regdata <- regdata[!(border_id %in% bad_borders)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, Border_Quarter := .GRP, .(border_id, Quarter)]
            regdata[, Border_Month := .GRP, .(border_id, Month)]
            regdata[, Border_DMA := .GRP, .(border_id, dma_code)]
            regdata[, Border_Week := .GRP, .(border_id, week_end)]
            regdata[, WOY := pmin(week(week_end), 52)]
            
         } else {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2)]
               }
               
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C2 Regressions for", delta, "\n")
               return(data.table())
            }
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, WOY := pmin(week(week_end), 52)]
         }	
         
         formula <- Formula.DT[Measure == measure & 
                                  Goodwill == goodwill & 
                                  Feature == feature &
                                  Maxcomp == "2",
                               Formula]
         
         formula <- paste0(formula,
                           fixed_effect, 
                           "|0|",
                           clustering)
         
         # cat(formula, "\n")
         
         fit <- NULL
         try(fit <- felm(as.formula(formula), regdata, exactDOF=TRUE))
         if (is.null(fit)) {
            fit <- felm(as.formula(formula), regdata)
         }
         
         resid_ls <- as.data.table(resid(fit))
         resid_ls[, resid := ls_GRP_13]
         resid_ls[, ls_GRP_13 := NULL]
         resid_ls[, abs_resid := abs(resid)]
         mean_resid_ls = mean(resid_ls[, resid])
         mean_abs_resid_ls = mean(resid_ls[, abs_resid])
         
         mean_ad = mean(regdata[, ls_GRP_13])
         
         fraction = resid_ls[abs_resid >= x * mean_ad, .N] / resid_ls[,.N]
         
         resid_variation = sd(resid_ls[, resid]) / mean_resid_ls
         
         return(data.table(delta = delta,
                           coef_var = sd(resid_ls[, resid]) / mean_ad,
                           resid_var = resid_variation,
                           mean_resid = mean_resid_ls,
                           mean_abs_resid = mean_abs_resid_ls,
                           mean_ad = mean_ad,
                           result  = mean_abs_resid_ls/mean_ad,
                           result1 = fraction))
      } 
      
      Run.C3.Regressions <- function(strategy, 
                                     feature) {
         if (strategy == "Border") {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    price_3=log(price_3), promo_3, 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    price_3=log(price_3), promo_3, 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,
                                    price_3=log(price_3), promo_3)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    border_id, week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,
                                    price_3=log(price_3), promo_3)]
               }
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C3 Regressions for", delta, "\n")
               return(data.table())
            }
            bad_borders <- regdata[, uniqueN(dma_code), 
                                   border_id][V1<2, border_id]
            regdata <- regdata[!(border_id %in% bad_borders)]
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, Border_Quarter := .GRP, .(border_id, Quarter)]
            regdata[, Border_Month := .GRP, .(border_id, Month)]
            regdata[, Border_DMA := .GRP, .(border_id, dma_code)]
            regdata[, Border_Week := .GRP, .(border_id, week_end)]
            regdata[, WOY := pmin(week(week_end), 52)]
            
         } else {
            if (feature == "Yes") {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    price_3=log(price_3), promo_3, 
                                    feature_0, display_0)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0), 
                                    price_0=log(price_0), 
                                    promo_0, 
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2, 
                                    price_3=log(price_3), promo_3, 
                                    feature_0, display_0)]
               }
               
            } else {
               if (level == "Market") {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,
                                    price_3=log(price_3), promo_3)]
               } else {
                  regdata <- move[week_end > date("2011-01-01"), 
                                  .(store_code_uc, dma_code, 
                                    week_end,
                                    units_0=log(1+units_0),
                                    price_0=log(price_0), promo_0,
                                    price_1=log(price_1), promo_1,
                                    price_2=log(price_2), promo_2,
                                    price_3=log(price_3), promo_3)]
               }
               
               
            }
            regdata <- regdata[complete.cases(regdata)]
            if(nrow(regdata) < 1000){
               cat("Skipping C2 Regressions for", delta, "\n")
               return(data.table())
            }
            
            regdata <- merge(regdata, ad, by=c("dma_code", "week_end"), allow.cartesian=TRUE)
            
            regdata[, Quarter := floor_date(week_end, "quarter")]
            regdata[, Month := floor_date(week_end, "month")]
            regdata[, WOY := pmin(week(week_end), 52)]
         }
         
         formula <- Formula.DT[Measure == measure & 
                                  Goodwill == goodwill & 
                                  Feature == feature &
                                  Maxcomp == "3",
                               Formula]
         
         formula <- paste0(formula,
                           fixed_effect, 
                           "|0|",
                           clustering)
         
         # cat(formula, "\n")
         
         fit <- NULL
         try(fit <- felm(as.formula(formula), regdata, exactDOF=TRUE))
         if (is.null(fit)) {
            fit <- felm(as.formula(formula), regdata)
         }
         
         resid_ls <- as.data.table(resid(fit))
         resid_ls[, resid := ls_GRP_13]
         resid_ls[, ls_GRP_13 := NULL]
         resid_ls[, abs_resid := abs(resid)]
         mean_resid_ls = mean(resid_ls[, resid])
         mean_abs_resid_ls = mean(resid_ls[, abs_resid])
         
         mean_ad = mean(regdata[, ls_GRP_13])
         
         fraction = resid_ls[abs_resid >= x * mean_ad, .N] / resid_ls[,.N]
         
         resid_variation = sd(resid_ls[, resid]) / mean_resid_ls
         
         return(data.table(delta = delta,
                           coef_var = sd(resid_ls[, resid]) / mean_ad,
                           resid_var = resid_variation,
                           mean_resid = mean_resid_ls,
                           mean_abs_resid = mean_abs_resid_ls,
                           mean_ad = mean_ad,
                           result  = mean_abs_resid_ls/mean_ad,
                           result1 = fraction))	
      }
      
      # 9.2.5: C3
      t3 <- NULL
      try(suppressWarnings(t3 <- Run.C3.Regressions(strategy, feature)))
      if(is.null(t3) || nrow(t3) == 0){
         cat("FATAL WARNING: C3 Failure at", module, brand, delta, "\n")
         
         # 9.2.4: C2
         t2 <- NULL
         try(suppressWarnings(t2 <- Run.C2.Regressions(strategy, feature)))
         if(is.null(t2) || nrow(t2) == 0){
            cat("FATAL WARNING: C2 Failure at", module, brand, delta, "\n")
            
            # 9.2.3: C1
            t1 <- NULL
            try(suppressWarnings(t1 <- Run.C1.Regressions(strategy, feature)))
            if(is.null(t1) || nrow(t1) == 0){
               cat("FATAL WARNING: C1 Failure at", module, brand, delta, "\n")
               
               # 9.2.2: C0
               t0 <- NULL
               try(suppressWarnings(t0 <- Run.C0.Regressions(strategy, feature)))
               if(is.null(t0) || nrow(t0) == 0){
                  cat("FATAL WARNING: C0 Failure at", module, brand, delta, "\n")
                  
                  # 9.2.1: Barebone
                  tb <- NULL
                  try(suppressWarnings(tb <- Run.Barebone.Regressions(strategy, feature)))
                  if(is.null(tb)){
                     cat("FATAL WARNING: Barebone Failure at", module, brand, delta, "\n")
                  } else {
                     tb[, Maxcomp := "-1"]
                     master[[i]] <- tb
                  }
                  
               } else {
                  t0[, Maxcomp := "0"]
                  master[[i]] <- t0
               }
               
            } else {
               t1[, Maxcomp := "1"]
               master[[i]] <- t1
            }
            
         } else {
            t2[, Maxcomp := "2"]
            master[[i]] <- t2
         }
         
      } else {
         
         t3[, Maxcomp := "3"]
         
         master[[i]] <- t3
      }
   }
   master <- rbindlist(master, fill=TRUE)
   
   # 9.3: Adding necessary information
   master[, new_module_code := module]
   master[, new_brand_code_uc := brand]
   
   master[, Strategy := strategy]
   master[, Level    := level]
   master[, Goodwill := goodwill]
   master[, Feature  := feature]
   
   cat("----------COMBINED PROCESSOR ENDS FOR", module, brand, "----------\n")
   return(master)
}

Ad.Generalizability.Regression <- function(output_dir,
                                           strategy = c("Border",
                                                        "Regular",
                                                        "Naive",
                                                        "MarketFE-only",
                                                        "MarketFE-WOY",
                                                        "MarketFE+WOY+National",
                                                        "MarketFE+WOY+National^2",
                                                        "MarketFE+WOY+National^3",
                                                        "MarketFE+WOY+National^4"),
                                           delta_list,
                                           fixed_effect,
                                           clustering,
                                           level = c("Store", "Market"),
                                           time  = c("Month", "Quarter", 
                                                     "None", "Week"),
                                           measure = c("GRP",
                                                       "Duration"),
                                           feature = c("Yes",
                                                       "No"),
                                           goodwill = c("logsum",
                                                        "sumlog"),
                                           maxcomp  = c("3",
                                                        "2",
                                                        "1",
                                                        "0",
                                                        "Barebone"),
                                           maxloss  = c("0.10", 
                                                        "0.20", 
                                                        "0.30", 
                                                        "0.40", 
                                                        "0.50")) {
   universe <<- fread(paste0(home_dir, 
                             "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
   universe <<- universe[, .(new_module_code, new_brand_code_uc, 
                             adcomp_code, comp1_code, 
                             comp2_code, comp3_code)]
   
   screening <<- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
   screening <<- screening[fracpos > 0.05 & cmean > 10]
   universe <<- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))
   
   case_table <<- universe[, .(new_module_code, new_brand_code_uc)]
   ncases <<- nrow(case_table)
   seed <<- as.integer(as.integer64(as.numeric(Sys.time())*10000000) %% 1000000)
   set.seed(seed)
   Sys.sleep(runif(1)*10)
   case_table <<- case_table[sample(ncases, ncases)]
   
   load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))
   if (strategy == "Border") {
      if (level == "Store") {
         stores <<- border.stores[cumshare<=0.9, .(store_code_uc, dma_code, border_id)]
      } else {
         stores <<- border.stores[cumshare<=0.9, .(store_code_uc, dma_code, border_id, total_revenue)]
      }
   } else {
      if (level == "Store") {
         stores <<- use.stores[cumshare<=0.9, .(store_code_uc, dma_code)]
      } else {
         stores <<- use.stores[cumshare<=0.9, .(store_code_uc, dma_code, total_revenue)]
      }
   }
   
   Nslots <- as.numeric(Sys.getenv("SLURM_CPUS_ON_NODE"))
   cat(as.character(Nslots), "\n")
   cl <- makeCluster(Nslots, outfile = "")
   registerDoParallel(cl)
   
   OUTPUT_DIR <<- output_dir
   STRATEGY   <<- strategy
   DELTA_LS   <<- delta_list
   FIXED_E    <<- fixed_effect
   CLUSTER    <<- clustering
   LEVEL      <<- level
   TIME       <<- time
   MEASURE    <<- measure
   FEATURE    <<- feature
   GOODWILL   <<- goodwill
   MAXCOMP    <<- maxcomp
   
   clusterExport(cl=cl,
                 varlist = ls(.GlobalEnv))
   
   foreach(i=1:ncases,
           .packages=c("data.table",
                       "bit64",
                       "lfe",
                       "RcppRoll",
                       "lubridate",
                       "Hmisc")) %dopar% {
                          module <- case_table[i, new_module_code]
                          brand <- case_table[i, new_brand_code_uc]
                          outfile <- paste0(OUTPUT_DIR, module, "-", brand, ".rds")
                          
                          if(file.exists(outfile)){
                             a <- readRDS(outfile)
                             if(!is.null(a)){
                                cat("SKIPPING brand", brand, "in module", module, "\n")
                                next
                             }
                          } else{
                             cat("PROCESSING brand", brand, "in module", module, "\n")
                             placeholder <- NULL
                             saveRDS(placeholder, outfile)
                          }
                          
                          aggregated <- NULL
                          try(aggregated <- Run.Single.Regression(module = module, 
                                                                  brand = brand,
                                                                  store = stores,
                                                                  universe = universe,
                                                                  strategy = STRATEGY,
                                                                  delta_list = DELTA_LS,
                                                                  fixed_effect = FIXED_E,
                                                                  clustering = CLUSTER,
                                                                  level = LEVEL,
                                                                  time = TIME,
                                                                  measure = MEASURE,
                                                                  feature = FEATURE,
                                                                  goodwill = GOODWILL,
                                                                  maxcomp = MAXCOMP))
                          if(is.null(aggregated)){
                             print("Remove failed file")
                             file.remove(outfile)
                          } else{
                             saveRDS(aggregated, outfile)
                          } 
                       }
   
}


