# ------------------------------------ #
# Run Estimations for Border Strategy
# 
#
# Estimation Parameters: 
# Strategy: Border
# Delta: seq(0,1,0.05)
# Custering: dma_code + week_end
# level: Store, Market
# Time: Month, Quarter, Week
# Measure: GRP
# Feature: No, Yes
# Goodwill: logsum, sumlog
# Maxcomp:  3
# Maxloss: 0.2
# ------------------------------------ #

# Call regression functions located in the same directory
script.dir <- dirname(sys.frame(1)$ofile)
source(paste0(script.dir, "/Regression-Functions.R"))

# Output directory, change accordingly
output.dir <- "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Full-Regression-Results/Regression-wo-anomaly/"

strategy      = "Border"
delta_ls      = seq(0, 1, 0.05)
cluster       = "Border_DMA+week_end"
level_sets    = c("Store", "Market")
time_sets     = c("Month", "Quarter", "Week")
measure       = "GRP"
feature_sets  = c("No", "Yes")
goodwill_sets = c("logsum", "sumlog")
maxcomp       = "3"
maxloss       = "0.20"

for (feature in feature_sets) {
	for (level in level_sets) {
		for (goodwill in goodwill_sets) {
			for (time in time_sets) {
			  
			  if (level == "Store") {
			    fe = paste0("|store_code_uc+Border_",
			                time,
			                "+WOY")
			  } else {
			    fe = paste0("|Border_DMA+Border_",
			                time,
			                "+WOY")
			  }
			  
			  if (time == "Week") {
			    cluster = "Border_DMA"
			  }
			  
			  output_location = paste0(output.dir,
			                           strategy,
			                           "-",
			                           level,
			                           "-",
			                           feature,
			                           "-",
			                           goodwill,
			                           "-",
			                           cluster,
			                           "-",
			                           time,
			                           "/")
				
				if (!dir.exists(file.path(output_location))) {
					dir.create(file.path(output_location),
					           recursive = TRUE,
							   showWarnings = FALSE)
					Ad.Generalizability.Regression(output_dir = output_location,
							   				   strategy   = strategy,
							   				   delta_list = delta_ls,
							   				   fixed_effect = fe,
							   				   clustering = cluster,
							   				   level = level,
							   				   time  = time,
							   				   measure = measure,
							   				   feature = feature,
							   				   goodwill = goodwill,
							   				   maxcomp = maxcomp,
							   				   maxloss = maxloss)
				} else {
					files = list.files(file.path(output_location))
					if (length(files) == 288) {
						missing_files = Check.Missing.Files(files,
															output_location)

						if (length(missing_files) == 0) {
							print("No Missing Files!")
							Collect.Results(input_location = output_location,
											strategy,
											level,
											feature,
											goodwill,
											cluster,
											time,
											measure)
							
							print("Collect Done!")

						} else {
							print(length(missing_files))
							print("Missing Files spotted.")

							Regression.Missing.Modules(output_dir = output_location,
											   missing_files = missing_files,
							   				   strategy   = strategy,
							   				   delta_list = delta_ls,
							   				   fixed_effect = fe,
							   				   clustering = cluster,
							   				   level = level,
							   				   time  = time,
							   				   measure = measure,
							   				   feature = feature,
							   				   goodwill = goodwill,
							   				   maxcomp = maxcomp,
							   				   maxloss = maxloss)
						}
					} else {
						Ad.Generalizability.Regression(output_dir = output_location,
							   				   strategy   = strategy,
							   				   delta_list = delta_ls,
							   				   fixed_effect = fe,
							   				   clustering = cluster,
							   				   level = level,
							   				   time  = time,
							   				   measure = measure,
							   				   feature = feature,
							   				   goodwill = goodwill,
							   				   maxcomp = maxcomp,
							   				   maxloss = maxloss)
					}
				}
			}
		}
	}
}


cat("Done.\n")
