# -------------------------------
# Run Esitmation and Elasticities
# 
# -------------------------------
rm(list = ls())
library(data.table)
library(bit64)
library(gridExtra)
library(ggplot2)
library(glmnet)
library(splines2)

home_dir <- "/project/hitsch_shapiro_ads/Advertising-Generalizability/"
ad_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel-wo-Anomaly/"
rms_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"

input_dir = "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/RNR-edits/Splines/Regular-Store-No-splines-dma_code+week_end-Month-GRP/"
resid_input_dir = paste0(input_dir, "Resid/")
adstk_input_dir = paste0(input_dir, "Adstock/")

knots_quantile = seq(0.1, 0.9, 0.1)

Predict.Q <- function(module,
                      brand,
                      lambda = "lambda.min",
                      knots_quantile,
                      splines_degree = 3) {
  set.seed(splines_degree)
  
  resid.DT <- readRDS(paste0(resid_input_dir, module,"-",brand,".rds"))
  newAdstock <- readRDS(paste0(adstk_input_dir, module, "-", brand, ".rds"))
  
  columns <- names(newAdstock)[grepl("spline|sum_GRP_13|ls_GRP_13", names(newAdstock))]
  
  resid.fit <- cv.glmnet(x = as.matrix(resid.DT[, columns, with = F]),
                         y = resid.DT$units_0)
  fit.coef <- coef(resid.fit, s = lambda)
  
  fit.predict <- predict(resid.fit, 
                         as.matrix(newAdstock[, columns, with = F]),
                         s = lambda)
  
  Q_hat.DT <- data.table(
    Adstock = newAdstock$Adstock,
    Q_hat   = as.vector(fit.predict)
  )
  
  Q_hat.DT[, `:=` (new_module_code = module,
                  new_brand_code_uc = brand)]
  
  return(Q_hat.DT)
}

Get.Knots <- function(module,
                      brand) {
  newAdstock <- readRDS(paste0(adstk_input_dir, module, "-", brand, ".rds"))
  
  knots.DT <- newAdstock[quantile %in% seq(0.1, 0.9, 0.1) | quantile == "Mean",
                         .(quantile, Adstock)]
  
  knots.DT[, `:=` (new_module_code = module,
                   new_brand_code_uc = brand)]
  
  return(knots.DT)
}

universe <- fread(paste0(home_dir, 
                         "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, 
                         adcomp_code, comp1_code, 
                         comp2_code, comp3_code)]

screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe  <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))

case_table <- universe[, .(new_module_code, new_brand_code_uc)]
ncases <- nrow(case_table)

for (i in 1:ncases) {
  cat(i, "\n")
  Q_hat.DT <- Predict.Q(module = case_table[i, new_module_code],
                        brand  = case_table[i, new_brand_code_uc],
                        knots_quantile = knots_quantile)
  
  saveRDS(Q_hat.DT,
          file = paste0("/project/hitsch_shapiro_ads",
                        "/scripts/Workspace-Ningyin/RNR-edits",
                        "/Splines/Regular-Store-No-splines-dma_code+week_end-Month-GRP",
                        "/Predict_Q/",
                        case_table[i, new_module_code],
                        "-",
                        case_table[i, new_brand_code_uc],
                       ".rds"))
  
}

knots.DT <- data.table()
for (i in 1:ncases) {
  cat(i, "\n")
  sub.knots.DT <- Get.Knots(module = case_table[i, new_module_code],
                            brand  = case_table[i, new_brand_code_uc])

  knots.DT <- rbind(knots.DT, sub.knots.DT)  
  
}

saveRDS(knots.DT,
        file = "./Processed-Data/cvspline_knots.rds")
