# ---------------------------------------- #
# Cost Calculation
# 
# ---------------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lfe)
library(RcppRoll)
library(lubridate)
library(Hmisc)
library(doParallel)

# Call the cost calculation functions located in the same directory
script.dir <- dirname(sys.frame(1)$ofile)
source(paste0(script.dir, "/Ad-Cost.R"))

# Output directory, change appropriately
output.dir <- "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/Brand-Level-ROI-Robust-woanomaly/"

Collect.Sub.Cost <- function(output_dir,
                             module,
                             brand,
                             cost_range = seq(0.8, 1.1, 0.1)) {

  savefile = paste0(output_dir,
                    module,
                    "-",
                    brand,
                    ".rds")
  
  if (file.exists(savefile)) {
    return(NULL)
  } else {
    Cost.DT = Cost.Calculation(module = module,
                               brand = brand,
                               cost_range = cost_range)
  
    saveRDS(Cost.DT,
            file = savefile)
  }

}

Nslots <- as.numeric(Sys.getenv("SLURM_CPUS_ON_NODE"))
cat(as.character(Nslots), "\n")

cl <- makeCluster(Nslots, outfile = "")

registerDoParallel(cl)

clusterExport(cl=cl,
              varlist = ls(.GlobalEnv))

universe <- fread(paste0(home_dir, 
                         "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, 
                         adcomp_code, comp1_code, 
                         comp2_code, comp3_code)]

screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe  <- merge(universe, 
                   screening[, .(new_module_code, new_brand_code_uc)], 
                   by=c("new_module_code", "new_brand_code_uc"))

case_table <- universe[, .(new_module_code, new_brand_code_uc)]
ncases     <- nrow(case_table)

foreach(i = 1:ncases,
        .packages = c("data.table",
                      "bit64",
                      "lfe",
                      "RcppRoll",
                      "lubridate",
                      "Hmisc")) %dopar% {
                        Collect.Sub.Cost(output_dir = output_dir,
                                         module = case_table[i, new_module_code],
                                         brand = case_table[i, new_brand_code_uc])
                      }

stopCluster(cl)
