# ---------------------------------------- #
# ROI Calculation - SE
# 
# ---------------------------------------- #
rm(list = ls())
library(data.table)
library(bit64)
library(lfe)
library(RcppRoll)
library(lubridate)
library(Hmisc)
library(doParallel)

home_dir <- "/project/hitsch_shapiro_ads/Advertising-Generalizability/"
ad_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel-wo-Anomaly/"
rms_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"

universe_dir <- "~/Workspace-Ningyin/overall-ROI/input/"

roi_dir <- "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/base-Brand-Level-ROI-avg-overall-observed/"

N_LAGS <- 52
WALMART_CODE <- 6920
CUMSHARE_CUTOFF <- 0.9

Calculate.Goodwill <- function(module,
                               brand,
                               delta) {
  # cat("Loading Ad-Intel Data...")
  
  # store data
  universe <- fread(paste0(home_dir, 
                           "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
  universe <- universe[, .(new_module_code, new_brand_code_uc, 
                           adcomp_code, comp1_code, 
                           comp2_code, comp3_code)]
  
  screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
  screening <- screening[fracpos > 0.05 & cmean > 10]
  universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))
  
  load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))
  
  store <- use.stores[cumshare<=CUMSHARE_CUTOFF, .(store_code_uc, dma_code)]
  
  # 2.1 Own Brand
  infile <- paste0(ad_dir, module, "-", brand, ".rds")
  ad <- readRDS(infile)
  
  ad <- ad[!(Class %in% c("Ghost Clearance", 
                          "National Network", 
                          "National Syndicated"))]
  ad[match_tier %in% c(1,3), match_tier := 13]
  ad[match_tier %in% c(2,4), match_tier := 24]
  ad[, Duration := Duration / 60]
  
  ad <- ad[, .(Duration=sum(Duration, na.rm=T), 
               GRP=sum(GRP, na.rm=T)), 
           .(dma_code, week_end, match_tier)]
  
  ad <- ad[year(week_end) <= 2014 
           & dma_code %in% unique(store$dma_code)]
  
  ad <- dcast(ad, 
              dma_code + week_end ~ match_tier, 
              value.var=c("GRP", "Duration"))
  
  start_week <- min(ad[, week_end])
  end_week   <- max(ad[, week_end])
  
  week_frame <- seq(date("2010-01-09"), date("2014-12-27"), 7)
  
  str <- data.table(expand.grid(dma_code = setdiff(unique(store$dma_code), c(740, 810, 630)),
                                week_end = week_frame))
  
  ad <- merge(str, ad, by=names(str), all.x=T)
  ad[is.na(ad)] <- 0
  if(!("GRP_13" %in% names(ad))){
    ad[, `:=`(GRP_13=0.0, Duration_13=0.0)]
  }
  if(!("GRP_24" %in% names(ad))){
    ad[, `:=`(GRP_24=0.0, Duration_24=0.0)]
  }
  
  # 2.2 Advertising Competitor
  adcomp <- universe[new_module_code==module & new_brand_code_uc==brand, adcomp_code]
  infile <- paste0(ad_dir, module, "-", adcomp, ".rds")
  if(!file.exists(infile)){
    ad1 <- NULL
  } else{
    ad1 <- readRDS(infile)
    ad1 <- ad1[!(Class %in% c("Ghost Clearance", 
                              "National Network", 
                              "National Syndicated"))]
    ad1[, Duration := Duration / 60]
    ad1 <- ad1[, .(Duration_Comp=sum(Duration, na.rm=T), GRP_Comp=sum(GRP, na.rm=T)), .(dma_code, week_end)]
    
    ad1 <- ad1[year(week_end) <= 2014 
               & dma_code %in% unique(store$dma_code)]
    
    str <- data.table(expand.grid(dma_code = setdiff(unique(store$dma_code), c(740, 810, 630)),
                                  week_end = week_frame))
    ad1 <- merge(str, ad1, by=names(str), all.x=T)
    ad1[is.na(ad1)] <- 0
  }
  
  # 2.3: Combine
  if(is.null(ad1)){
    ad[, `:=`(Duration_Comp=0.0, GRP_Comp=0.0)]
  } else{
    ad <- merge(ad, ad1, by=c("dma_code", "week_end"))
  }
  
  # cat("Done\n")
  # 
  # cat("Calculating Goodwill...")
  # 
  setkey(ad, dma_code, week_end)
  
  geom_weights <- sort(cumprod(c(1.0, rep(delta, times = N_LAGS))))
  
  ad[, sum_GRP_13  := roll_sum(GRP_13, 
                               n = N_LAGS + 1, 
                               weights = geom_weights,
                               normalize = FALSE, 
                               align = "right",  
                               fill = NA), dma_code]
  
  ad[, ls_GRP_24  := log(roll_sum(GRP_24, 
                                  n = N_LAGS+1, 
                                  weights = geom_weights,
                                  normalize = FALSE, 
                                  align = "right",  
                                  fill = NA)+1), dma_code]
  
  ad[, ls_GRP_Comp  := log(roll_sum(GRP_Comp, 
                                    n = N_LAGS+1, 
                                    weights = geom_weights,
                                    normalize = FALSE, 
                                    align = "right",  
                                    fill = NA)+1), dma_code]
  
  
  # ad <- ad[complete.cases(ad)]
  ad[, c("GRP_24", "GRP_Comp", "Duration_24", "Duration_Comp") := NULL]
  
  # cat("Done\n")
  return(ad)
}

Calculate.Q.Factor <- function(module,
                               brand,
                               timeframe) {
  
  # universe
  universe <- fread(paste0(home_dir, 
                           "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
  universe <- universe[, .(new_module_code, new_brand_code_uc, 
                           adcomp_code, comp1_code, 
                           comp2_code, comp3_code)]
  
  screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
  screening <- screening[fracpos > 0.05 & cmean > 10]
  universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))
  
  load(paste0(home_dir, 
              "4-Store-Panelist-Selection/Processed-Data/Selected-Panelists.RData"))
  
  # load HMS data
  if (timeframe == 2014) {
    load("/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/HMS_2014.RData")
  } else {
    purchases_temp <- data.table()
    
    for (y in timeframe) {
      load(paste0("/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/HMS_",
                  y,
                  ".RData"))
      
      purchases_temp <- rbind(purchases_temp, purchases_all)
      
      rm(purchases_all)
    }
    
  }
  
  hms <- purchases_temp[new_module_code == module
                        & new_brand_code_uc == brand]
  
  hms <- hms[, .(household_code, week_end,
                 retailer_code, volume, price,
                 panel_year)]
  
  keys <- c("dma_code", "week_end")
  hms <- merge(hms, use.hh, by=c("household_code", "panel_year"))
  str <- data.table(expand.grid(dma_code=unique(use.hh$dma_code),
                                week_end=seq(date("2010-01-09"), 
                                             date("2014-12-27"), 
                                             7)))
  
  # 1. General HMS
  agg1 <- hms[, .(units_wt=sum(volume*projection_factor, na.rm=T),
                  price_wt=wtd.mean(price, volume*projection_factor, na.rm=T)), 
              by=keys]
  
  # Fill Holws
  agg1 <- merge(str, agg1, by=keys, all.x=T)
  agg1[, class := "All Retailers"]
  agg1[is.na(units_wt), `:=`(units_wt=0)]
  agg1[is.na(price_wt), `:=`(price_wt=0)]
  
  hms_DT <- copy(agg1)
  
  rm(agg1)
  
  # Load RMS Quantity
  load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))
  stores <- use.stores[cumshare<=CUMSHARE_CUTOFF, .(store_code_uc, dma_code, total_revenue)]
  
  infile <- paste0(rms_dir, module, "/", brand, ".rds")
  move <- readRDS(infile)
  move[, base_price := NULL]
  
  move <- merge(move, stores, by="store_code_uc", allow.cartesian=TRUE)
  
  move_store <- copy(move)
  
  move <- move[, .(units_rms=sum(units, na.rm=T), 
                   price_rms=wtd.mean(price, total_revenue, 
                                      na.rm=T),
                   promo_rms=wtd.mean(promo, total_revenue, 
                                      na.rm=T)), 
               .(dma_code, week_end)]
  
  move[, brand_code := 0]
  
  move <- move[!is.na(units_rms)]
  
  move_DT <- move[year(week_end) %in% timeframe]
  
  rm(move)
  # Merge and Create Quantity Factor
  
  quantity_DT <- merge(hms_DT,
                       move_DT,
                       by = keys)
  
  quantity_DT[, roll_mean_hms_q := roll_meanl(units_wt,
                                              n = N_LAGS + 1,
                                              fill = NA),
              .(dma_code)]
  
  quantity_DT[, roll_mean_rms_q := roll_meanl(units_rms,
                                              n = N_LAGS + 1,
                                              fill = NA),
              .(dma_code)]
  
  quantity_DT[, factor := roll_mean_hms_q / roll_mean_rms_q]
  
  quantity_DT[factor == Inf | factor == -Inf, factor := NA]
  # mean_factor = mean(quantity_DT$factor, na.rm = T)
  
  return(list(quantity_DT,
              move_store))
}

Get.Baseline.Coef <- function(module,
                            brand) {
  # Coef
  coef_file = paste0("/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Full-Regression-Results/Regression-wo-anomaly/fe-Regular-Store-No-logsum-dma_code+week_end-Month/",
                     module,
                     "-",
                     brand,
                     ".rds")
  
  coef <- readRDS(coef_file)
  coef[Maxcomp == "Barebone", Maxcomp := "-1"]
  coef[, ncomp := as.numeric(Maxcomp)]
  
  obs <- coef[ncomp>=0, c("new_module_code", "new_brand_code_uc", "ncomp", "nobs"), with=FALSE]
  obs <- unique(obs)
  obs <- dcast(obs, new_module_code+new_brand_code_uc~ncomp, value.var="nobs")
  
  if (!("3" %in% names(obs))) {
    obs[, "3" := as.integer(NA)]
  }
  if (!("2" %in% names(obs))) {
    obs[, "2" := as.integer(NA)]
  }
  if (!("1" %in% names(obs))) {
    obs[, "1" := as.integer(NA)]
  }
  
  setnames(obs, as.character(0:3), paste0("n", 0:3))
  obs[, r1 := n1 / n0]
  obs[, r2 := n2 / n0]
  obs[, r3 := n3 / n0]
  obs <- obs[, .(new_module_code, new_brand_code_uc, r1, r2, r3)]
  
  retain <- 0.8
  
  cases <- copy(obs)
  cases[, nc := as.integer(NA)]
  cases[(r1 < retain | is.na(r1)), nc := 0]
  cases[r1 >= retain & (r2 < retain | is.na(r2)), nc := 1]
  cases[r2 >= retain & (r3 < retain | is.na(r3)), nc := 2]
  cases[r3 >= retain, nc := 3]
  cases <- cases[, .(new_module_code, new_brand_code_uc, nc)]
  
  stopifnot(cases[, .N] == 1)
  nc <- cases[, nc]
  
  coef <- coef[ncomp == nc & variable == "ls_GRP_13"]
  
  beta_mean = coef$coef
  beta_sd   = coef$se
  
  return(list(beta_mean, beta_sd))
}

ROI.SE.Calculation <- function(module,
                               brand,
                               delta     = 0.9,
                               margin_range,
                               cost_range,
                               timeframe = 2010:2014,
                               RMS       = TRUE) {
  original_ROI_DT <- readRDS(paste0(roi_dir,
                                    module, 
                                    "-",
                                    brand,
                                    ".rds"))
  
  original_ROI_DT <- original_ROI_DT[Cost == 1 & (Margin == "0.3" | Margin == 0.3)]
  
  original_ROI <- original_ROI_DT[, ROI]
  
  baseline_coef <- Get.Baseline.Coef(module, brand)
  baseline_beta <- baseline_coef[[1]]
  baseline_beta_se <- baseline_coef[[2]]
  
  universe_DT <- readRDS(paste0(universe_dir, "universe_DT.rds"))
  
  quantity_res <- Calculate.Q.Factor(module,
                                     brand,
                                     timeframe)
  
  quantity_DT    <- quantity_res[[1]]
  observed_Q_DT  <- quantity_res[[2]]
  observed_Q_DT[, rev_observed := units * price]
  
  cat("quantity_DT Done \n")
  
  goodwill    <- Calculate.Goodwill(module,
                                    brand,
                                    delta)
  
  if (length(timeframe) == 1) {
    goodwill <- goodwill[year(week_end)  ==  timeframe]
  } else {
    goodwill <- goodwill[year(week_end) %in% timeframe]
  }
  
  # Shut-off Predicted Q
  goodwill_shutoff <- copy(goodwill)
  goodwill_shutoff[GRP_13 > 0, GRP_13 := 0]
  geom_weights <- sort(cumprod(c(1.0, rep(delta, times = N_LAGS))))
  
  goodwill_shutoff[, sum_GRP_13 := roll_sum(GRP_13, 
                                            n = N_LAGS + 1, 
                                            weights = geom_weights,
                                            normalize = FALSE, 
                                            align = "right",  
                                            fill = NA), dma_code]
  
  observed_Q_DT <- observed_Q_DT[!(dma_code %in% c(740, 810, 630)) & week_end != "2010-01-02"]
  lambda_DT <- merge(observed_Q_DT,
                     goodwill,
                     by = c("dma_code", "week_end"))
  
  stopifnot(observed_Q_DT[, .N] == lambda_DT[, .N])
  
  lambda_DT <- merge(lambda_DT,
                     goodwill_shutoff,
                     by = c("dma_code", "week_end"),
                     suffixes = c("_observed", "_shutoff"))
  
  stopifnot(observed_Q_DT[, .N] == lambda_DT[, .N])
  
  lambda_DT <- lambda_DT[week_end > "2011-01-01"]
  
  lambda_DT[, lambda_numerator := 1 + sum_GRP_13_shutoff]
  lambda_DT[, lambda_denom     := 1 + sum_GRP_13_observed]
  lambda_DT[, lambda := (lambda_numerator / lambda_denom) ^ baseline_beta]
  lambda_DT[, lnk    := log(lambda_numerator / lambda_denom)]
  
  stopifnot(lambda_DT[is.na(lambda), .N] == 0)
  
  # Prepare G_Beta DT
  G_Beta_DT <- merge(lambda_DT[, .(dma_code, week_end, rev_observed, lambda, lnk)],
                     quantity_DT[, .(dma_code, week_end, factor)],
                     by = c("dma_code", 
                            "week_end"),
                     all.x = T)
  
  cat(G_Beta_DT[, .N], "\n")
  
  stopifnot(length(intersect(unique(G_Beta_DT$dma_code), c(740, 810, 630))) == 0)
  
  cat("G_Beta_DT Done \n")
  
  G_Beta_results = data.table()
  
  for (margin in margin_range) {
    for (cost_control in cost_range) {
      G_Beta_DT[, Delta_pi     := lambda * lnk * rev_observed * factor * margin]

      national_G_Beta_DT   <- G_Beta_DT[, .(Delta_pi = sum(Delta_pi,
                                                     na.rm = T)),
                                  .(week_end)]
      
      Delta_pi <- sum(national_G_Beta_DT[, Delta_pi], na.rm = T)
      
      cost.value <- original_ROI_DT$Cost.value * cost_control
      
      G_Beta_prime <- Delta_pi / cost.value
      
      # stopifnot(!is.na(G_Beta_prime))
      
      sub.G_Beta.results <- data.table(
        Module = module,
        Brand  = brand,
        Margin = margin,
        Cost   = cost_control,
        Cost.value = cost.value,
        Beta   = baseline_beta,
        Beta_se = baseline_beta_se,
        G_Beta_prime = G_Beta_prime,
        ROI_se  = sqrt(G_Beta_prime^2) * baseline_beta_se
      )
      
      # print(sub.G_Beta.results)
      
      G_Beta_results = rbind(G_Beta_results, sub.G_Beta.results, fill = T)
    }
  }
  return(G_Beta_results)
}

