# ---------------------------------------- #
# Weekly ROI Calculation - breakeven ad effect
# 
# ---------------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lfe)
library(RcppRoll)
library(lubridate)
library(Hmisc)
library(doParallel)

# Call the ROI calculation functions located in the same directory
script.dir <- dirname(sys.frame(1)$ofile)
source(paste0(script.dir, "/Weekly-ROI-breakeven.R"))

Nslots <- as.numeric(Sys.getenv("SLURM_CPUS_ON_NODE"))
cat(as.character(Nslots), "\n")

Collect.Sub.ROI <- function(output_dir,
                            module,
                            brand,
                            margin_range = seq(0.15, 0.4, 0.05),
                            cost_range   = seq(0.8, 1.1, 0.1)) {
  
  savefile = paste0(output_dir,
                    module,
                    "-",
                    brand,
                    ".rds")
  
  if (file.exists(savefile)) {
    return(NULL)
  } else {
    ROI = ROI.Calculation(module = module,
                          brand = brand,
                          margin_range = margin_range,
                          cost_range = cost_range)
    
    saveRDS(ROI,
            file = savefile)
  }
  
  
}

cl <- makeCluster(Nslots, outfile = "")

registerDoParallel(cl)

clusterExport(cl=cl,
              varlist = ls(.GlobalEnv))

# change accordingly
output_dir = "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/base-Brand-Level-Average-ROI-breakeven-observed/"

if (!dir.exists(output_dir)) {
  dir.create(output_dir)
}


universe <- fread(paste0(home_dir, 
                         "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, 
                         adcomp_code, comp1_code, 
                         comp2_code, comp3_code)]

screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe <- merge(universe, 
                  screening[, .(new_module_code, new_brand_code_uc)], 
                  by=c("new_module_code", "new_brand_code_uc"))

case_table <- universe[, .(new_module_code, new_brand_code_uc)]
ncases <- nrow(case_table)

foreach(i = 1:ncases,
        .packages = c("data.table",
                      "bit64",
                      "lfe",
                      "RcppRoll",
                      "lubridate",
                      "Hmisc")) %dopar% {
                        Collect.Sub.ROI(output_dir = output_dir,
                                        module = case_table[i, new_module_code],
                                        brand = case_table[i, new_brand_code_uc])
                      }

stopCluster(cl)
