# Accessing the Data

All of the data used in this study were provided by Nielsen and accessed through the Kilts Center for Marketing at the University of Chicago Booth School of Business.  Interested researchers may register to gain access to the RMS scanner data, the HMS panel data and the Ad Intel Data online at https://www.chicagobooth.edu/research/kilts/datasets/nielsen. Upon approval, Kilts will provide instructions on how to access each of these data sets. Our code will run on these data files from 2010-2014.  For additional questions not addressed on the website, please contact marketingdata@chicagobooth.edu.

# Advertising-Generalizability

## Project Overview

The objective of this project is to estimate the effect of TV advertising on retail sales for a wide range of brands. To do that, we need the following data for each brand:

* Weekly volume, price, promotion, and feature/display at store or market level.
* Weekly advertising (GRP, duration, or spending) at market level.

We create the data we want in the following steps:

* (1.1) The ad occurrences and viewerships are separate in the raw Ad-Intel data. We need to merge them in order to find the GRP for each advertisement.
* (1.2) There are some discrepancies between the national and local records of Network TV ads. We need to resolve those discrepancies.
* (2) Ad-Intel and RMS use different brand definitions, so for each RMS brand, we need to find all the corresponding Ad-Intel brands.
* (3.1) RMS data comes in `UPC-Store-Week level`, so we need to aggregate it to `Brand-Store-Week` level.
* (3.2) Ad-Intel data comes in `{AdIntel Brand}-Market-Channel-Second` level, so we need to aggregate it to `{RMS Brand}-Market-Week` level.
* (4) We need to remove some small or problematic stores from the RMS data.

## (1) Build Ad-Intel Data

### General Concepts

#### Media Types

Ad-Intel covers 4 TV media types: Cable, Network, Syndicated, and Spot. 

* For Cable TV, ads are purchased at national level. Impressions are observed only nationally.
* For Network and Syndicated TV, ads are purchased at national level. The programs are broadcast at local TV stations. Impressions are observable locally through the Network Clearance and Syndicated Clearance impressions files.
	* The local TV stations are typically affiliated to a national network. For example, `WBZ` is the Boston affiliate of CBS. 
* For Spot TV, ads are purchased at DMA level. The programs are also broadcast at local TV stations. 

Since Network and Syndicated TV ads are purchased nationally but broadcast locally, the Ad-Intel record them in two ways:

* The `Network TV` and `Syndicated TV` occurrence files record them at national level.
	* i.e. the date and time each ad is *supposed* to be broadcast at every local station
* The `Network Clearance Spot TV` and `Syndicated Clearance Spot TV` occurrence files record them at local channel level.
	* i.e. the date and time each ad is *actually* broadcast at every local station
* It is important to emphasize that the Network (Syndicated) TV occurrences and the Network (Syndicated) Clearance Spot TV occurrences are meant to represent the exact same ad occurrences. The main distinction here is that the "clearance" occurrences are measured at the local DMA level, allowing for us to match them with local impressions data. They do not represent two distinct forms of advertising. For our purposes, the value derived from the Network (Syndicated) TV occurrences (measured nationally) is (1) to measure the estimated cost of the ad, which is purchased nationally, and (2) to audit the advertising occurrence schedule in the "clearance" data to detect times when a local station might override a nationally purchased ad and distinguish it from local measurement error. 

* To futher explain the previous point,The local channels have some authority to replace or move nationally scheduled ads, and it possible that the Nielsen local measurement devices data are alsoare not perfect. Hence there are discrepancies between those national and local files not every "Network TV" advertising occurrence will match perfectly with a "Network Clearance Spot TV" occurrence, even though they represent the same ad buys. Any incidence of "Network TV" occurrences which do not have associated "Network Clearance Spot TV" occurrences in a local market will be referred to as the "Missing Network Discrepancy."

#### Occurrence Data

The occurrence data provides detailed information for each advertisement, including:

* Date `[AdDate]`
* Time `[AdTime]`
* Media Type `[MediaTypeID]`
* Channel `[DistributorCode, DistributorID]`
* Market (can be national) `[MarketCode]`
* Primary, Secondary, and Tertiary Brands `[PrimBrandCode, ScndBrandCode, TerBrandCode]`
* Duration `[Duration]`
* The associated TV program `[NielsenProgramCode, TelecastNumber]`
* Other time-related info `[TVDayPartCode, DayOfWeek, TimeIntervalNumber]`

Note that Ad-Intel does not capture any local ad between 2AM and 5AM.

#### Impression (Viewership) Data

For the national media types (Cable, Network, and Syndicated), Ad Intel provides the estimated number of impressions for each TV program--defined as a pair of NielsenProgramCode and TelecastNumber.

For the local media types (Network Clearance, Syndicated Clearance, and Spot), Ad Intel provides the estimated impressions at {Local Station}-Month-{Day of Week}-{5 Minute Time Interval} level.

Note: There are only 25 markets (the "Local People Meter" markets) for which the local impressions are available in all months. In those markets, impressions are measured using set top boxes. For the rest of the markets, local impressions data are only available in four "sweeps months": February, May, July, and November and are measured by Nielsen households filling out diaries. Therefore, we need to impute the impressions for the non-sweeps months in non-LPM markets. Now we use an average between the two closest available months, weighted by the time difference. For example, for June we use 1/2 May + 1/2 July, and for March we use 2/3 February + 1/3 May. 

#### Universe Estimates

Ad-Intel also provides the estimated total number of TV audience at national and market level. Those universe estimates are updated yearly.  

### Build the Regular Parts

The logic of the regular build is very simple. For each media type in each month, we need to do the following:

1. Merge occurrences with impressions

(a) For national data (Cable TV), merge on NielsenProgramCode and TelecastNumber

(b) For local data (Spot TV, Network Clearance Spot TV, Syndicated Clearance Spot TV), merge on DistributorID, DayOfWeek, and TimeIntervalNumber

(c) Remember to do the imputation for non-LPM markets in non-sweep months.

2. Merge the result with universe estimates

3. Calculate the GRP as 100 * Impression / Universe for each ad occurrence

The script `Build-Regular.R` follows exactly those steps.

### Resolve the "Missing Network" Discrepancy

As mentioned previously, each Network ad that is actually shown on TV in a local market should be associated with a Network Clearance Spot TV ad. These represent the exact same showing of the exact same ads. For the most part, ads purchased as Network TV ads are shown "simultaneously" (in the same slot during the same television program) across the country. However, in rare circumstances, a Network TV ad may not actually run in one or more local markets. Additionally, in even rarer circumstances, a Network TV ad may run in a local market but not get recorded in the Network Clearance occurrences due to an error in the recording devices at the local level. Our goal in this part of the build is to distinguish the explanation when Network TV and Network Clearance Spot TV ads do not match in a given market at a given time.

We do this by reconstructing the TV schedule, down to the second. From the observed schedule, we infer whether or not it was a recording error by the local measurement device or whether the local station likely displaced the Network ad. If, for example, there is a "gap" in the schedule that is 30 seconds long, a 30 second long Network TV ad that purportedly ran in that time interval, we infer that the Network TV ad actually ran on the local station in that slot but the recording device failed in that moment. In these cases, we "insert" the Network TV ad occurrences into the advertising schedule into the "gap" where we inferred that it belonged. Alternatively, if there is no "gap" in the schedule (i.e. we observe ads back-to-back every second followed by programming), we then infer that the local station displaced the Network ad either for extended programming (e.g., local news alert, sporting event gone long) or for an additional locally purchased Spot TV ad. Network Clearance TV ads also are always missing between 2AM and 5AM, as local ratings are not recording during those time intervals.

The procedure of this part, however, is **much more complicated**. Basically, we need the following steps:

1. Find the information for each local station, including:
	* The market (`MarketCode`) and network (`Affiliation`) for each local station (`DistributorCode`). 
	* The `DistributorID` for each `DistributorCode`. 
		* This is in fact an one-to-one relationship, but we have to record that because the "Station Affiliation" data only has `DistributorCode`, while the impressions data only has `DistributorID`. 
	* *Corresponding scripts*: `1-Extract-Local-Distributors.R` and `2-Process-Station-Information.R`
2. For each network and each local station, we stack all the monthly data.
	* We cannot use the raw monthly data because the national and local files have different dates. 
	* Stacking also prevents errors at month boundaries. For example, a national ad at `2012/05/31 23:30:00` may be distributed locally at `2012/06/01 00:30:00`. This will not be captured if we process the data month-by-month.
	* *Corresponding scripts*: `3-Extract-Station-Level-Data.R`
3. For each local station, we find the "unexpectedly missing" occurrences. Please see the relevant code for details, but in short we categorize all the national ads as following:
	* A national ad is **directly matched** to the local data if its closest local occurrence has the same primary brand code. 
	* A national ad is **indirectly matched** to the local data if there's a local occurrence that is aired within some time limit before or after the scheduled air-time. 
		* This step accounts for the ads that are moved around. The time limit is 3 hours for ETZ/CTZ, 6 hours for MTZ, and 7 hours for PTZ.
	* A national ad is **replaced by another ad** if another spot / network clearance / syndicated clearance ad runs into its scheduled time slot.
	* A national ad is **not captured locally** if its scheduled air-time is between 2AM and 5AM. 
	* We mark all remaining national ads as **unexpectly missing** at this local station. These are the "gaps" described above.
	* *Corresponding scripts*: `4-Categorize-Stations.R` and `5-Run-Station-Level.R`
4. We get all the "unexpectly missing" occurrences at each station, and we re-organize them into monthly files. We then merge those monthly files with the monthly local impressions data. 
	* *Corresponsing scripts*: `6-Station-Level-To-Month-Level.R` and `7-Merge-Viewership-On-Monthly-Files.R`. 

**An important note:** We must be careful to account for the "broadcast delay" for mountain and pacific time zones.

* A nationally scheduled program or ad can be broadcast with a delay of 0/1/2/3 hours in pacific-time markets or 0/1 hours in mountain-time markets. This delay can be seemingly arbitrary. 
* In step 3, we say a national ad is "unexpectly missing" only if it's "unexpectly missing" under *all the possible delays*, i.e. 0/1 hour in MTZ and 0/1/2/3 hours in PTZ.
* In step 4, for PTZ/MTZ markets we average the impressions at the airtime and 3/1 hours after the airtime. 

## (2) String Matching and RMS Module/Flavor Aggregation

### General Concepts

**String matching** means matching the brands in RMS and Ad-Intel. We classify the matches in 4 "tiers" below. 

Please see Shapiro, Hitsch & Tuchman (2020) for more details ("Research Note: Procedure for Building Nielsen Ad Intel Data and Merging with Nielsen RMS Scanner Data")

In theory, tier-1 and tier-2 advertising should have a positive effect on sales, while the effect of tier-3 and tier-4 ads can be either positive or negative.

1. RMS and Ad-Intel brand names are identical.
2. Ad-Intel brand is more specific than the RMS brand.
	* Example: Ad-Intel brand `LAYS POTATO CHIPS CHICKEN AND WAFFLE` is a tier-2 match to RMS brand `LAY'S`.
3. Ad-Intel brand is more general than the RMS brand. 
	* Example: Ad-Intel brand `COCA-COLA SOFT DRINKS` is a tier-3 match to RMS brand `COCA-COLA R`.
4. Ad-Intel brand is an "associate" to the RMS brand.
	* Example: Ad-Intel brand `COCA-COLA ZERO DT` is a tier-4 match to RMS brand `COCA-COLA R`. 

**Module aggregation** means aggregating some too specific RMS modules together. For example, the RMS modules `NUTS-BAGS`, `NUTS-CANS`, `NUTS-JARS`, and `NUTS-UNSHELLED` are essentially the same thing, and advertisements never distinguish between them. 

**Flavor aggregation** means aggregating many sub-brands together. For example, the brand "Lean Cuisine Frozen Entree" has 50 sub-brands in RMS (e.g. `LEAN CUISINE ONE DISH FAVORITE` or `LEAN CUISINE SPA COLLECTION`). Aggregating them together makes the matching easier, and it creates more tier-3 matches and fewer tiers-2/4 matches.

### Folder Structure

#### Exported-Files/

`Universe-For-Estimation-180508.csv`: Detailed information about our final universe of 358 brands.

`Consolidated-UPC-Dictionary-For-Aggregation-180508.rds`: A revised version of the original `Products` meta-data that reflects all module and flavor aggregations. 

`All-Main-Matches-180508.csv`: String matches for the 358 estimation brands. 

`All-Comp-Matches-180508.csv`: String matches for the top competitors not included in the main universe.

#### Information-Files/ and Meta-Data/

`Information-Files/`: Contains intermediate files for brand matching and flavor aggregation.  

`Meta-Data/`: Meta data from RMS data build and version corrrection.  


## (3) Brand Aggregation

### Ad-Intel

The Ad-Intel data build comes at `{AdIntel Brand}-Channel-Time` level, and in the end we want to aggregate it to `{RMS Brand}-Market-Week` level. 

* To make things more flexible, in this section we only aggregate the ad data to `{AdIntel Brand}-{Media Type}-Market-Week` level and save the data for each RMS brand. The further aggregation is done on-the-fly in section 6. 
* The aggregation here only involves adding up `Duration` and `GRP`.
	* Some ad occurrences come with 2/3 brands, but those brands are mostly the same product (e.g. Snapple Black Tea and Snapple Green Tea, which we eventually combine to a single "brand" as per above). To avoid double-counting the ads, we use the following trick: if an occurrence has two/three brands, we treat it as two/three occurrences with half/one-third of the Duration and GRP. 

The corresponding script is `1-Ad-Intel.R`.

### RMS

The RMS data build comes at `UPC-Store-Week` level, and we want to aggregate it to `Brand-Store-Week` level. As mentioned before, UPCs are generally much more specific than brands, as they refleect many different sizes and presentations.. 

* One RMS brand may contain hundreds of UPCs with different sizes (`size1_amount`, say 12 OZ or 24 OZ) and different multi-pack status (`multi`, say 6-pack or 12-pack). 
	* Therefore, instead of using the `units` field in RMS data, we need calculate the volume in equivalency units: `volume := units * multi * size1_amount`. We adjust price accordingly.
* For each store-week, the brand-level variables are calculated as follows:
	* Volume: sum of UPC-level volumes
	* Price: weighted average of UPC-level prices. The weight for a UPC is its average weekly revenue in this store.
	* Promotion: weighted average of UPC-level promotion indicators (`price / base_price < 0.95`).
	* Feature/Display: weighted average of UPC-level feature/display indicators (remove missing values).

Corresponding scripts:

* `2-RMS-Extract-Data-From-All-Other.R` extracts small UPCs that are not stored individually in the RMS data build. 
* `3-RMS-Run-Brand-Aggregation.R` does the aggregation. It uses a pseudo-parallelization trick so that it can be submitted in parallel to different nodes.
* `4-RMS-Job-Submission.sh` submits script No.3 to different nodes on the Grid.


## (4) Select Stores and Panelists

This is a housekeeping step that deals with the inconsistencies between various location-related files. Note that `Panelist.RData` comes from Homescan data build in `0-Build-Nielsen-Data/`. The exported files are `Processed-Data/Selected-Stores.RData` and `Processed-Data/Selected-Panelists.RData`. 

### Remove Shifting FIPS
For the implementation of the border strategy, we use the Nielsen provided mapping between counties and DMAs. From this, we constructed a data set that flags the counties that lie on a border between DMAs. However, some counties change DMAs over time, since the borders are re-drawn periodically. Therefore, we removed all the counties that did not stay in a single DMA, and we removed the borders that were re-drawn.

### Select Usable Panelists 
We removed the panelists in the HMS data that switch between different FIPS-codes. 

### Select Usable Stores
We removed the stores in the RMS data that switch between different FIPS-codes, or stores that are not continuously covered in 2010-2014. We then rank the stores by the total 2010-2014 revenue (across all products), and find the stores that constitute 90% of total revenue. We will use those stores for all estimations.

## (5) Basic Descriptions

### Screening Based on Tiers 1+2 Advertising

* Scripts: `Grid-Scripts/Ad-Intel-Screening-and-Share-Analysis.R` and `Local-Scripts/Inspect-Shares.R` 
* Results: `Processed-Data/Screening.rds`
* Description: For each of the 358 RMS brands in our universe, calculate the fraction of market-weeks with positive tiers 1+2 GRP, and the mean tiers 1+2 GRP conditional on it being positive. 
	* We kick out 70 brands whose "positive fraction" is below 5%, or whose "positive mean" is below 10 points. 

### Share of Advertising by Media Type

* Scripts: `Grid-Scripts/Ad-Intel-Screening-and-Share-Analysis.R` and `Local-Scripts/Inspect-Shares.R`
* Results: `Processed-Data/Shares-By-Media-Type.rds`
* Description: For each brand, we calculate the share of GRP/Duration in each media type. 

### R2 of Advertising Against Fixed Effects

* Scripts: `Grid-Scripts/Ad-Intel-R2-Analysis.R` and `Local-Scripts/Inspect-R2.R`
* Results: `Processed-Data/Advertising-R2.rds`
* Description: For each brand, regress GRP/Duration on fixed effects and get the R2. The rationale is to see how much variation we are left with in the regressor after controlling for the FEs. The list of FEs is:
	* Store + Quarter/Month + Week of Year
	* Store + Border-[Quarter/Month] + Week of Year
	* DMA + Quarter/Month + Week of Year
	* Border-DMA + Border-[Quarter/Month] + Week of Year
	* Store + Week
	* Store + Border-Week
	* DMA + Week
	* Border-DMA + Border-Week

## (6) Basic Estimations

### Fixed Configurations

* Advertising Control: Own Tiers 1+2, Own Tiers 3+4, and Top Competitor (all tiers)
* Number of Lags: 52
* Stores: those constituting top 90% of total revenue

### Variable Configurations

1. Data Level: Store / Market
2. Strategy: Border / Regular
3. Own Feature-Display Control: Yes / No
4. Price-Promotion Control:
	* None
	* Own brand only
	* Up to 1 competitor (subject to the 80% revenue share limit)
	* Up to 2 competitors
	* Up to 3 competitors
5. Time FE: Quarter + Week-of-Year / Month + Week-of-Year
6. Advertising Measure: GRP / Duration
7. Goodwill Formula: Log of Sum / Sum of Log
8. Delta: `seq(0,1,0.01)`

Therefore, we need to run up to `2*2*2*5*2*2*2*101 = 32320` regressions for each brand, and that amounts to `9308160` regressions across all 288 brands. Note that we decided to try all the possible price/promo controls and impose the maximum-observation-loss restrictions *ex post*. 

### Basic Steps for Running A Regression

For a given brand, fixing parts (1), (2), and (3) above, we do the following:

1. Load RMS data, merge in DMA and border information, and do the proper aggregation.
2. Load Ad-Intel data and do the proper aggregation.
3. Loop through `delta = seq(0,1,0.01)` (at most 40 regressions for each delta):
	* Calculate goodwill (4 possible ways: log-sum GRP, log-sum Duration, sum-log GRP, sum-log Duration)
	* Loop through 5 kinds of price-promo controls:
		* Run the regression with quarter or month fixed effects.

## (7) Semi-parametric Model

### Estimating Advertising Elasticities Using Semi-parametric Estimation

* Scripts: `1-Baseline-Residualized.R` and `2-Baseline-Ad-Elasticities.R` 
* Source: `Regression-Functions-Residualized.R`
* Results: `Processed-Data/Elasticities.rds`
* Description: Refer to Appendix E. Flexible functional form for advertising response.
	
### Predicting Quantities from Semi-parametric Model

* Scripts: `3-Predict-Q.R` and `4-Collect-Predicted-Q.R` 
* Results: `Processed-Data/cvspline_knots.rds` and  `Processed-Data/cvspline_Q_hat.rds`
* Description: Refer to Appendix E. Flexible functional form for advertising response.

## (8) ROI Analyses

### Advertising Cost  

* Scripts: `Ad-Cost/0-Ad-Cost.R`
* Source: `Ad-Cost/Ad-Cost.R`
* Results: `Processed-Data/Elasticities.rds`
* Description: Refer to `Data-Build-Document.pdf`, section 2 Advertising Cost.

### Estimating Weekly Advertising ROI

#### ROI calculation

* Scripts: `Weekly/1-Weekly-ROI.R` and `Weekly/2-Weekly-ROI-se.R`
* Source: `Weekly/Weekly-ROI.R` and `Weekly/Weekly-ROI-se.R`
* Results: `Weekly/Processed-Data/base-average-ROI-observed.RData` and `Weekly/Processed-Data/base-average-std-err-observed.RData`
* Description: Refer to Appendix G. ROI calculation.

#### Break-even ad effects

* Scripts: `Weekly/3-Weekly-ROI-breakeven.R`
* Source: `Weekly/Weekly-ROI-breakeven.R`
* Results: `Weekly/Processed-Data/average-breakeven-observed.RData`
* Description: Refer to Appendix G.3. Break-even ad effects.
	
### Estimating Overall Advertising ROI

#### ROI calculation

* Scripts: `Overall/1-Overall-ROI.R` and `Overall/2-Overall-ROI-se.R`
* Source: `Overall/Overall-ROI.R` and `Overall/Overall-ROI-se.R`
* Results: `Weekly/Processed-Data/base-avg-ROI-overall-observed.RData` and `Weekly/Processed-Data/base-avg-overall-std-err-observed.RData`
* Description: Refer to Appendix G. ROI calculation.  

#### Break-even ad effects

* Scripts: `Overall/3-Overall-ROI-breakeven.R`
* Source: `Overall/Overall-ROI-breakeven.R`
* Results: `Weekly/Processed-Data/avg-overall-breakeven-observed.RData`
* Description: Refer to Appendix G.3. Break-even ad effects.  

## (9) Additional Notes

An intuitive guide to building the data is provided in Online Appendix D of the paper. For your convenience, it is also duplicated below.

\newpage

## References

Shapiro, Bradley T., Günter J. Hitsch, and Anna E. Tuchman. 2021. “TV Advertising Effectiveness and Profitability: Generalizable Results from 288 Brands.” \textit{Manuscript.}  



