
PrintStandAloneTable <- function(doc, output_file){
  doc_start <-
    '\\documentclass[tightpage]{standalone}
\\usepackage{varwidth}
\\usepackage[utf8]{inputenc}
\\usepackage{amsmath}
\\usepackage{amsfonts}
\\usepackage{amssymb}
\\usepackage{threeparttable}
\\usepackage{array}
\\usepackage{preview}
\\usepackage{booktabs}
\\usepackage{graphicx}
\\usepackage{rotating}
\\begin{document}
\\begin{preview}
\\begin{varwidth}{\\linewidth}
%REPLACE
\\end{varwidth}
\\end{preview}
\\end{document}'
  
  doc_end <- '\\end{varwidth}
\\end{preview}
\\end{document}'
  
  doc <- paste(doc, collapse = '\n')
  doc <- gsub("\\begin{table}[H]", "\\begin{sidewaystable}", doc,
              fixed = TRUE)
  doc <- gsub("\\end{table}", "\\end{sidewaystable}", doc,
              fixed = TRUE)
  doc_all <- sub('%REPLACE', doc, doc_start, fixed = TRUE)
  tmp_file = 'tmp_latex.tex'
  writeLines(doc_all, tmp_file)
  compile_command <- sprintf('pdflatex -interaction nonstopmode -halt-on-error -file-line-error %s', tmp_file)
  system(compile_command)
  file.rename(sub('tex$', 'pdf', tmp_file), output_file)
  file.remove(grep('tmp_latex\\.', dir(), value = TRUE))
}

PrintStandAloneText <- function(doc, 
                                output_file, 
                                columnwidth = FALSE,
                                adjustbox   = FALSE,
                                change_align= FALSE,
                                remove_linespace = FALSE){
  
  if (adjustbox) {
    doc <- gsub("\\begin{table}[H]", 
                "\\begin{adjustbox}{center}", 
                doc,
                fixed = TRUE)
    doc <- gsub("\\end{table}", 
                "\\end{adjustbox}", 
                doc,
                fixed = TRUE)
  } else {
    doc <- gsub("\\begin{table}[H]", 
                "", 
                doc,
                fixed = TRUE)
    doc <- gsub("\\end{table}", 
                "", 
                doc,
                fixed = TRUE)
  }

  if (remove_linespace) {
    doc <- gsub("\\addlinespace[0.3em]", 
                "", 
                doc,
                fixed = TRUE)
    
  }
  
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){2-2} \\cmidrule(l{3pt}r{3pt}){3-3} \\cmidrule(l{3pt}r{3pt}){4-11}",
              "\\cmidrule(l{3pt}r{3pt}){2-11}",
              doc,
              fixed = TRUE)
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){2-2} \\cmidrule(l{3pt}r{3pt}){3-3} \\cmidrule(l{3pt}r{3pt}){4-4} \\cmidrule(l{3pt}r{3pt}){5-6} \\cmidrule(l{3pt}r{3pt}){7-10}",
              "\\cmidrule(l{3pt}r{3pt}){2-10}",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){3-3} \\cmidrule(l{3pt}r{3pt}){4-4} \\cmidrule(l{3pt}r{3pt}){5-5} \\cmidrule(l{3pt}r{3pt}){6-6} \\cmidrule(l{3pt}r{3pt}){7-8} \\cmidrule(l{3pt}r{3pt}){9-12}",
              "\\cmidrule(l{3pt}r{3pt}){3-12}",
              doc,
              fixed = TRUE)
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){4-4} \\cmidrule(l{3pt}r{3pt}){5-5} \\cmidrule(l{3pt}r{3pt}){6-6} \\cmidrule(l{3pt}r{3pt}){7-8} \\cmidrule(l{3pt}r{3pt}){9-12}",
              "\\cmidrule(l{3pt}r{3pt}){4-12}",
              doc,
              fixed = TRUE)
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){3-3} \\cmidrule(l{3pt}r{3pt}){4-4} \\cmidrule(l{3pt}r{3pt}){5-8}",
              "\\cmidrule(l{3pt}r{3pt}){3-8}",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\cmidrule(l{3pt}r{3pt}){2-2} \\cmidrule(l{3pt}r{3pt}){3-3} \\cmidrule(l{3pt}r{3pt}){4-4} \\cmidrule(l{3pt}r{3pt}){5-5} \\cmidrule(l{3pt}r{3pt}){6-7} \\cmidrule(l{3pt}r{3pt}){8-11}",
              "\\cmidrule(l{3pt}r{3pt}){2-11}",
              doc,
              fixed = TRUE)
  
  
  doc <- gsub("\\$\\textbackslash{}delta=0.9\\$",
              "$\\delta=0.9$",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\$p\\$",
              "$p$",
              doc,
              fixed = TRUE)
  
  if (columnwidth) {
    doc <- gsub("\\begin{tabular}",
                "\\setlength\\tabcolsep{1.5pt}\\begin{tabular}",
                doc,
                fixed = TRUE)
  }
  
  if (change_align) {
    doc <- gsub("\\begin{tabular}{cccccccc}",
                "\\begin{tabular}{llcccccc}",
                doc,
                fixed = TRUE)
    doc <- gsub(">{\\centering\\arraybackslash}",
                ">{\\raggedright\\arraybackslash}",
                doc,
                fixed = TRUE)
    
    doc <- gsub("{>{\\raggedright\\arraybackslash}p{0.1in}>{\\raggedright\\arraybackslash}p{3.7cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.6cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}>{\\raggedright\\arraybackslash}p{1.2cm}}",
                "{>{\\raggedright\\arraybackslash}p{0.1in}llccccccccc}",
                doc, fixed = T)
    
  }
  
  doc <- gsub("begin{tablenotes}", "begin{tablenotes}[para,flushleft]",
             doc, fixed = TRUE)
  doc <- gsub("\\item", "", doc,
             fixed = TRUE)
  doc <- gsub("\\small", "\\footnotesize", doc,
              fixed = TRUE)
  doc <- gsub("\\begin{tablenotes}", "\\vspace{0.15in}\\begin{tablenotes}",
              doc, fixed = TRUE)
  doc <- gsub("textbf{Note: }", "textbf{Note:}", doc,
              fixed = TRUE)
  
  doc <- gsub("\\textbf{\\% p >= 0.05}",
              "\\% $\\mathbf{p \\ge 0.05}$",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\textbf{\\% p < 0.05}",
              "\\% $\\mathbf{p < 0.05}$",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\textbf{$> 0$}",
              "$\\mathbf{> 0}$",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\textbf{$\\le 0$}",
              "$\\mathbf{\\le 0}$",
              doc,
              fixed = TRUE)
  
  doc <- gsub("\\multicolumn{11}{l}{\\textbf{blah}}\\\\",
              "",
              doc, fixed = T)
  
  doc <- gsub("NA",
              "-",
              doc,
              fixed = T)
  
  tmp_file = 'tmp_latex.txt'
  writeLines(doc, tmp_file)
  file.rename(tmp_file, output_file)
  file.remove(grep('tmp_latex\\.', dir(), value = TRUE))
}