# --------------------------------- #
# CI graph
#
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(ggplot2)
library(ggExtra)

source("./Generic-Plot-Settings-v2.R")

load("../Data/Data-used-for-draft0.RData")

# Baseline Strategy
data <- copy(Regular_AdStock)

data <- unique(data)

names(data) <- c("module", "brand", "revenue_RMS", "decay", "ncomp", "nobs", "c", "s", "p")

coef <- data$c
se <- data$s

data <- data[!is.na(coef)]
coef <- coef[!is.na(coef)]

data[, CI_lower := c - 1.96*s]
data[, CI_higher :=c + 1.96*s]

data[CI_lower < -0.3, CI_lower := -0.3]
data[CI_higher > 0.4, CI_higher:= 0.4]

data <- data[order(c)]
data[, id := .I]

plt <- ggplot(data = data) +
  geom_pointrange(aes(x = id,
                      y = c,
                      ymin = CI_lower,
                      ymax = CI_higher),
                  color = "violetred3",
                  size  = 0.2,
                  alpha = 0.5)

scalex <- scale_x_continuous(name = "Brands")
scaley <- scale_y_continuous(name = "Advertising Effects",
                             limits = c(-0.31, 0.41),
                             breaks = round(seq(-0.3, 0.4, 0.1),1))

plt <- plt +
  scalex +
  scaley +
  theme(axis.line.x      = element_line(),
        axis.line.y      = element_line(),
        panel.background = element_blank(),
        panel.grid.minor = element_line(colour = "grey95"),
        panel.grid.major = element_line(colour = "grey90")) +
  theme_bw(base_size = 13) %+replace%
  theme(panel.background = element_blank(),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_line(colour = "grey90"),
        axis.text        = element_text(size = 13),
        legend.key       = element_blank(),
        legend.position  = "none",
        panel.border     = element_blank(),
        axis.line        = element_line())

plt <- ggMarginal(plt,
                  type     = "histogram",
                  binwidth = 0.025,
                  boundary = 0.025,
                  margins  = "y",
                  size     = 3,
                  fill     = "lightsteelblue2")

pdf(paste0("../Figures/fig2.pdf"),
    height = 6,
    width  = 8.5)

print(plt)

dev.off()
