# ---------------------------------------- #
# ROI Calculation - Functions
# 
# 4/3/2019
# 
# ---------------------------------------- #
rm(list = ls())
library(data.table)
library(bit64)
library(lfe)
library(RcppRoll)
library(lubridate)
library(Hmisc)
library(doParallel)

source("./Generic-Plot-Settings-v1.R")

home_dir <- "/project/hitsch_shapiro_ads/Advertising-Generalizability/"
ad_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel-wo-Anomaly/"
rms_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"

N_LAGS <- 52
WALMART_CODE <- 6920
CUMSHARE_CUTOFF <- 0.9

Get.National.Cost <- function(module,
                              brand) {
  cost_dir <- "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/Brand-Level-ROI-Robust-woanomaly/"
  infile <- paste0(cost_dir, module, "-", brand, ".rds")
  Cost_DT <- readRDS(infile)
  
  Cost_DT <- unique(Cost_DT[Cost == 1, .(start_week, Annual_Cost)])
  
  setnames(Cost_DT, "start_week", "week_end")
  
  return(Cost_DT)
}

Get.Avg.Cost <- function(module,
                         brand) {
  
  Cost_DT <- readRDS("../Data/Brand-Cost.rds")
  
  return(Cost_DT[Module == module
                 & Brand  == brand,
                 Spend_per_GRP])
}

Get.Nat.Universe <- function(timeframe) {
  universe_nat <- data.table()
  for (year in timeframe) {
    sub_universe <- fread(paste0("/kilts/adintel/",
                                 year,
                                 "/Universe_Estimates/UENationalTV.tsv"))
    sub_universe[, `:=`(week_end = ceiling_date(date(StartDate), "week", week_start=7)-1,
                        EndDate   = NULL)]
    remove.cols = grep("male|Male|ww|children",
                       names(sub_universe))
    sub_universe[, (remove.cols):=NULL]
    sub_universe <- sub_universe[HispanicFlag == 'N']
    sub_universe[, HispanicFlag := NULL]
    setnames(sub_universe, 'TV_HH', 'Universe_nat')
    sub_universe[, StartDate := NULL]
    
    universe_nat <- rbind(universe_nat,
                          sub_universe)
  }
  
  universe_nat <- unique(universe_nat)
  return(universe_nat)
}

Get.Spot.Universe <- function(timeframe) {
  universe_local <- data.table()
  for (year in timeframe) {
    sub_universe <- fread(paste0("/kilts/adintel/",
                                 year,
                                 "/Universe_Estimates/UESpotTV.tsv"))
    sub_universe[, `:=`(week_end = ceiling_date(date(StartDate), "week", week_start=7)-1,
                        EndDate   = NULL)]
    remove.cols = grep("male|Male|ww|children",
                       names(sub_universe))
    sub_universe[, (remove.cols):=NULL]
    sub_universe <- sub_universe[HispanicFlag == 'N']
    sub_universe[, HispanicFlag := NULL]
    setnames(sub_universe, 'TV_HH', 'Universe_local')
    sub_universe[, StartDate := NULL]
    
    universe_local <- rbind(universe_local,
                            sub_universe)
  }
  
  universe_local <- unique(universe_local)
  
  universe_local[, dma_code := MarketCode + 400]
  
  return(universe_local)
}

Get.Universe.Fraction <- function() {
  universe_nat <- readRDS("../Data/national_universe.rds")
  universe_local <- readRDS("../Data/local_universe.rds")
  
  universe <- fread(paste0(home_dir, 
                           "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
  universe <- universe[, .(new_module_code, new_brand_code_uc, 
                           adcomp_code, comp1_code, 
                           comp2_code, comp3_code)]
  screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
  screening <- screening[fracpos > 0.05 & cmean > 10]
  universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))
  load(paste0(home_dir, "4-Store-Panelist-Selection/Processed-Data/Selected-Stores.RData"))
  store <- use.stores[cumshare<=CUMSHARE_CUTOFF, .(store_code_uc, dma_code)]
  
  week_frame <- seq(date("2010-01-09"), date("2014-12-27"), 7)
  str <- data.table(expand.grid(dma_code = setdiff(unique(store$dma_code), c(740, 810, 630)),
                                week_end = week_frame))
  
  setkey(str, dma_code, week_end)
  setkey(universe_local, dma_code, week_end)
  
  str <- universe_local[str, roll = T]
  
  setkey(str, week_end)
  setkey(universe_nat, week_end)
  str <- universe_nat[str, roll = T]
  
  universe_DT <- str[, .(dma_code, week_end, Universe_local, Universe_nat)]
  
  setkey(universe_DT, dma_code, week_end)
  
  universe_DT[, universe_frac := Universe_local / Universe_nat]
  
  return(universe_DT)
}

ROI.Calculation <- function(module,
                            brand) {
  
  universe_DT <- readRDS("../Data/universe_DT.rds")
  
  cost_DT  <- Get.National.Cost(module,
                                brand)
  
  Avg.Cost <- Get.Avg.Cost(module,
                           brand)
  
  Cost.Value <- copy(universe_DT)
  Cost.Value <- merge(Cost.Value,
                      cost_DT,
                      by = "week_end")
  Cost.Value[, national_cost := Annual_Cost]
  Cost.Value[, local_cost    := national_cost * universe_frac]
  
  Cost.Value[, `:=` (Module = module,
                     Brand  = brand)]
  
  return(Cost.Value)
}

universe <- fread(paste0(home_dir, 
                         "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
universe <- universe[, .(new_module_code, new_brand_code_uc, 
                         adcomp_code, comp1_code, 
                         comp2_code, comp3_code)]

screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
screening <- screening[fracpos > 0.05 & cmean > 10]
universe <- merge(universe, 
                  screening[, .(new_module_code, new_brand_code_uc)], 
                  by=c("new_module_code", "new_brand_code_uc"))

case_table <- universe[, .(new_module_code, new_brand_code_uc)]
ncases <- nrow(case_table)

cost.DT <- data.table()
for (i in 1:ncases) {
  cost.DT <- rbind(cost.DT,
                   ROI.Calculation(module = case_table[i, new_module_code],
                                   brand = case_table[i, new_brand_code_uc]))
}

cost.DT <- cost.DT[year(week_end) > 2010,]

cost.results <- cost.DT[, .(local_cost = mean(local_cost, na.rm = T)),
                        by = .(Module, Brand, 
                               dma_code, year = year(week_end))]

median_cost  <- median(cost.results$local_cost, na.rm = TRUE)

plt <- ggplot() +
  geom_histogram(data  = cost.results,
                 aes(x = local_cost,
                     y = ..count../sum(..count..)*100),
                 fill  = "lightsteelblue2",
                 color = "grey60",
                 boundary = 0,
                 binwidth = 20) +
  geom_vline(data = data.table(median = median_cost),
             mapping = aes(xintercept = median),
             color = "red",
             size = 1.3) +
  geom_text(data = data.table(median = median_cost),
             mapping = aes(x = median,
                           y = 32,
                           label = as.character(round(median, 3))),
            hjust = -0.1,
            vjust = 1,
            size  = 9,
            inherit.aes = F) +
  scale_x_continuous("Advertising Cost",
                     limits = c(-0.01, 500.01),
                     breaks = seq(0, 500, 100)) +
  scale_y_continuous("Percent") +
  theme_bw(base_size = 25) %+replace%
  theme(panel.background = element_blank(),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_line(colour = "grey90"),
        axis.text        = element_text(size = 25),
        axis.title        = element_text(size = 25),
        legend.key       = element_blank(),
        legend.position  = "none",
        panel.border     = element_blank(),
        axis.line        = element_line())

pdf(paste0("../Figures/",
           "fig5.pdf"),
    height = 12,
    width  = 12.8)

print(plt)

dev.off()
