# ---------------------------------------- #
# ROI Presentation
# ---------------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(ggplot2)
library(gridExtra)
library(RcppRoll)

source("./Generic-Plot-Settings-v1.R")

home_dir <- "/project/hitsch_shapiro_ads/Advertising-Generalizability/"
ad_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/AdIntel-wo-Anomaly/"
rms_dir <- "/project/hitsch_shapiro_ads/data/Brand-Level-Data/RMS/"

N_LAGS <- 52
WALMART_CODE <- 6920
CUMSHARE_CUTOFF <- 0.9

Get.Log.Coef <- function(module,
                         brand) {
  coef_file = paste0("/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Regression-wo-anomaly/fe-Regular-Store-No-logsum-dma_code+week_end-Month/",
                     module,
                     "-",
                     brand,
                     ".rds")
  
  coef <- readRDS(coef_file)
  
  coef <- coef[grepl("ls|sl", variable)]
  coef[, variable := gsub("(.*)_(.*)_(.*)", "\\3", variable)]
  coef[variable=="Comp", variable := "5"]
  coef <- coef[Measure == "GRP"
               & Goodwill == "logsum"
               & Feature  == "No"
               & decay    == 0.9]
  
  coef[Maxcomp == "Barebone", Maxcomp := "-1"]
  coef[, ncomp := as.numeric(Maxcomp)]
  
  coef <- coef[variable == "13"]
  
  obs <- coef[ncomp>=0, c("new_module_code", "new_brand_code_uc", "ncomp", "nobs"), with=FALSE]
  obs <- unique(obs)
  obs <- dcast(obs, new_module_code+new_brand_code_uc~ncomp, value.var="nobs")
  
  if (!("3" %in% names(obs))) {
    obs[, "3" := as.integer(NA)]
  }
  if (!("2" %in% names(obs))) {
    obs[, "2" := as.integer(NA)]
  }
  if (!("1" %in% names(obs))) {
    obs[, "1" := as.integer(NA)]
  }
  
  setnames(obs, as.character(0:3), paste0("n", 0:3))
  obs[, ratio1 := n1 / n0]
  obs[, ratio2 := n2 / n0]
  obs[, ratio3 := n3 / n0]
  obs <- obs[, .(new_module_code, new_brand_code_uc, ratio1, ratio2, ratio3)]
  
  retain <- 0.8
  
  cases <- copy(obs)
  cases[, nc := as.integer(NA)]
  cases[ratio1 < retain | is.na(ratio1), nc := 0]
  cases[ratio1 >= retain & (ratio2 < retain | is.na(ratio2)), nc := 1]
  cases[ratio2 >= retain & (ratio3 < retain | is.na(ratio3)), nc := 2]
  cases[ratio3 >= retain, nc := 3]
  cases <- cases[, .(new_module_code, new_brand_code_uc, nc, ratio1, ratio2, ratio3)]
  
  output <- merge(coef, cases, by=c("new_module_code", "new_brand_code_uc"))
  output <- output[ncomp == nc]
  
  return(output)
}

Get.Coef.DT <- function() {
  universe <- fread(paste0(home_dir, 
                           "2-String-Matching/Exported-Files/Universe-For-Estimation-180508.csv"))
  universe <- universe[, .(new_module_code, new_brand_code_uc, 
                           adcomp_code, comp1_code, 
                           comp2_code, comp3_code)]
  
  screening <- readRDS("/project/hitsch_shapiro_ads/data/Descriptive-Statistics/Screening.rds")
  screening <- screening[fracpos > 0.05 & cmean > 10]
  universe <- merge(universe, screening[, .(new_module_code, new_brand_code_uc)], by=c("new_module_code", "new_brand_code_uc"))
  
  case_table <- universe[, .(new_module_code, new_brand_code_uc)]
  ncases <- nrow(case_table)
  
  coef.DT <- data.table()
  
  for (i in 1:ncases) {
    module_code = case_table[i, new_module_code]
    brand_code  = case_table[i, new_brand_code_uc]
    
    sub.coef.DT <- Get.Log.Coef(module_code, brand_code)
    
    coef.DT <- rbind(coef.DT, sub.coef.DT)
  }
  
  return(coef.DT)
}

Gen.Breakeven.plot <- function(input_dir,
                               input_dir1,
                               output_dir,
                               cost = 1,
                               plot = TRUE) {
  load("../Data/breakeven-5-brands.RData")
  
  select.brands <- select.brands[grepl("CHOBANI", brand)]
  
  for (i in 1:nrow(select.brands)) {
    module_code = select.brands[i, new_module_code]
    brand_code  = select.brands[i, new_brand_code_uc]
    
    load(paste0("../Data/",
                "brand-name-vs-brand-code.RData"))
    
    Module.name = obs[new_module_code == as.numeric(module_code) 
                      & new_brand_code_uc == as.numeric(brand_code),
                      module]
    Brand.name  = obs[new_module_code == as.numeric(module_code) 
                      & new_brand_code_uc == as.numeric(brand_code),
                      brand]
    
    DT <- readRDS(paste0(input_dir,
                         as.character(module_code),
                         "-",
                         as.character(brand_code),
                         ".rds"))
    
    DT <- DT[Cost == cost]
    DT[, breakeven_beta := New_Beta]
    DT[, margin := as.numeric(Margin)]
    DT[Margin == "0.3", margin := 0.3]
    
    mean_DT <- DT[, .(New_beta = mean(breakeven_beta)),
                  by = .(Margin)]
    mean_DT[, margin := as.numeric(Margin)]
    mean_DT[Margin == "0.3", margin := 0.3]
    
    origin_beta <- unique(DT[, Original_beta])
    origin_ROI  <- unique(DT[, ROI])
    
    test <- Get.Log.Coef(module = module_code,
                         brand  = brand_code)
    upp_bd <- test$coef + 1.96 * test$se
    low_bd <- test$coef - 1.96 * test$se
    
    overall_DT <- readRDS(paste0(input_dir1,
                                 as.character(module_code),
                                 "-",
                                 as.character(brand_code),
                                 ".rds"))
    
    overall_DT <- overall_DT[Cost == cost]
    overall_DT[, breakeven_beta := New_Beta]
    overall_DT[, margin := as.numeric(Margin)]
    overall_DT[Margin == "0.3", margin := 0.3]
    
    origin_overall_beta <- unique(overall_DT[, Original_beta])
    origin_overall_ROI  <- unique(overall_DT[, ROI])
    
    y_range = round(c(min(min(mean_DT$New_beta),
                          origin_beta,
                          min(overall_DT$breakeven_beta),
                          low_bd),
                      max(max(mean_DT$New_beta),
                          origin_beta,
                          max(overall_DT$breakeven_beta),
                          upp_bd)),
                    4)
    
    ylimits = c(min(y_range) - 0.02, max(y_range) + 0.02)
    
    ybreaks = c(seq(y_range[1],
                    y_range[2],
                    length.out = 5),
                round(origin_beta, 4))
    
    plt <- ggplot() +
      geom_smooth(data = DT,
                  aes(x = margin,
                      y = breakeven_beta),
                  fullrange = TRUE,
                  se    = FALSE,
                  color = "cornflowerblue") +
      geom_line(data = overall_DT,
                aes(x = margin,
                    y = breakeven_beta),
                color = "violetred3") +
      geom_point(data = mean_DT,
                aes(x = margin,
                    y = New_beta),
                color = "black",
                size  = 1) +
      geom_point(data = overall_DT,
                 aes(x = margin,
                     y = breakeven_beta),
                 color = "black",
                 size  = 1) +
      geom_hline(data = data.table(ad_effect = origin_beta),
                 aes(yintercept = ad_effect),
                 color  = "gray50",
                 linetype = "dotdash") +
      geom_ribbon(data = data.table(lower_bd = rep(low_bd, 2),
                                    upper_bd = rep(upp_bd, 2),
                                    margin   = c(0.14, 0.41)),
                  aes(ymin = lower_bd,
                      ymax = upp_bd,
                      x = margin),
                  fill = "gray60",
                  alpha = 0.3) +
      scale_y_continuous(name = "Advertising Effect",
                         breaks = round(sort(ybreaks), 4)) +
      scale_x_continuous(name = "Margin Factor",
                         breaks = seq(0.15, 0.4, 0.05),
                         limits = c(0.14, 0.41)) +
      theme_bw(base_size = 15) %+replace%
      theme(panel.background = element_blank(),
            panel.grid.minor = element_blank(),
            panel.grid.major = element_line(colour = "grey90"),
            axis.text        = element_text(size = 15),
            axis.title       = element_text(size = 15),
            axis.title.x     = element_text(size = 15),
            legend.key       = element_blank(),
            legend.position  = "none",
            panel.border     = element_blank(),
            axis.line        = element_line()) +
      theme(
        strip.background = element_rect(colour = "black", size = 0.5),
        legend.key       = element_blank()
      )
    
    pdf(paste0(output_dir,
               "fig6.pdf"),
        height = 6,
        width  = 6.6)
    
    print(plt)
    
    dev.off()
  }
}

input_dir = "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/base-Brand-Level-Average-ROI-breakeven-observed/"
input_dir1 = "/project/hitsch_shapiro_ads/scripts/Workspace-Ningyin/Profit-Calculation/base-Brand-Level-ROI-avg-overall-breakeven-observed/"
output_dir = "../Figures/"

Gen.Breakeven.plot(input_dir,
                   input_dir1,
                   output_dir,
                   cost = 1,
                   plot = TRUE)

