## ---------------------------------------------------------------- ##
## Recreate histograms of break-even adeffect multiplier
## ---------------------------------------------------------------- ##
rm(list = ls())
library(bit64)
library(shiny)
library(ggplot2)
library(DT)
library(data.table)
library(scales)
library(D3TableFilter)
library(shinysky)
library(Hmisc)
library(acepack)
library(checkmate)
library(htmlTable)
library(latticeExtra)
# library(emo)

source('./Generic-Plot-Settings-v1.R')
graph_par$round <- 4

# ------------------------------------ #
# Load and Process Regression Results
# ------------------------------------ #

load("../Data/shiny-data-0910.RData")
load("../../8-ROI-Analyses/Weekly/Processed-Data/base-average-ROI-observed.RData")
load("../../8-ROI-Analyses/Weekly/Processed-Data/average-breakeven-observed.RData")
load("../../8-ROI-Analyses/Overall/Processed-Data/base-avg-ROI-overall-observed.RData")
load("../../8-ROI-Analyses/Overall/Processed-Data/avg-overall-breakeven-observed.RData")

average <- copy(average_observed)
average_breakeven <- copy(average_breakeven_observed)
avg_overall <- copy(avg_overall_observed)
avg_overall_breakeven <- copy(avg_overall_breakeven_observed)

rm(average_observed, average_breakeven_observed, 
   avg_overall_observed, avg_overall_breakeven_observed)

Get.Coef.DT <- function() {
  data <- coef_output[level      == "Store" 
                      & strategy == "Regular" 
                      & feature  == "No"
                      & cluster  == "dma_code+week_end"]
  data <- data[time       == "Month" 
               & measure  == "GRP" 
               & goodwill == "logsum"]
  
  data <- data[variable       == "13" 
               & delta_status == "0.90"]
  
  data <- data[, .(new_module_code, new_brand_code_uc, revenue_RMS, decay, ncomp, nobs, c, s, p)]
  
  obs <- master_obs[level      == "Store"
                    & strategy == "Regular" 
                    & feature  == "No"
                    & cluster  == "dma_code+week_end"
                    & time       == "Month" 
                    & goodwill == "logsum"]
  
  retain <- 0.8
  
  cases <- copy(obs)
  cases[, nc := as.integer(NA)]
  cases[r1 < retain, nc := 0]
  cases[r1 >= retain & r2 < retain, nc := 1]
  cases[r2 >= retain & r3 < retain, nc := 2]
  cases[r3 >= retain, nc := 3]
  cases <- cases[, .(new_module_code, new_brand_code_uc, nc)]
  
  coef.DT <- merge(data, cases, by=c("new_module_code", "new_brand_code_uc"))
  coef.DT <- coef.DT[ncomp == nc]
  coef.DT <- coef.DT[, .(new_module_code, new_brand_code_uc, revenue_RMS, decay, ncomp, nobs, c, s, p)]
  setorder(coef.DT, -revenue_RMS)
  names(coef.DT) <- c("new_module_code", "new_brand_code_uc", 
                      "Revenue", "Delta", "Comp", "Obs", 
                      "coef", "se", "pval")
  
  return(coef.DT)
}

Get.Nonnegative.Brands <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  coef <- data$c
  se <- data$s
  
  condition <- data$c > 0
  
  data <- data[condition == T]
  
  return(data[, .(Module, Brand, c)])
}

Get.SE.Brands <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  coef <- data$c
  se <- data$s
  
  condition <- data$s <= 0.025
  
  data <- data[condition == T]
  
  return(data[, .(Module, Brand, c, s)])
}

Get.Pos.Sig.Brands <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  coef <- data$c
  se <- data$s
  
  condition <- data$p < 0.05 & data$c > 0
  
  data <- data[condition == T]
  
  return(data[, .(Module, Brand, c)])
}

Get.Neg.Sig.Brands <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  coef <- data$c
  se <- data$s
  
  condition <- data$p < 0.05 & data$c < 0
  
  data <- data[condition == T]
  
  return(data[, .(Module, Brand, c)])
}

Get.Non.Sig.Brands <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  coef <- data$c
  se <- data$s
  
  condition <- data$p >= 0.05
  
  data <- data[condition == T]
  
  return(data[, .(Module, Brand, c)])
}

Get.CI <- function() {
  data <- Get.Coef.DT()
  
  data <- data[, .(new_module_code, new_brand_code_uc, coef, se, pval)]
  
  names(data) <- c("Module", 
                   "Brand", 
                   "c", "s", "p")
  
  data[, `:=` (lb = c - 1.96 * s,
               ub = c + 1.96 * s)]
  
  return(data[, .(Module, Brand, c, lb, ub)])
}

## Weekly Average

plot.DT <- copy(average_breakeven)
plot.DT <- plot.DT[!is.na(ROI)]

CI.DT <- Get.CI()

## percentage of brands with significantly large break-even ad elasticity


plot.DT <- plot.DT[Cost == 1 & Original_beta > 0]

plot.DT[, breakeven_beta := New_Beta]
plot.DT[, beta_factor := breakeven_beta / Original_beta]
plot.DT[, margin := as.numeric(Margin)]
plot.DT[Margin == "0.3", margin := 0.3]
plot.DT[, c("Margin") := NULL]
plot.DT[, Margin := margin]

plot.DT[, Test := uniqueN(Original_beta), 
        by = .(Module, Brand, Margin)]
stopifnot(unique(plot.DT$Test) == 1)
plot.DT[, Test := NULL]

plot.DT <- plot.DT[, .(beta_factor = mean(beta_factor, na.rm = T),
                       original_beta = mean(Original_beta, na.rm = T)),
                   by = .(Module, Brand, Margin)]

plot.DT <- merge(plot.DT,
                 CI.DT,
                 by = c("Module", "Brand"))

stopifnot(plot.DT[round(original_beta, 4) != c, .N] == 0)

plot.DT[, "within CI" := (original_beta * beta_factor >= lb 
                          & original_beta * beta_factor <= ub)]

plot.DT[, count := 1]
plot.DT[, sum_group := sum(count),
        by = .(Margin)]

plot.DT[, freq := count/sum_group]

median_table = unique(plot.DT[, median := median(beta_factor, na.rm=T),
                              by = .(Margin)][, 
                                              .(Margin, median)])

selected_margin = as.numeric(gsub("%","", "30%")) / 100

sub.plot.DT <- plot.DT[Margin == selected_margin]
sub.median_table <- median_table[Margin == selected_margin]

plt <- ggplot(data = sub.plot.DT) +
  geom_vline(mapping = aes(xintercept = 0),
             color = "grey60",
             linetype = "dotdash") +
  geom_histogram(aes(x = beta_factor,
                     weight = freq,
                     fill = `within CI`),
                 # fill = "lightsteelblue2",
                 color = "grey60",
                 boundary = 0,
                 binwidth = 0.5) + 
  geom_vline(data = sub.median_table,
             mapping = aes(xintercept = median),
             color = "violetred3") +
  geom_text(data = sub.median_table,
            mapping = aes(x = median,
                          y = 0.6,
                          label = as.character(round(median,3))),
            hjust = -0.1,
            vjust = 1,
            size  = 16/3,
            inherit.aes = F) +
  scale_x_continuous("Beta Multiplier",
                     limits = c(-0.01, 20.01),
                     breaks = seq(0, 20, 5)) +
  scale_y_continuous("Frequency") +
  labs(title = "") +
  scale_fill_manual(values = c("grey92",
                               "lightsteelblue2")) + 
  theme_bw(base_size = 16) %+replace%
  theme(panel.background = element_blank(),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_line(colour = "grey90"),
        axis.text        = element_text(size = 16),
        axis.title       = element_text(size = 16),
        axis.title.x     = element_text(size = 16),
        legend.key       = element_blank(),
        legend.position  = "none",
        panel.border     = element_blank(),
        axis.line        = element_line())

pdf("../Figures/fig7_weekly.pdf",
    width = 6.4, height = 6)
print(plt)
dev.off()

# Average overall

plot.DT <- copy(avg_overall_breakeven)
plot.DT <- plot.DT[!is.na(ROI)]

CI.DT <- Get.CI()

plot.DT <- plot.DT[Cost == 1 & Original_beta > 0]

plot.DT[, breakeven_beta := New_Beta]
plot.DT[, beta_factor := breakeven_beta / Original_beta]
plot.DT[, margin := as.numeric(Margin)]
plot.DT[Margin == "0.3", margin := 0.3]
plot.DT[, c("Margin") := NULL]
plot.DT[, Margin := margin]

plot.DT[, Test := uniqueN(Original_beta), 
        by = .(Module, Brand, Margin)]
stopifnot(unique(plot.DT$Test) == 1)
plot.DT[, Test := NULL]

plot.DT <- plot.DT[, .(beta_factor = mean(beta_factor, na.rm = T),
                       original_beta = mean(Original_beta, na.rm = T)),
                   by = .(Module, Brand, Margin)]

plot.DT <- merge(plot.DT,
                 CI.DT,
                 by = c("Module", "Brand"))

stopifnot(plot.DT[round(original_beta, 4) != c, .N] == 0)

plot.DT[, "within CI" := (original_beta * beta_factor >= lb 
                          & original_beta * beta_factor <= ub)]

plot.DT[, count := 1]
plot.DT[, sum_group := sum(count),
        by = .(Margin)]

plot.DT[, freq := count/sum_group]

median_table = unique(plot.DT[, median := median(beta_factor, na.rm=T),
                              by = .(Margin)][, 
                                              .(Margin, median)])

selected_margin = as.numeric(gsub("%","", "30%")) / 100

sub.plot.DT <- plot.DT[Margin == selected_margin]
sub.median_table <- median_table[Margin == selected_margin]

plt <- ggplot(data = sub.plot.DT) +
  geom_vline(mapping = aes(xintercept = 0),
             color = "grey60",
             linetype = "dotdash") +
  geom_histogram(aes(x = beta_factor,
                     weight = freq,
                     fill = `within CI`),
                 # fill = "lightsteelblue2",
                 color = "grey60",
                 boundary = 0,
                 binwidth = 0.5) + 
  geom_vline(data = sub.median_table,
             mapping = aes(xintercept = median),
             color = "violetred3") +
  geom_text(data = sub.median_table,
            mapping = aes(x = median,
                          y = 0.6,
                          label = as.character(round(median,3))),
            hjust = -0.1,
            vjust = 1,
            size  = 16/3,
            inherit.aes = F) +
  scale_x_continuous("Beta Multiplier",
                     limits = c(-0.01, 20.01),
                     breaks = seq(0, 20, 5)) +
  scale_y_continuous("Frequency") +
  labs(title = "") +
  scale_fill_manual(values = c("grey92",
                               "lightsteelblue2")) + 
  theme_bw(base_size = 16) %+replace%
  theme(panel.background = element_blank(),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_line(colour = "grey90"),
        axis.text        = element_text(size = 16),
        axis.title       = element_text(size = 16),
        axis.title.x     = element_text(size = 16),
        legend.key       = element_blank(),
        legend.position  = "none",
        panel.border     = element_blank(),
        axis.line        = element_line())

pdf("../Figures/fig7_overall.pdf",
    width = 6.4, height = 6)
print(plt)
dev.off()
