# -------------------------------------------- #
# Advertising Elasticities - Semiparametric
# -------------------------------------------- #

# 1. Histogram

rm(list = ls())
library(data.table)
library(bit64)
library(ggplot2)
library(knitr)
library(kableExtra)
library(DT)

source("./Generic-Plot-Settings-v2.R")

Elas.DT <- readRDS("../../7-Semiparametric/Processed-Data/Elasticities.rds")

summary.Elas.DT <- Elas.DT[Type == "Median-25-75"]

save(summary.Elas.DT, file = "../Data/summary_lasso_coef.RData")

plt.DT <- Elas.DT[Type == "Median-25-75"]

Plot.Hist(data               = plt.DT,
          selected_column    = "Value",
          strategy_hist      = FALSE,
          selected_universe  = NA,
          normalize_selected = FALSE,
          vline_stat         = "median",
          vline_text         = TRUE,
          binwidth           = 0.025,
          x_breaks           = seq(-0.4, 0.4, 0.2),
          x_name             = "Adstock Elasticity (Flexible Functional Form)",
          y_breaks           = seq(0, 30, 10),
          y_name             = "Percent",
          y_type             = "percent",
          emphasize_color    = "lightsteelblue2",
          compare_color      = "gray95",
          transparency       = 1,
          save_to_file       = TRUE,
          output_location    = "../Figures/fig8_hist.pdf",
          text_size          = 13)

# 2. Scatter Plot
load("../Data/Data-used-for-draft.RData")
load("../Data/brand-name-vs-brand-code.RData")

Regular_AdStock <- merge(Regular_AdStock,
                         obs,
                         by = c("module", "brand"))

Plot.Scatter(x_data = plt.DT[, .(new_module_code,
                                 new_brand_code_uc,
                                 c = Value)],
             y_data = Regular_AdStock[, .(new_module_code,
                                          new_brand_code_uc,
                                          c)],
             merge_cols = c("new_module_code",
                            "new_brand_code_uc"),
             x_breaks   = seq(-0.4, 0.4, 0.2),
             x_name     = "Advertising Elasticity (Flexible Functional Form)",
             y_name     = "Advertising Elasticity (Parametric Model)",
             transparency = 0.5,
             save_to_file       = TRUE,
             output_location    = "../Figures/fig8_scatter.pdf")

