# --------------------------------- #
# CV Splines
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(ggplot2)
library(ggExtra)

source("./Generic-Plot-Settings-v2.R")

cvsplines <- readRDS("../../7-Semiparametric/Processed-Data/cvspline_Q_hat.rds")
cvspline_vline <- readRDS("../../7-Semiparametric/Processed-Data/cvspline_knots.rds")

load("../Data/shiny-data-0910.RData")
master_obs = unique(master_obs)

load("../Data/chobani_gatorade.RData")

Gen.Plot <- function(brand) {
   border_time    = "Month"
   border_cluster = "dma_code+week_end"
   border_level   = "Store"
   border_feature = "No"
   
   data <- coef_output[level      == border_level 
                       & strategy == "Regular"
                       & feature  == border_feature 
                       & cluster  == border_cluster]
   data <- data[time       == border_time 
                & measure  == "GRP" 
                & goodwill == "logsum"]
   
   data <- data[variable       == "13" 
                & delta_status == "0.90"]
   
   data <- data[, .(new_module_code, new_brand_code_uc,
                    module, brand, revenue_RMS, decay, ncomp, nobs, c, s, p)]
   # print(nrow(data))
   
   obs <- master_obs[level      == border_level 
                     & strategy == "Regular"
                     & feature  == border_feature
                     & cluster  == border_cluster
                     & time       == border_time 
                     & goodwill == "logsum"]
   
   # print(obs[1:3])
   # print(nrow(obs))
   retain <- 0.8
   
   cases <- copy(obs)
   cases[, nc := as.integer(NA)]
   cases[r1 < retain, nc := 0]
   cases[r1 >= retain & r2 < retain, nc := 1]
   cases[r2 >= retain & r3 < retain, nc := 2]
   cases[r3 >= retain, nc := 3]
   cases <- cases[, .(module, brand, nc)]
   # print(nrow(cases))
   output <- merge(data, cases, by=c("module", "brand"))
   # print(nrow(output))
   output <- output[ncomp == nc]
   # print(nrow(output))
   output <- output[, .(new_module_code, new_brand_code_uc,
                        module, brand, decay, ncomp, c, s, p)]
   # setorder(output, -revenue_RMS)
   names(output) <- c("new_module_code", "new_brand_code_uc",
                      "Module", "Brand", "Delta", "Comp", "Coef", "SE", "Pval")
   
   DT <- output[grepl(toupper(brand), 
                      Brand)]
   
   cvspline_DT <- merge(unique(cvsplines),
                        DT[,.(new_module_code, new_brand_code_uc, Module, Brand)],
                        by = c("new_module_code", "new_brand_code_uc"))
   setnames(cvspline_DT, c("Module",  "Brand", "Adstock", "Q_hat", "new_module_code", "new_brand_code_uc"),
            c("module_name", "brand_name", "sum_GRP_13", "value", "module", "brand"))
   
   cvspline_DT[, `:=` (Type = "Q_hat_cvspline",
                       Type_f = "Flexible Functional Form")]
   
   log_DT <- merge(collect.splines[Type == "Q_hat_log"],
                   DT[,.(Module, Brand)],
                   by.x = c("module_name", "brand_name"),
                   by.y = c("Module", "Brand"))
   
   log_DT <- log_DT[, -c("id", "ls_GRP_13")]
   
   DT_long <- rbind(log_DT,
                    cvspline_DT[, names(log_DT), with = F])
   
   vline_DT <- merge(cvspline_vline,
                     DT[,.(new_module_code, new_brand_code_uc, Module, Brand)],
                     by = c("new_module_code", "new_brand_code_uc"))
   setnames(vline_DT, c("Module",  "Brand", "Adstock", "new_module_code", "new_brand_code_uc"),
            c("module_name", "brand_name", "sum_GRP_13", "module", "brand"))
   
   y_range1 <- range(DT_long[Type == "Q_hat_cvspline", value])
   y_range2 <- range(DT_long[Type == "Q_hat_log", value])
   
   min_y <- min(y_range1[1], y_range2[1])
   max_y <- max(y_range1[2], y_range2[2])
   
   gap   <- (max_y - min_y) / 2
   
   # Adjust to mean value:
   diff_models <- mean(DT_long[Type == "Q_hat_cvspline", value]) - mean(DT_long[Type == "Q_hat_log", value])
   if (abs(diff_models) > gap * 0.5) {
      DT_long <- DT_long[Type == "Q_hat_cvspline",
                         value := value - diff_models]
      y_range1 <- range(DT_long[Type == "Q_hat_cvspline", value])
      y_range2 <- range(DT_long[Type == "Q_hat_log", value])
      
      min_y <- min(y_range1[1], y_range2[1])
      max_y <- max(y_range1[2], y_range2[2])
      
      gap   <- (max_y - min_y) / 2
      
      y_start <- round(min_y - gap * 0.5, 1)
      y_end   <- round(min_y + gap * 2.5, 1)
      
   } else {
      y_start <- round(min_y - gap * 0.5, 1)
      y_end   <- round(min_y + gap * 2.5, 1)
   }
   
   
   plt <- ggplot() +
      geom_vline(data = vline_DT[quantile != "Mean"],
                 aes(xintercept = sum_GRP_13),
                 color   = "palegreen4",
                 linetype= "dashed") +
      geom_point(data = DT_long,
                 aes(x = sum_GRP_13,
                     y = value,
                     color = Type_f),
                 shape = 19,
                 size  = 2) +
      scale_y_continuous(name   = "Predicted Quantities",
                         limits = c(y_start, y_end),
                         breaks = round(seq(y_end, y_start, length.out = 11), 2)) +
      scale_x_continuous(name = "Advertising Stock") +
      scale_color_manual(values = c("violetred3",
                                    "dodgerblue3"),
                         name   = "") +
      theme_bw(base_size = 10, base_family = "") %+replace%
      theme(legend.justification = c(0.9, 0.9), 
            legend.position      = c(0.75, 0.95),
            legend.title         = element_blank(),
            panel.background = element_blank(),
            panel.grid.minor = element_blank(),
            panel.grid.major = element_line(colour = "grey90")) +
      theme(
         strip.background = element_rect(colour = "black", size = 0.5),
         legend.key       = element_blank()
      ) + 
      theme(panel.grid.major = element_line(colour='grey90'), 
            panel.grid.minor = element_blank())
   
   rm(y_range1, y_range2,
      min_y, max_y, gap,
      y_start, y_end)
   
   return(plt)
}



# Chobani
# ---------------------------------------

plt <- Gen.Plot("Chobani")

pdf("../Figures/fig9_chobani.pdf",
    width = 6.4, height = 6)

print(plt)

dev.off()

# Gatorade
# ---------------------------------------

plt <- Gen.Plot("Gatorade")

pdf("../Figures/fig9_gatorade.pdf",
    width = 6.4, height = 6)

print(plt)

dev.off()

