# -------------------------------- # 
# Create Summary Stats at Brand Level
# Unit: 288 brands
# -------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)
library(scales)

source("./Gen-Table-Function.R")
load("../Data/brand-summary-uw.RData")
load("../Data/brand-summary-updated-grp.RData")
load("../Data/Collected_Cost_National_1.RData")
universe <- readRDS("../Data/national_universe.rds")

{
  collect.ad <- collect.ad[Class != "Missing Network"]
  total.spend <- collect.ad[, .(Spend = sum(Spend, na.rm=T)),
                                 .(new_module_code, new_brand_code_uc, Class)]
  
  total.spend[, Total_Spend := sum(Spend, na.rm=T),
              by = c("new_module_code", "new_brand_code_uc")]
  total.spend[, Spend_Share := Spend / Total_Spend * 100]
  
  total.spend <- total.spend[Total_Spend!=0]
  
  spend.class <- unique(total.spend[, .(new_module_code,
                                        new_brand_code_uc)])
  
  num_brands <- nrow(spend.class)
  
  spend.class <- spend.class[rep(1:.N, 4)]
  
  setkey(spend.class, new_module_code, new_brand_code_uc)
  
  spend.class[, Class := rep(c("Cable", "Network", "Spot", "Syndicated"),
                             num_brands)]
  
  total.spend <- merge(total.spend,
                       spend.class,
                       by = c("new_module_code",
                              "new_brand_code_uc",
                              "Class"),
                       all.y = T)
  
  total.spend[is.na(Spend), Spend := 0]
  
  total.spend[, Spend := Spend / 1000000]
  
  total.spend.share.by.class <- 
    total.spend[, .("  " = round(median(Spend_Share, na.rm = T),2),
                    "   " = round(mean(Spend_Share, na.rm = T),2),
                    "\\textbf{1\\%}" = round(quantile(Spend_Share, probs = 0.01, na.rm = T),1),
                    "\\textbf{5\\%}" = round(quantile(Spend_Share, probs = 0.05, na.rm = T),1),
                    "\\textbf{10\\%}" = round(quantile(Spend_Share, probs = 0.1, na.rm = T),1),
                    "\\textbf{25\\%}" = round(quantile(Spend_Share, probs = 0.25, na.rm = T),1),
                    "\\textbf{75\\%}" = round(quantile(Spend_Share, probs = 0.75, na.rm = T),1),
                    "\\textbf{90\\%}" = round(quantile(Spend_Share, probs = 0.9, na.rm = T),1),
                    "\\textbf{95\\%}" = round(quantile(Spend_Share, probs = 0.95, na.rm = T),1),
                    "\\textbf{99\\%}" = round(quantile(Spend_Share, probs = 0.99, na.rm = T),1)),
                .(Class)]
  
  setnames(total.spend.share.by.class, "Class", "     ")
  
  total.spend.share.by.class <- 
    cbind(data.table(
      " " = c("\\% of Adv. spending",
              "\\% of Adv. spending",
              "\\% of Adv. spending",
              "\\% of Adv. spending")),
      total.spend.share.by.class)
  
}

table_DT <- summary_DT[, .(rms_revenue,
                           hms_revenue, 
                           annual_cost)]

table_DT[, `:=` (rms_revenue = rms_revenue / 1000000,
                 hms_revenue = hms_revenue / 1000000,
                 annual_cost = annual_cost / 1000000)]

table    <- data.table(
  " "  = rep("blah", 5),
  "     "      = c("RMS revenue", 
               "Homescan revenue", 
               "Advertising spending",
               "Mean weekly GRP's",  
               "Adv./sales ratio"
               ),
  "  "     = as.character(c(round(sapply(table_DT, 
                            quantile, probs = 0.5, na.rm = T), 1),
               round(quantile(weekly_GRP_DT$weekly_GRP, 
                              probs = 0.5, na.rm = T),
                     1),
               round(quantile(summary_DT$ad_sales * 100, 
                              probs = 0.5, na.rm = T),
                     1))),
  "   "    = as.character(c(round(sapply(table_DT, 
                            mean, na.rm = T), 1),
               round(mean(weekly_GRP_DT$weekly_GRP, 
                          na.rm = T),
                     1),
               round(mean(summary_DT$ad_sales * 100, 
                          na.rm = T),
                     1))),
  "\\textbf{1\\%}" = as.character(c(round(sapply(table_DT, 
                                                 quantile, probs = 0.01, na.rm = T), 1),
                                    round(quantile(weekly_GRP_DT$weekly_GRP, 
                                                   probs = 0.01, na.rm = T),
                                          1),
                                    round(quantile(summary_DT$ad_sales * 100, 
                                                   probs = 0.01, na.rm = T),
                                          1))),
  "\\textbf{5\\%}" = as.character(c(round(sapply(table_DT, 
                                    quantile, probs = 0.05, na.rm = T), 1),
                       round(quantile(weekly_GRP_DT$weekly_GRP, 
                                      probs = 0.05, na.rm = T),
                             1),
                       round(quantile(summary_DT$ad_sales * 100, 
                                      probs = 0.05, na.rm = T),
                             1))),
  "\\textbf{10\\%}" = as.character(c(round(sapply(table_DT, 
                                                 quantile, probs = 0.1, na.rm = T), 1),
                                    round(quantile(weekly_GRP_DT$weekly_GRP, 
                                                   probs = 0.1, na.rm = T),
                                          1),
                                    round(quantile(summary_DT$ad_sales * 100, 
                                                   probs = 0.1, na.rm = T),
                                          1))),
  "\\textbf{25\\%}" = as.character(c(round(sapply(table_DT, 
                                     quantile, probs = 0.25, na.rm = T), 1),
                        round(quantile(weekly_GRP_DT$weekly_GRP, 
                                       probs = 0.25, na.rm = T),
                              1),
                        round(quantile(summary_DT$ad_sales * 100, 
                                       probs = 0.25, na.rm = T),
                              1))),
  "\\textbf{75\\%}" = as.character(c(round(sapply(table_DT, 
                                     quantile, probs = 0.75, na.rm = T), 1),
                        round(quantile(weekly_GRP_DT$weekly_GRP, 
                                       probs = 0.75, na.rm = T),
                              1),
                        round(quantile(summary_DT$ad_sales * 100, 
                                       probs = 0.75, na.rm = T),
                              1))),
  "\\textbf{90\\%}" = as.character(c(round(sapply(table_DT, 
                                                  quantile, probs = 0.9, na.rm = T), 1),
                                     round(quantile(weekly_GRP_DT$weekly_GRP, 
                                                    probs = 0.9, na.rm = T),
                                           1),
                                     round(quantile(summary_DT$ad_sales * 100, 
                                                    probs = 0.9, na.rm = T),
                                           1))),
  "\\textbf{95\\%}" = as.character(c(round(sapply(table_DT, 
                                     quantile, probs = 0.95, na.rm = T), 1),
                        round(quantile(weekly_GRP_DT$weekly_GRP, 
                                       probs = 0.95, na.rm = T),
                              1),
                        round(quantile(summary_DT$ad_sales * 100, 
                                       probs = 0.95, na.rm = T),
                              1))),
  "\\textbf{99\\%}" = as.character(c(round(sapply(table_DT, 
                                                  quantile, probs = 0.99, na.rm = T), 1),
                                     round(quantile(weekly_GRP_DT$weekly_GRP, 
                                                    probs = 0.99, na.rm = T),
                                           1),
                                     round(quantile(summary_DT$ad_sales * 100, 
                                                    probs = 0.99, na.rm = T),
                                           1)))
)

table <- rbindlist(list(table, total.spend.share.by.class))

table  <- kable(table,
                format = "latex",
                booktabs = T,
                align = "c",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 9) %>%
  add_header_above(c(" "     = 2,
                     "Median"= 1,
                     "Mean"  = 1,
                     "Percentiles" = 8),
                   bold = T) %>%
  column_spec(1, width = "0.1in") %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  footnote(general = "The sample includes 288 brands. Revenue and advertising spending are expressed in millions of dollars. The advertising/sales ratio is calculated using Homescan revenue.",
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table,
                    "../Tables/tab1.txt",
                    columnwidth = FALSE,
                    change_align = TRUE)

