# -------------------------------- # 
# Create Revenue Share by Department
# 
# -------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)
library(scales)

source("./Gen-Table-Function.R")
load("../Data/brand-summary-uw.RData")
load("../Data/product_dept_group.RData")

summary <- merge(summary_DT,
                 dept_descrp,
                 by.x = c("module_code", "brand_code"),
                 by.y = c("new_module_code",
                          "new_brand_code_uc"))

summary_by_dept <- summary[, .("No. of brands" = .N,
                               "Group RMS Revenue"= sum(rms_revenue,
                                                        na.rm = T),
                               "Group HMS Revenue"= sum(hms_revenue,
                                                        na.rm = T)),
                           by = .(department_code_all,
                                  department_descr_all)]

summary_by_dept[, `:=` ("Group RMS Revenue Share" = `Group RMS Revenue` / sum(summary_DT$rms_revenue),
                        "Group HMS Revenue Share" = `Group HMS Revenue` / sum(summary_DT$hms_revenue))]

summary_by_dept <- summary_by_dept[order(-`No. of brands`)]

table6 <- summary_by_dept[, .(department_descr_all,
                              `No. of brands`,
                              `Group HMS Revenue Share`)]

table6[, "Department" := department_descr_all]
table6[, "Homescan revenue share" := round(`Group HMS Revenue Share` * 100, 2)]
table6[, c("department_descr_all", "Group HMS Revenue Share") := NULL]

table6[Department == "HEALTH & BEAUTY CARE", Department := "HEALTH \\& BEAUTY CARE"]

setcolorder(table6, c("Department"))

table6 <- kable(table6,
                format = "latex",
                booktabs = T,
                align = "lrr",
                linesep = "",
                escape = FALSE) %>%
  kable_styling(font_size = 9) %>%
  column_spec(1, width = "5cm") %>%
  column_spec(2:3, width = "2cm") %>%
  footnote(general = "Three brands in our sample have products in two departments.",
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table6,
                    "../Tables/tab10.txt")

summary <- merge(summary_DT,
                 product_descrp,
                 by.x = c("module_code", "brand_code"),
                 by.y = c("new_module_code",
                          "new_brand_code_uc"))

summary_by_product <- summary[, .("No. of brands" = .N,
                               "Group RMS Revenue"= sum(rms_revenue,
                                                        na.rm = T),
                               "Group HMS Revenue"= sum(hms_revenue,
                                                        na.rm = T)),
                           by = .(product_group_code_all,
                                  product_group_descr_all)]

summary_by_product[, `:=` ("Group RMS Revenue Share" = `Group RMS Revenue` / sum(summary_DT$rms_revenue),
                        "Group HMS Revenue Share" = `Group HMS Revenue` / sum(summary_DT$hms_revenue))]

summary_by_product <- summary_by_product[order(-`No. of brands`)]

table7 <- summary_by_product[, .(product_group_descr_all,
                              `No. of brands`,
                              `Group HMS Revenue Share`)]

table7[, "Category" := product_group_descr_all]
table7[, "Homescan revenue share" := round(`Group HMS Revenue Share` * 100, 2)]
table7[, c("product_group_descr_all", "Group HMS Revenue Share") := NULL]

table7[grepl("&", Category) == T, Category := gsub("&", "\\&", Category, fixed = TRUE)]

setcolorder(table7, c("Category"))

table7.1 <- kable(table7[, .(Category, `No. of brands`)][1:35],
                format = "latex",
                booktabs = T,
                align = "lr",
                linesep = "",
                escape = FALSE) %>%
  kable_styling(font_size = 8) %>%
  column_spec(1, width = "6.5cm") %>%
  column_spec(2, width = "1cm") %>%
  row_spec(1:35, extra_latex_after = "[0.01ex]") %>%
  footnote(general = "Four brands in our sample have products in two categories.",
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

table7.2 <- kable(table7[, .(Category, `No. of brands`)][36:70],
                  format = "latex",
                  booktabs = T,
                  align = "lr",
                  linesep = "",
                  escape = FALSE) %>%
  kable_styling(font_size = 8) %>%
  column_spec(1, width = "6.5cm") %>%
  column_spec(2, width = "1cm") %>%
  row_spec(1:35, extra_latex_after = "[0.01ex]") 
# %>%
#   footnote(general = "Four brands in our sample have products in two categories.",
#            threeparttable = T,
#            fixed_small_size = TRUE,
#            title_format = c("bold"))

PrintStandAloneText(table7.1,
                    "../Tables/tab11-1.txt",
                    columnwidth = TRUE)

PrintStandAloneText(table7.2,
                    "../Tables/tab11-2.txt",
                    columnwidth = TRUE)
