# --------------------------------- #
# Table for main results
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

options(scipen = 999)

source("./Gen-Table-Function.R")
load("../Data/Data-used-for-draft.RData")
load("../Data/Data-used-for-draft-best.RData")
load("../Data/Data-used-for-Table2.RData")

load("../Data/summary_lasso_coef.RData")
load("../Data/summary_lasso_coef_baseline.RData")

Regular_AdStock_power <- Regular_AdStock[s <= 0.025]

table1_DT <- data.table(
  "Specification" = c(rep("Main Results", 5),
                      rep("Robustness", 8)
  ),
  "     " = c("Naive",
              "+ Store FE",
              "+ Season FE",
              "+ Time trend",
              "Baseline specification",
              "Semi-parametric",
              "Estimated $\\delta$",
              "Prices excluded",
              "+ Own price",
              "+ Top 1 competitors",
              "+ Up to top 2 competitors", 
              "Feature \\& display included",
              "50\\% power to detect 0.05"
  ),
  " "             = round(c(quantile(Naive_AdStock$c, probs = 0.5),
                            quantile(`MarketFE-only_AdStock`$c, probs = 0.5),
                            quantile(`MarketFE-WOY_AdStock`$c, probs = 0.5),
                            quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.5),
                            quantile(Regular_AdStock$c, probs = 0.5),
                            quantile(summary.Elas.DT[Value < 10, Value], na.rm = T, probs = 0.5),
                            quantile(Regular_Best$c, probs = 0.5),
                            quantile(Regular_ncomp_Barebone$c, probs = 0.5),
                            quantile(Regular_ncomp_0$c, probs = 0.5),
                            quantile(Regular_ncomp_1$c, probs = 0.5),
                            quantile(Regular_ncomp_2$c, probs = 0.5),
                            quantile(Regular_Feature_Display$c, probs = 0.5),
                            quantile(Regular_AdStock_power$c, probs = 0.5)), 4),
  "  "            = round(c(mean(Naive_AdStock$c),
                            mean(`MarketFE-only_AdStock`$c),
                            mean(`MarketFE-WOY_AdStock`$c),
                            mean(`MarketFE+WOY+National_AdStock`$c),
                            mean(Regular_AdStock$c),
                            mean(summary.Elas.DT[Value < 10, Value], na.rm = T),
                            mean(Regular_Best$c),
                            mean(Regular_ncomp_Barebone$c),
                            mean(Regular_ncomp_0$c),
                            mean(Regular_ncomp_1$c),
                            mean(Regular_ncomp_2$c),
                            mean(Regular_Feature_Display$c),
                            mean(Regular_AdStock_power$c)
                            ), 4),
  "   "           = round(c(Naive_AdStock[p >= 0.05,.N]/Naive_AdStock[, .N]*100,
                            `MarketFE-only_AdStock`[p >= 0.05,.N]/`MarketFE-only_AdStock`[, .N]*100,
                            `MarketFE-WOY_AdStock`[p >= 0.05,.N]/`MarketFE-WOY_AdStock`[, .N]*100,
                            `MarketFE+WOY+National_AdStock`[p >= 0.05,.N]/`MarketFE+WOY+National_AdStock`[, .N]*100,
                            Regular_AdStock[p >= 0.05,.N]/Regular_AdStock[, .N]*100,
                            NA,
                            Regular_Best[p >= 0.05,.N]/Regular_Best[, .N]*100,
                            Regular_ncomp_Barebone[p >= 0.05,.N]/Regular_ncomp_Barebone[, .N]*100,
                            Regular_ncomp_0[p >= 0.05,.N]/Regular_ncomp_0[, .N]*100,
                            Regular_ncomp_1[p >= 0.05,.N]/Regular_ncomp_1[, .N]*100,
                            Regular_ncomp_2[p >= 0.05,.N]/Regular_ncomp_2[, .N]*100,
                            Regular_Feature_Display[p >= 0.05,.N]/Regular_Feature_Display[, .N]*100,
                            Regular_AdStock_power[p >= 0.05,.N]/Regular_AdStock_power[, .N]*100
                            ),
                          2),
  "\\textbf{$> 0$}" = round(c(Naive_AdStock[p < 0.05 & c > 0,.N]/Naive_AdStock[, .N]*100,
                            `MarketFE-only_AdStock`[p < 0.05 & c > 0,.N]/`MarketFE-only_AdStock`[, .N]*100,
                            `MarketFE-WOY_AdStock`[p < 0.05 & c > 0,.N]/`MarketFE-WOY_AdStock`[, .N]*100,
                            `MarketFE+WOY+National_AdStock`[p < 0.05 & c > 0,.N]/`MarketFE+WOY+National_AdStock`[, .N]*100,
                            Regular_AdStock[p < 0.05 & c > 0,.N]/Regular_AdStock[, .N]*100,
                            NA,
                            Regular_Best[p < 0.05 & c > 0,.N]/Regular_Best[, .N]*100,
                            Regular_ncomp_Barebone[p < 0.05 & c > 0,.N]/Regular_ncomp_Barebone[, .N]*100,
                            Regular_ncomp_0[p < 0.05 & c > 0,.N]/Regular_ncomp_0[, .N]*100,
                            Regular_ncomp_1[p < 0.05 & c > 0,.N]/Regular_ncomp_1[, .N]*100,
                            Regular_ncomp_2[p < 0.05 & c > 0,.N]/Regular_ncomp_2[, .N]*100,
                            Regular_Feature_Display[p < 0.05 & c > 0,.N]/Regular_Feature_Display[, .N]*100,
                            Regular_AdStock_power[p < 0.05 & c > 0,.N]/Regular_AdStock_power[, .N]*100
                            ),
                          2),
  "\\textbf{$\\le 0$}"= round(c(Naive_AdStock[p < 0.05 & c <= 0,.N]/Naive_AdStock[, .N]*100,
                            `MarketFE-only_AdStock`[p < 0.05 & c <= 0,.N]/`MarketFE-only_AdStock`[, .N]*100,
                            `MarketFE-WOY_AdStock`[p < 0.05 & c <= 0,.N]/`MarketFE-WOY_AdStock`[, .N]*100,
                            `MarketFE+WOY+National_AdStock`[p < 0.05 & c <= 0,.N]/`MarketFE+WOY+National_AdStock`[, .N]*100,
                            Regular_AdStock[p < 0.05 & c <= 0,.N]/Regular_AdStock[, .N]*100,
                            NA,
                            Regular_Best[p < 0.05 & c <= 0,.N]/Regular_Best[, .N]*100,
                            Regular_ncomp_Barebone[p < 0.05 & c <= 0,.N]/Regular_ncomp_Barebone[, .N]*100,
                            Regular_ncomp_0[p < 0.05 & c <= 0,.N]/Regular_ncomp_0[, .N]*100,
                            Regular_ncomp_1[p < 0.05 & c <= 0,.N]/Regular_ncomp_1[, .N]*100,
                            Regular_ncomp_2[p < 0.05 & c <= 0,.N]/Regular_ncomp_2[, .N]*100,
                            Regular_Feature_Display[p < 0.05 & c <= 0,.N]/Regular_Feature_Display[, .N]*100,
                            Regular_AdStock_power[p < 0.05 & c <= 0,.N]/Regular_AdStock_power[, .N]*100
                            ),
                          2),
  "\\textbf{10\\%}" = round(c(quantile(Naive_AdStock$c, probs = 0.1),
                              quantile(`MarketFE-only_AdStock`$c, probs = 0.1),
                              quantile(`MarketFE-WOY_AdStock`$c, probs = 0.1),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.1),
                              quantile(Regular_AdStock$c, probs = 0.1),
                              quantile(summary.Elas.DT[Value < 10, Value], na.rm = T, probs = 0.1),
                              quantile(Regular_Best$c, probs = 0.1),
                              quantile(Regular_ncomp_Barebone$c, probs = 0.1),
                              quantile(Regular_ncomp_0$c, probs = 0.1),
                              quantile(Regular_ncomp_1$c, probs = 0.1),
                              quantile(Regular_ncomp_2$c, probs = 0.1),
                              quantile(Regular_Feature_Display$c, probs = 0.1),
                              quantile(Regular_AdStock_power$c, probs = 0.1)
                              ), 4),
  "\\textbf{25\\%}" = round(c(quantile(Naive_AdStock$c, probs = 0.25),
                              quantile(`MarketFE-only_AdStock`$c, probs = 0.25),
                              quantile(`MarketFE-WOY_AdStock`$c, probs = 0.25),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.25),
                              quantile(Regular_AdStock$c, probs = 0.25),
                              quantile(summary.Elas.DT[Value < 10, Value], na.rm = T, probs = 0.25),
                              quantile(Regular_Best$c, probs = 0.25),
                              quantile(Regular_ncomp_Barebone$c, probs = 0.25),
                              quantile(Regular_ncomp_0$c, probs = 0.25),
                              quantile(Regular_ncomp_1$c, probs = 0.25),
                              quantile(Regular_ncomp_2$c, probs = 0.25),
                              quantile(Regular_Feature_Display$c, probs = 0.25),
                              quantile(Regular_AdStock_power$c, probs = 0.25)
                              ), 4),
  "\\textbf{75\\%}" = round(c(quantile(Naive_AdStock$c, probs = 0.75),
                              quantile(`MarketFE-only_AdStock`$c, probs = 0.75),
                              quantile(`MarketFE-WOY_AdStock`$c, probs = 0.75),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.75),
                              quantile(Regular_AdStock$c, probs = 0.75),
                              quantile(summary.Elas.DT[Value < 10, Value], na.rm = T, probs = 0.75),
                              quantile(Regular_Best$c, probs = 0.75),
                              quantile(Regular_ncomp_Barebone$c, probs = 0.75),
                              quantile(Regular_ncomp_0$c, probs = 0.75),
                              quantile(Regular_ncomp_1$c, probs = 0.75),
                              quantile(Regular_ncomp_2$c, probs = 0.75),
                              quantile(Regular_Feature_Display$c, probs = 0.75),
                              quantile(Regular_AdStock_power$c, probs = 0.75)
                              ), 4),
  "\\textbf{90\\%}" = round(c(quantile(Naive_AdStock$c, probs = 0.9),
                              quantile(`MarketFE-only_AdStock`$c, probs = 0.9),
                              quantile(`MarketFE-WOY_AdStock`$c, probs = 0.9),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.9),
                              quantile(Regular_AdStock$c, probs = 0.9),
                              quantile(summary.Elas.DT[Value < 10, Value], na.rm = T, probs = 0.9),
                              quantile(Regular_Best$c, probs = 0.9),
                              quantile(Regular_ncomp_Barebone$c, probs = 0.9),
                              quantile(Regular_ncomp_0$c, probs = 0.9),
                              quantile(Regular_ncomp_1$c, probs = 0.9),
                              quantile(Regular_ncomp_2$c, probs = 0.9),
                              quantile(Regular_Feature_Display$c, probs = 0.9),
                              quantile(Regular_AdStock_power$c, probs = 0.9)
                              ), 4)
)

table1 <- kable(table1_DT,
      format = "latex",
      booktabs = T,
      align = "llccccccccc",
      linesep = "",
      escape  = FALSE) %>%
  kable_styling(font_size = 9) %>%
  add_header_above(c(" "     = 2,
                     "Median"= 1,
                     "Mean"  = 1,
                     "% p >= 0.05" = 1,
                     "% p < 0.05"  = 2,
                     "Percentiles" = 4),
                   bold = T) %>%
  column_spec(1, width = "0.1in") %>%
  column_spec(2, width = "3.7cm") %>%
  column_spec(3:4, width = "1.2cm") %>%
  column_spec(5, width = "1.6cm") %>%
  column_spec(6:11, width = "1.2cm") %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  footnote(general = paste0("Descriptive statistics of estimated advertising ",
  "elasticities reported for 288 brands. ",
  "The naive model includes own and competitor advertising stocks ",
  "and prices but no additional controls. ",
  "The baseline model includes store, week-of-year (season), ",
  "and month fixed effects. ",
  "All robustness results are obtained using the baseline strategy. ",
  "Standard errors are two-way clustered at the DMA and week level ",
  "in the naive and baseline specifications."),
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table1, 
                    "../Tables/tab2.txt",
                    columnwidth = FALSE,
                    change_align = TRUE,
                    adjustbox = TRUE)

