# --------------------------------- #
# Table for main results
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

source("./Gen-Table-Function.R")
load("../Data/coef_DT.RData")

Get.Coef.DT <- function() {
  load("../Data/shiny-data-0910.RData")
  data <- coef_output[level      == "Store" 
                      & strategy == "Regular" 
                      & feature  == "No"
                      & cluster  == "dma_code+week_end"]
  data <- data[time       == "Month" 
               & measure  == "GRP" 
               & goodwill == "logsum"]
  
  data <- data[variable       == "13" 
               & delta_status == "0.90"]
  
  data <- data[, .(new_module_code, new_brand_code_uc, revenue_RMS, decay, ncomp, nobs, c, s, p)]
  
  obs <- master_obs[level      == "Store"
                    & strategy == "Regular" 
                    & feature  == "No"
                    & cluster  == "dma_code+week_end"
                    & time       == "Month" 
                    & goodwill == "logsum"]
  
  retain <- 0.8
  
  cases <- copy(obs)
  cases[, nc := as.integer(NA)]
  cases[r1 < retain, nc := 0]
  cases[r1 >= retain & r2 < retain, nc := 1]
  cases[r2 >= retain & r3 < retain, nc := 2]
  cases[r3 >= retain, nc := 3]
  cases <- cases[, .(new_module_code, new_brand_code_uc, nc)]
  
  coef.DT <- merge(data, cases, by=c("new_module_code", "new_brand_code_uc"))
  coef.DT <- coef.DT[ncomp == nc]
  coef.DT <- coef.DT[, .(new_module_code, new_brand_code_uc, revenue_RMS, decay, ncomp, nobs, c, s, p)]
  setorder(coef.DT, -revenue_RMS)
  names(coef.DT) <- c("new_module_code", "new_brand_code_uc", 
                      "Revenue", "Delta", "Comp", "Obs", 
                      "coef", "se", "pval")
  
  save(coef.DT,
       file = "../input/coef_DT.RData")
}

load("../../8-ROI-Analyses/Overall/Processed-Data/base-avg-ROI-overall-observed.RData")
load("../../8-ROI-Analyses/Overall/Processed-Data/base-avg-overall-std-err-observed.RData")

overall_roi <- merge(avg_overall_observed[, Margin := as.character(Margin)],
                     avg_overall_std_err[, Margin := as.character(Margin)],
                     by = c("Module", "Brand", "Margin", "Cost", "Cost.value"))

overall_roi[, `:=` (ROI    = ROI * 100,
                    ROI_se = ROI_se * 100)]

overall_roi[, CI_lower  := ROI - 1.96*ROI_se]
overall_roi[, CI_higher := ROI + 1.96*ROI_se]

overall_roi <- overall_roi[Cost == 1]

load("../../8-ROI-Analyses/Weekly/Processed-Data/base-average-ROI-observed.RData")
load("../../8-ROI-Analyses/Weekly/Processed-Data/base-average-std-err-observed.RData")

mean_avg_observed <- average_observed[, .(ROI = mean(ROI)),
                                      by = .(Module, Brand, Margin, Cost)]
mean_avg_std_err  <- average_std_err_observed[, .(ROI_se = mean(ROI_se)),
                                              by = .(Module, Brand, Margin, Cost, Beta, Beta_se)]


average_roi <- merge(mean_avg_observed[, Margin := as.character(Margin)],
                     mean_avg_std_err[, Margin := as.character(Margin)],
                     by = c("Module", "Brand", "Margin", "Cost"))

average_roi[, `:=` (ROI    = ROI * 100,
                    ROI_se = ROI_se * 100)]

average_roi[, CI_lower  := ROI - 1.96*ROI_se]
average_roi[, CI_higher := ROI + 1.96*ROI_se]

average_roi <- average_roi[Cost == 1]

table1_DT <- data.table(
  " " = c(rep("Average ROI of Weekly Advertising",
                          3),
                      rep("ROI of All Observed Advertising", 3)),
  "     " = c("20\\% Margin",
              "30\\% Margin",
              "40\\% Margin",
              "20\\% Margin",
              "30\\% Margin",
              "40\\% Margin"),
  "    " = round(c(quantile(average_roi[Margin == "0.2", ROI], probs = 0.5, na.rm = T),
                   quantile(average_roi[Margin == "0.3", ROI], probs = 0.5, na.rm = T),
                   quantile(average_roi[Margin == "0.4", ROI], probs = 0.5, na.rm = T),
                   quantile(overall_roi[Margin == "0.2", ROI], probs = 0.5, na.rm = T),
                   quantile(overall_roi[Margin == "0.3", ROI], probs = 0.5, na.rm = T),
                   quantile(overall_roi[Margin == "0.4", ROI], probs = 0.5, na.rm = T)), 2),
  "  "            = round(c(mean(average_roi[Margin == "0.2", ROI], na.rm = T),
                            mean(average_roi[Margin == "0.3", ROI], na.rm = T),
                            mean(average_roi[Margin == "0.4", ROI], na.rm = T),
                            mean(overall_roi[Margin == "0.2", ROI], na.rm = T),
                            mean(overall_roi[Margin == "0.3", ROI], na.rm = T),
                            mean(overall_roi[Margin == "0.4", ROI], na.rm = T)), 2),
  "   "           = round(c(average_roi[Margin == "0.2" & ROI > 0,.N]/average_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                            average_roi[Margin == "0.3" & ROI > 0,.N]/average_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                            average_roi[Margin == "0.4" & ROI > 0,.N]/average_roi[Margin == "0.4" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.2" & ROI > 0,.N]/overall_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.3" & ROI > 0,.N]/overall_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.4" & ROI > 0,.N]/overall_roi[Margin == "0.4" & !is.na(ROI), .N]*100),
                          2),
  "   "           = round(c(average_roi[Margin == "0.2" & CI_lower <= 0 & CI_higher >= 0,.N]/average_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                            average_roi[Margin == "0.3" & CI_lower <= 0 & CI_higher >= 0,.N]/average_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                            average_roi[Margin == "0.4" & CI_lower <= 0 & CI_higher >= 0,.N]/average_roi[Margin == "0.4" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.2" & CI_lower <= 0 & CI_higher >= 0,.N]/overall_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.3" & CI_lower <= 0 & CI_higher >= 0,.N]/overall_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                            overall_roi[Margin == "0.4" & CI_lower <= 0 & CI_higher >= 0,.N]/overall_roi[Margin == "0.4" & !is.na(ROI), .N]*100),
                          2),
  "\\textbf{ROI $> 0$}" = round(c(average_roi[Margin == "0.2" & CI_lower > 0,.N]/average_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                                  average_roi[Margin == "0.3" & CI_lower > 0,.N]/average_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                                  average_roi[Margin == "0.4" & CI_lower > 0,.N]/average_roi[Margin == "0.4" & !is.na(ROI), .N]*100,
                                  overall_roi[Margin == "0.2" & CI_lower > 0,.N]/overall_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                                  overall_roi[Margin == "0.3" & CI_lower > 0,.N]/overall_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                                  overall_roi[Margin == "0.4" & CI_lower > 0,.N]/overall_roi[Margin == "0.4" & !is.na(ROI), .N]*100),
                                2),
  "\\textbf{ROI $< 0$}"= round(c(average_roi[Margin == "0.2" & CI_higher < 0,.N]/average_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                                 average_roi[Margin == "0.3" & CI_higher < 0,.N]/average_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                                 average_roi[Margin == "0.4" & CI_higher < 0,.N]/average_roi[Margin == "0.4" & !is.na(ROI), .N]*100,
                                 overall_roi[Margin == "0.2" & CI_higher < 0,.N]/overall_roi[Margin == "0.2" & !is.na(ROI), .N]*100,
                                    overall_roi[Margin == "0.3" & CI_higher < 0,.N]/overall_roi[Margin == "0.3" & !is.na(ROI), .N]*100,
                                    overall_roi[Margin == "0.4" & CI_higher < 0,.N]/overall_roi[Margin == "0.4" & !is.na(ROI), .N]*100),
                                  2))

table1 <- kable(table1_DT,
                format = "latex",
                booktabs = T,
                align = "llcccccc",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 8) %>%
  add_header_above(c(" "     = 2,
                     # "No. of Brands" = 1,
                     "Median"= 1,
                     "Mean"  = 1,
                     "% ROI $> 0$" = 1,
                     "$% \\mathbf{p \\ge 0.05}$" = 1,
                     "$% \\mathbf{p < 0.05}$"  = 2),
                   bold = T) %>%
  column_spec(1, width = "0.1cm") %>%
  column_spec(2, width = "3.8cm") %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  footnote(general = paste0("The estimates are obtained using the baseline strategy and assuming a carryover parameter $\\delta = 0.9$."),
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table1, 
                    "../Tables/tab3.txt",
                    columnwidth = FALSE,
                    change_align = FALSE,
                    adjustbox = T)
