# --------------------------------- #
# Table for umbrella brand
# (Table in appendix C)
# 
# 8/9/2019
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

load("../Data/Data-used-for-draft-umbrella.RData")
load("../Data/Data-used-for-draft-top.RData")
load("../Data/Data-used-for-draft.RData")
load("../Data/Data-used-for-draft-ownprice.RData")
load("../Data/Data-used-for-draft-topprice.RData")
load("../Data/Data-used-for-draft-top2price.RData")
load("../Data/Data-used-for-draft-top3price.RData")

Regular_AdStock_24 <- Regular_AdStock_24[complete.cases(Regular_AdStock_24)]
Regular_AdStock_5 <- Regular_AdStock_5[complete.cases(Regular_AdStock_5)]
Regular_AdStock_ownprice <- Regular_AdStock_ownprice[complete.cases(Regular_AdStock_ownprice)]
Regular_AdStock_topprice <- Regular_AdStock_topprice[complete.cases(Regular_AdStock_topprice)]
Regular_AdStock_top2price <- Regular_AdStock_top2price[complete.cases(Regular_AdStock_top2price)]
Regular_AdStock_top3price <- Regular_AdStock_top3price[complete.cases(Regular_AdStock_top3price)]

rm(list = setdiff(ls(), 
           c("Regular_AdStock", "Regular_AdStock_24",
             "Regular_AdStock_5", "Regular_AdStock_ownprice",
             "Regular_AdStock_topprice",
             "Regular_AdStock_top2price", "Regular_AdStock_top3price")))

source("./Gen-Table-Function.R")

stopifnot(Regular_AdStock[!is.na(c), .N] == 288)

table_DT <- data.table(
  "blah" = rep("Baseline", each = 4),
  "     " = c("Affiliated Brand Advertising",
                  "Top Competitor Advertising",
                  "Own Price Elasticity",
                  "Top Competitor Price Elasticity"),
  " "             = round(c(quantile(Regular_AdStock_24$c, probs = 0.5),
                            quantile(Regular_AdStock_5$c, probs = 0.5),
                            quantile(Regular_AdStock_ownprice$c, probs = 0.5),
                            quantile(Regular_AdStock_topprice$c, probs = 0.5)), 4),
  "  "            = round(c(mean(Regular_AdStock_24$c),
                            mean(Regular_AdStock_5$c),
                            mean(Regular_AdStock_ownprice$c),
                            mean(Regular_AdStock_topprice$c)), 4),
  "    "          = round(c(Regular_AdStock_24[, .N]/Regular_AdStock[!is.na(c), .N]*100,
                            Regular_AdStock_5[, .N]/Regular_AdStock[!is.na(c), .N]*100,
                            Regular_AdStock_ownprice[, .N]/Regular_AdStock[!is.na(c), .N]*100,
                            Regular_AdStock_topprice[, .N]/Regular_AdStock[!is.na(c), .N]*100
                            ),
                          2),
  "   "           = round(c(Regular_AdStock_24[p >= 0.05,.N]/Regular_AdStock_24[, .N]*100,
                            Regular_AdStock_5[p >= 0.05,.N]/Regular_AdStock_5[, .N]*100,
                            Regular_AdStock_ownprice[p >= 0.05,.N]/Regular_AdStock_ownprice[, .N]*100,
                            Regular_AdStock_topprice[p >= 0.05,.N]/Regular_AdStock_topprice[, .N]*100
                            ),
                          2),
  "\\textbf{$> 0$}" = round(c(Regular_AdStock_24[p < 0.05 & c > 0,.N]/Regular_AdStock_24[, .N]*100,
                              Regular_AdStock_5[p < 0.05 & c > 0,.N]/Regular_AdStock_5[, .N]*100,
                              Regular_AdStock_ownprice[p < 0.05 & c > 0,.N]/Regular_AdStock_ownprice[, .N]*100,
                              Regular_AdStock_topprice[p < 0.05 & c > 0,.N]/Regular_AdStock_topprice[, .N]*100),
                            2),
  "\\textbf{$\\le 0$}"= round(c(Regular_AdStock_24[p < 0.05 & c <= 0,.N]/Regular_AdStock_24[, .N]*100,
                                Regular_AdStock_5[p < 0.05 & c <= 0,.N]/Regular_AdStock_5[, .N]*100,
                                Regular_AdStock_ownprice[p < 0.05 & c <= 0,.N]/Regular_AdStock_ownprice[, .N]*100,
                                Regular_AdStock_topprice[p < 0.05 & c <= 0,.N]/Regular_AdStock_topprice[, .N]*100),
                              2),
  "\\textbf{10\\%}" = round(c(quantile(Regular_AdStock_24$c, probs = 0.1),
                              quantile(Regular_AdStock_5$c, probs = 0.1),
                              quantile(Regular_AdStock_ownprice$c, probs = 0.1),
                              quantile(Regular_AdStock_topprice$c, probs = 0.1)), 3),
  "\\textbf{25\\%}" = round(c(quantile(Regular_AdStock_24$c, probs = 0.25),
                              quantile(Regular_AdStock_5$c, probs = 0.25),
                              quantile(Regular_AdStock_ownprice$c, probs = 0.25),
                              quantile(Regular_AdStock_topprice$c, probs = 0.25)), 3),
  "\\textbf{75\\%}" = round(c(quantile(Regular_AdStock_24$c, probs = 0.75),
                              quantile(Regular_AdStock_5$c, probs = 0.75),
                              quantile(Regular_AdStock_ownprice$c, probs = 0.75),
                              quantile(Regular_AdStock_topprice$c, probs = 0.75)), 3),
  "\\textbf{90\\%}" = round(c(quantile(Regular_AdStock_24$c, probs = 0.9),
                              quantile(Regular_AdStock_5$c, probs = 0.9),
                              quantile(Regular_AdStock_ownprice$c, probs = 0.9),
                              quantile(Regular_AdStock_topprice$c, probs = 0.9)), 3)
)

table <- kable(table_DT,
                format = "latex",
                booktabs = T,
                align = "llcccccccccc",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 8) %>%
  add_header_above(c(" "     = 2,
                     "Median"= 1,
                     "Mean"  = 1,
                     "% Brands"    = 1,
                     "% p >= 0.05" = 1,
                     "% p < 0.05"  = 2,
                     "Percentiles" = 4),
                   bold = T) %>%
  column_spec(1, width = "0.1in") %>%
  column_spec(2, width = "4.5cm") %>%
  column_spec(3:4, width = "1.2cm") %>%
  column_spec(5:6, width = "1.2cm") %>%
  column_spec(7:11, width = "1cm") %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  footnote(general = "The estimates are obtained using the baseline strategy and assuming a carryover parameter $\\delta=0.9$. Standard errors are two-way clustered at the DMA level and the week level.",
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table, 
                    "../Tables/tab4.txt",
                    columnwidth = TRUE,
                    adjustbox   = FALSE)
