# --------------------------------- #
# Table for various delta
# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

source("./Gen-Table-Function.R")
load("../Data/Data-used-for-table3.RData")
load("../Data/Data-used-for-draft-best.RData")

table2_DT <- data.table(
  "Specification" = rep("Baseline Specification", each = 8),
  "    " = c(rep("Specified $\\delta$", 7),
             "Estimated $\\delta$"),
  "     " = c("0.00",
              "0.25",
              "0.50",
              "0.75",
              "0.90",
              "0.95",
              "1.00",
              " "),
  " "             = round(c(quantile(Regular_delta_0.00$c, probs = 0.5),
                            quantile(Regular_delta_0.25$c, probs = 0.5),
                            quantile(Regular_delta_0.50$c, probs = 0.5),
                            quantile(Regular_delta_0.75$c, probs = 0.5),
                            quantile(Regular_delta_0.90$c, probs = 0.5),
                            quantile(Regular_delta_0.95$c, probs = 0.5),
                            quantile(Regular_delta_1.00$c, probs = 0.5),
                            quantile(Regular_Best$c, probs = 0.5)), 4),
  "  "            = round(c(mean(Regular_delta_0.00$c),
                            mean(Regular_delta_0.25$c),
                            mean(Regular_delta_0.50$c),
                            mean(Regular_delta_0.75$c),
                            mean(Regular_delta_0.90$c),
                            mean(Regular_delta_0.95$c),
                            mean(Regular_delta_1.00$c),
                            mean(Regular_Best$c)), 4),
  "   "           = round(c(Regular_delta_0.00[p >= 0.05,.N]/Regular_delta_0.00[, .N]*100,
                            Regular_delta_0.25[p >= 0.05,.N]/Regular_delta_0.25[, .N]*100,
                            Regular_delta_0.50[p >= 0.05,.N]/Regular_delta_0.50[, .N]*100,
                            Regular_delta_0.75[p >= 0.05,.N]/Regular_delta_0.75[, .N]*100,
                            Regular_delta_0.90[p >= 0.05,.N]/Regular_delta_0.90[, .N]*100,
                            Regular_delta_0.95[p >= 0.05,.N]/Regular_delta_0.95[, .N]*100,
                            Regular_delta_1.00[p >= 0.05,.N]/Regular_delta_1.00[, .N]*100,
                            Regular_Best[p >= 0.05,.N]/Regular_Best[, .N]*100), 2),
  "\\textbf{$> 0$}" = round(c(Regular_delta_0.00[p < 0.05 & c > 0,.N]/Regular_delta_0.00[, .N]*100,
                              Regular_delta_0.25[p < 0.05 & c > 0,.N]/Regular_delta_0.25[, .N]*100,
                              Regular_delta_0.50[p < 0.05 & c > 0,.N]/Regular_delta_0.50[, .N]*100,
                              Regular_delta_0.75[p < 0.05 & c > 0,.N]/Regular_delta_0.75[, .N]*100,
                              Regular_delta_0.90[p < 0.05 & c > 0,.N]/Regular_delta_0.90[, .N]*100,
                              Regular_delta_0.95[p < 0.05 & c > 0,.N]/Regular_delta_0.95[, .N]*100,
                              Regular_delta_1.00[p < 0.05 & c > 0,.N]/Regular_delta_1.00[, .N]*100,
                              Regular_Best[p < 0.05 & c > 0,.N]/Regular_Best[, .N]*100), 2),
  "\\textbf{$\\le 0$}"= round(c(Regular_delta_0.00[p < 0.05 & c <= 0,.N]/Regular_delta_0.00[, .N]*100,
                                Regular_delta_0.25[p < 0.05 & c <= 0,.N]/Regular_delta_0.25[, .N]*100,
                                Regular_delta_0.50[p < 0.05 & c <= 0,.N]/Regular_delta_0.50[, .N]*100,
                                Regular_delta_0.75[p < 0.05 & c <= 0,.N]/Regular_delta_0.75[, .N]*100,
                                Regular_delta_0.90[p < 0.05 & c <= 0,.N]/Regular_delta_0.90[, .N]*100,
                                Regular_delta_0.95[p < 0.05 & c <= 0,.N]/Regular_delta_0.95[, .N]*100,
                                Regular_delta_1.00[p < 0.05 & c <= 0,.N]/Regular_delta_1.00[, .N]*100,
                                Regular_Best[p < 0.05 & c <= 0,.N]/Regular_Best[, .N]*100), 2),
  "\\textbf{10\\%}" = round(c(quantile(Regular_delta_0.00$c, probs = 0.1),
                              quantile(Regular_delta_0.25$c, probs = 0.1),
                              quantile(Regular_delta_0.50$c, probs = 0.1),
                              quantile(Regular_delta_0.75$c, probs = 0.1),
                              quantile(Regular_delta_0.90$c, probs = 0.1),
                              quantile(Regular_delta_0.95$c, probs = 0.1),
                              quantile(Regular_delta_1.00$c, probs = 0.1),
                              quantile(Regular_Best$c, probs = 0.1)), 4),
  "\\textbf{25\\%}" = round(c(quantile(Regular_delta_0.00$c, probs = 0.25),
                              quantile(Regular_delta_0.25$c, probs = 0.25),
                              quantile(Regular_delta_0.50$c, probs = 0.25),
                              quantile(Regular_delta_0.75$c, probs = 0.25),
                              quantile(Regular_delta_0.90$c, probs = 0.25),
                              quantile(Regular_delta_0.95$c, probs = 0.25),
                              quantile(Regular_delta_1.00$c, probs = 0.25),
                              quantile(Regular_Best$c, probs = 0.25)), 4),
  "\\textbf{75\\%}" = round(c(quantile(Regular_delta_0.00$c, probs = 0.75),
                              quantile(Regular_delta_0.25$c, probs = 0.75),
                              quantile(Regular_delta_0.50$c, probs = 0.75),
                              quantile(Regular_delta_0.75$c, probs = 0.75),
                              quantile(Regular_delta_0.90$c, probs = 0.75),
                              quantile(Regular_delta_0.95$c, probs = 0.75),
                              quantile(Regular_delta_1.00$c, probs = 0.75),
                              quantile(Regular_Best$c, probs = 0.75)), 4),
  "\\textbf{90\\%}" = round(c(quantile(Regular_delta_0.00$c, probs = 0.9),
                              quantile(Regular_delta_0.25$c, probs = 0.9),
                              quantile(Regular_delta_0.50$c, probs = 0.9),
                              quantile(Regular_delta_0.75$c, probs = 0.9),
                              quantile(Regular_delta_0.90$c, probs = 0.9),
                              quantile(Regular_delta_0.95$c, probs = 0.9),
                              quantile(Regular_delta_1.00$c, probs = 0.9),
                              quantile(Regular_Best$c, probs = 0.9)), 4)
)

table2 <- kable(table2_DT,
                format = "latex",
                booktabs = T,
                align = "c",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 9) %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  add_header_above(c(" "     = 3,
                     "Median"= 1,
                     "Mean"  = 1,
                     "% p >= 0.05" = 1,
                     "% p < 0.05"  = 2,
                     "Percentiles" = 4),
                   bold = T) %>%
  footnote(general = paste0("Descriptive statistics of estimated advertising elasticities ",
                            "reported for two model specifications and 288 brands. ",
                            "Elasticities derived from regressions of log quantity ",
                            "on log advertising GRP stock (own and competitor) ",
                            "and log prices (own and competitor). ",
                            "The baseline model includes store, month, and week-of-year fixed effects. ",
                            "Regressions are estimated separately for each brand. ",
                            "The unit of observation in each regression model is a store-brand-week. ",
                            "Standard errors are two-way clustered at the DMA level ",
                            "and the week level."),
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))

PrintStandAloneText(table2, 
                    "../Tables/tab5.txt",
                    columnwidth = FALSE,
                    adjustbox   = FALSE)
