
rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

source("./Gen-Table-Function.R")
load("../Data/Data-used-for-draft.RData")
load("../Data/measurement-error-data.RData")
load("../Data/Lagged-Adstock.RData")
load("../Data/base-lag.RData")

table11_DT <- data.table(
  "Specification" = c("Baseline Specification",
                      "Baseline Specification",
                      "Baseline Specification",
                      "Baseline Specification"
  ),
  "     " = c("GRP, All markets, Time FE",
              "Occurrence",
              "LPM markets",
              "Time Trend"
  ),
  " "             = round(c(quantile(Regular_AdStock$c, probs = 0.5),
                            quantile(Regular.Duration.0.9$coef, probs = 0.5),
                            quantile(Regular.LPM.0.9$coef, probs = 0.5),
                            quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.5)), 4),
  "  "            = round(c(mean(Regular_AdStock$c),
                            mean(Regular.Duration.0.9$coef),
                            mean(Regular.LPM.0.9$coef),
                            mean(`MarketFE+WOY+National_AdStock`$c)), 4),
  "   "           = round(c(Regular_AdStock[p >= 0.05, .N] / Regular_AdStock[, .N] * 100,
                            Regular.Duration.0.9[pval >= 0.05, .N] / Regular.Duration.0.9[, .N] * 100,
                            Regular.LPM.0.9[pval >= 0.05, .N] / Regular.LPM.0.9[, .N] * 100,
                            `MarketFE+WOY+National_AdStock`[p >= 0.05, .N] / `MarketFE+WOY+National_AdStock`[, .N] * 100),
                          2),
  "\\textbf{$> 0$}" = round(c(Regular_AdStock[p < 0.05 & c > 0, .N] / Regular_AdStock[, .N] * 100,
                              Regular.Duration.0.9[pval < 0.05 & coef > 0, .N] / Regular.Duration.0.9[, .N] * 100,
                              Regular.LPM.0.9[pval < 0.05 & coef > 0, .N] / Regular.LPM.0.9[, .N] * 100,
                              `MarketFE+WOY+National_AdStock`[p < 0.05 & c > 0, .N] / `MarketFE+WOY+National_AdStock`[, .N] * 100),
                            2),
  "\\textbf{$\\le 0$}"= round(c(Regular_AdStock[p < 0.05 & c <= 0, .N] / Regular_AdStock[, .N] * 100,
                                Regular.Duration.0.9[pval < 0.05 & coef <= 0, .N] / Regular.Duration.0.9[, .N] * 100,
                                Regular.LPM.0.9[pval < 0.05 & coef <= 0, .N] / Regular.LPM.0.9[, .N] * 100,
                                `MarketFE+WOY+National_AdStock`[p < 0.05 & c <= 0, .N] / `MarketFE+WOY+National_AdStock`[, .N] * 100),
                              2),
  "\\textbf{10\\%}" = round(c(quantile(Regular_AdStock$c, probs = 0.1),
                              quantile(Regular.Duration.0.9$coef, probs = 0.1),
                              quantile(Regular.LPM.0.9$coef, probs = 0.1),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.1)), 4),
  "\\textbf{25\\%}" = round(c(quantile(Regular_AdStock$c, probs = 0.25),
                              quantile(Regular.Duration.0.9$coef, probs = 0.25),
                              quantile(Regular.LPM.0.9$coef, probs = 0.25),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.25)), 4),
  "\\textbf{75\\%}" = round(c(quantile(Regular_AdStock$c, probs = 0.75),
                              quantile(Regular.Duration.0.9$coef, probs = 0.75),
                              quantile(Regular.LPM.0.9$coef, probs = 0.75),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.75)), 4),
  "\\textbf{90\\%}" = round(c(quantile(Regular_AdStock$c, probs = 0.9),
                              quantile(Regular.Duration.0.9$coef, probs = 0.9),
                              quantile(Regular.LPM.0.9$coef, probs = 0.9),
                              quantile(`MarketFE+WOY+National_AdStock`$c, probs = 0.9)), 4)
)

table11 <- kable(table11_DT,
                format = "latex",
                booktabs = T,
                align = "llccccccccc",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 9) %>%
  add_header_above(c(" "     = 2,
                     "Median"= 1,
                     "Mean"  = 1,
                     "% p >= 0.05" = 1,
                     "% p < 0.05"  = 2,
                     "Percentiles" = 4),
                   bold = T) %>%
  column_spec(1, width = "0.1in") %>%
  column_spec(2, width = "3.7cm") %>%
  column_spec(3:4, width = "1.2cm") %>%
  column_spec(5, width = "1.6cm") %>%
  column_spec(6:11, width = "1.2cm") %>%
  collapse_rows(columns = 1:2, 
                row_group_label_position = 'stack',
                latex_hline = 'major') %>%
  footnote(general = paste0("Elasticities derived from regressions of log quantity ",
                            "on log advertising stock (own and competitor) and log prices (own and competitor). ",
                            "All use modifications to the baseline strategy as described. "),
           threeparttable = T,
           fixed_small_size = TRUE,
           title_format = c("bold"))


PrintStandAloneText(table11, 
                    "../Tables/tab6.txt",
                    columnwidth = FALSE,
                    change_align = TRUE)

