


rm(list = ls())
library(data.table)
library(bit64)
library(ggplot2)
library(knitr)
library(kableExtra)
library(lubridate)
library(Hmisc)

source("./Gen-Table-Function.R")

load("../Data/ad-price-correlation.RData")

single_vars <- c("price_0", "promo_0", "feature_0", "display_0")

DT <- proc[RHS %in% single_vars]
DT[, significant := (p <= 0.05)]
DT[, pos_sig     := (p <= 0.05 & c > 0)]
DT[, nonpos_sig  := (p <= 0.05 & c <= 0)]

multi_vars <- setdiff(unique(proc$RHS), single_vars)

DT1 <- proc[RHS %in% multi_vars]
DT1[, sig := (p <= 0.05)]

# Single Var

table.DT <- data.table()

for (var in single_vars) {
   plot.DT <- DT[RHS == var & variable == var & decay == 0]

   plot.DT[Strategy == "Border", Strategy := "Border Strategy"]
   plot.DT[Strategy == "Regular", Strategy := "Baseline"]

   plot.DT[FixedEffect == T, FixedEffect_name := "With Time+Market+Seasonal FEs"]
   plot.DT[FixedEffect == F, FixedEffect_name := "Without FEs"]

   plot.DT[, group_count := .N, by = .(Strategy, FixedEffect)]
   plot.DT[, freq        := 1 / group_count]

   sub.table.DT <- data.table(
      "strategy" = rep("Baseline", each = 2),
      "fixedeffect" = c("without FEs", "with FEs"),
      "median" = round(c(quantile(plot.DT[Strategy == "Baseline" 
                                       & FixedEffect == FALSE, c], probs = 0.5, na.rm = T),
                      quantile(plot.DT[Strategy == "Baseline" 
                                       & FixedEffect == TRUE, c], probs = 0.5, na.rm = T)), 4),
      "mean" = round(c(mean(plot.DT[Strategy == "Baseline" 
                                    & FixedEffect == FALSE, c], na.rm = T),
                       mean(plot.DT[Strategy == "Baseline" 
                                    & FixedEffect == TRUE, c], na.rm = T)), 4),
      "nonsig" = round(c(plot.DT[Strategy == "Baseline" 
                                & FixedEffect == FALSE
                                & significant == F, .N] / plot.DT[Strategy == "Baseline" 
                                                                  & FixedEffect == FALSE, .N] * 100,
                        plot.DT[Strategy == "Baseline" 
                                & FixedEffect == T
                                & significant == F, .N] / plot.DT[Strategy == "Baseline" 
                                                                  & FixedEffect == T, .N] * 100), 2),
      "possig" = round(c(plot.DT[Strategy == "Baseline" 
                                          & FixedEffect == FALSE
                                          & pos_sig == T, .N] / plot.DT[Strategy == "Baseline" 
                                                                        & FixedEffect == FALSE, .N] * 100,
                                  plot.DT[Strategy == "Baseline" 
                                          & FixedEffect == T
                                          & pos_sig == T, .N] / plot.DT[Strategy == "Baseline" 
                                                                        & FixedEffect == T, .N] * 100), 2),
      "nonpossig" = round(c(plot.DT[Strategy == "Baseline" 
                                             & FixedEffect == FALSE
                                             & nonpos_sig == T, .N] / plot.DT[Strategy == "Baseline" 
                                                                              & FixedEffect == FALSE, .N] * 100,
                                     plot.DT[Strategy == "Baseline" 
                                             & FixedEffect == T
                                             & nonpos_sig == T, .N] / plot.DT[Strategy == "Baseline" 
                                                                              & FixedEffect == T, .N] * 100), 2),
      "q10" = round(c(quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == FALSE, c], probs = 0.1, na.rm = T),
                                  quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == TRUE, c], probs = 0.1, na.rm = T)), 4),
      "q25" = round(c(quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == FALSE, c], probs = 0.25, na.rm = T),
                                  quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == TRUE, c], probs = 0.25, na.rm = T)), 4),
      "q75" = round(c(quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == FALSE, c], probs = 0.75, na.rm = T),
                                  quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == TRUE, c], probs = 0.75, na.rm = T)), 4),
      "q90" = round(c(quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == FALSE, c], probs = 0.9, na.rm = T),
                                  quantile(plot.DT[Strategy == "Baseline" 
                                                   & FixedEffect == TRUE, c], probs = 0.9, na.rm = T)), 4)
   )
   
   sub.table.DT[, price_var := gsub("_0","",var)]
   
   table.DT <- rbind(table.DT, sub.table.DT)
}

setcolorder(table.DT, c("strategy", "price_var"))

table.DT[, strategy := factor(strategy, levels = c("Baseline"))]
table.DT[, price_var := factor(price_var, levels = c("price", "promo", "feature", "display"))]
table.DT[, fixedeffect := factor(fixedeffect, levels = c("without FEs", "with FEs"))]

setkey(table.DT, strategy, price_var, fixedeffect)

names(table.DT) <- c(" ", "  ", "   ", "    ", "     ",
                     "      ", "\\textbf{$> 0$}", "\\textbf{$\\le 0$}",
                     "\\textbf{10\\%}","\\textbf{25\\%}","\\textbf{75\\%}","\\textbf{90\\%}")

table <- kable(table.DT,
                format = "latex",
                booktabs = T,
                align = "lllccccccccc",
                linesep = "",
                escape  = FALSE) %>%
   kable_styling(font_size = 9) %>%
   add_header_above(c(" "     = 3,
                      "Median"= 1,
                      "Mean"  = 1,
                      "% p >= 0.05" = 1,
                      "% p < 0.05"  = 2,
                      "Percentiles" = 4),
                    bold = T) %>%
   column_spec(4:5, width = "1.2cm") %>%
   column_spec(6, width = "1.6cm") %>%
   column_spec(7:12, width = "1.2cm") %>%
   collapse_rows(columns = 1:3, 
                 row_group_label_position = 'stack',
                 latex_hline = 'major',
                 valign = 'middle')

PrintStandAloneText(table, 
                    "../Tables/tab7.txt",
                    columnwidth = T,
                    change_align = F,
                    adjustbox = F,
                    remove_linespace = T)

