# --------------------------------- #

# --------------------------------- #

rm(list = ls())
library(data.table)
library(bit64)
library(lubridate)
library(Hmisc)
library(tidyverse)
library(kableExtra)

source("./Gen-Table-Function.R")
load("../Data/grp_by_tier.RData")

tier_1_grp <- grp_by_tier[match_tier==1]
tier_3_grp <- grp_by_tier[match_tier==3]

table_DT <- data.table(
  " " = c("Exact Matches",
          "Inexact Matches"),
  " "             = round(c(quantile(tier_1_grp$proportion, probs = 0.5),
                            quantile(tier_3_grp$proportion, probs = 0.5)), 4),
  "  "            = round(c(mean(tier_1_grp$proportion),
                            mean(tier_3_grp$proportion)), 4),
  "\\textbf{10\\%}" = round(c(quantile(tier_1_grp$proportion, probs = 0.1),
                              quantile(tier_3_grp$proportion, probs = 0.1)), 4),
  "\\textbf{25\\%}" = round(c(quantile(tier_1_grp$proportion, probs = 0.25),
                              quantile(tier_3_grp$proportion, probs = 0.25)), 4),
  "\\textbf{75\\%}" = round(c(quantile(tier_1_grp$proportion, probs = 0.75),
                              quantile(tier_3_grp$proportion, probs = 0.75)), 4),
  "\\textbf{90\\%}" = round(c(quantile(tier_1_grp$proportion, probs = 0.9),
                              quantile(tier_3_grp$proportion, probs = 0.9)), 4)
)

table <- kable(table_DT,
                format = "latex",
                booktabs = T,
                align = "lcccccc",
                linesep = "",
                escape  = FALSE) %>%
  kable_styling(font_size = 8.5) %>%
  add_header_above(c(" "     = 1,
                     "Median"= 1,
                     "Mean"  = 1,
                     "Percentiles" = 4),
                   bold = T) %>%
  column_spec(1, width = "1in") %>%
  column_spec(2:3, width = "1.2cm") %>%
  column_spec(4:7, width = "1.2cm")

PrintStandAloneText(table, 
                    "../Tables/tab9.txt",
                    columnwidth = TRUE,
                    adjustbox   = FALSE)
